@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -23,7 +23,6 @@
 
 #include <fstream>
 #include <sstream>
-#include <iostream>
 #include <map>
 #include <boost/format.hpp>
 #include <boost/foreach.hpp>
@@ -61,10 +60,15 @@
 #   define _(String) String
 #endif
 
+#define MAX_STRING_LENGTH 200
+
 // Useful shortcut
 #define fmt(a) boost::format(a)
 
 
+INIT_LOGGER(dic, CompDic);
+
+
 CompDic::CompDic()
     : m_currentRec(0), m_maxRec(0), m_loadTime(0), m_buildTime(0)
 {
@@ -74,6 +78,15 @@ CompDic::CompDic()
     m_headerInfo.edgesused  = 1;
     m_headerInfo.nodessaved = 0;
     m_headerInfo.edgessaved = 0;
+
+    m_stringBuf = new wchar_t[MAX_STRING_LENGTH];
+    m_endString = m_stringBuf;
+}
+
+
+CompDic::~CompDic()
+{
+    delete[] m_stringBuf;
 }
 
 
@@ -89,7 +102,7 @@ void CompDic::addLetter(wchar_t chr, int points, int frequency,
     if (!iswalpha(chr) && chr != L'?')
     {
         ostringstream ss;
-        ss << fmt(_("'%1%' is not a valid letter.")) % convertToMb(chr) << endl;
+        ss << fmt(_("'%1%' is not a valid letter.")) % lfw(chr) << endl;
         ss << fmt(_("For technical reasons, Eliot currently only supports "
                     "alphabetical characters as internal character "
                     "representation, even if the tile has a display string "
@@ -151,8 +164,13 @@ void CompDic::loadWordList(const string &iFileName, vector<wstring> &oWordList)
         {
             line = line.substr(3);
         }
-        oWordList.push_back(readFromUTF8(line.data(),
-                                        line.size(), "loadWordList"));
+        // Remove potential \r
+        if (line[line.size() - 1] == '\r')
+            line = line.substr(0, line.size() - 1);
+        // Ignore empty lines
+        if (line == "")
+            continue;
+        oWordList.push_back(readFromUTF8(line, "loadWordList"));
     }
 
     // Sort the word list, to perform a better compression
@@ -170,23 +188,17 @@ Header CompDic::writeHeader(ostream &outFile) const
 }
 
 
-void CompDic::writeNode(uint32_t *ioEdges, unsigned int num, ostream &outFile)
+void CompDic::writeNode(DicEdge *ioEdges, unsigned int num, ostream &outFile)
 {
+    uint32_t *edgesAsUint = reinterpret_cast<uint32_t*>(ioEdges);
     // Handle endianness
     for (unsigned int i = 0; i < num; ++i)
     {
-        ioEdges[i] = htonl(ioEdges[i]);
+        edgesAsUint[i] = htonl(edgesAsUint[i]);
     }
 
-#ifdef DEBUG_OUTPUT
-    cout << fmt(_("writing %1% edges")) % num << endl;
-    for (int i = 0; i < num; i++)
-    {
-        outFile.write((char*)(ioEdges + i), sizeof(DicEdge));
-    }
-#else
-    outFile.write((char*)ioEdges, num * sizeof(DicEdge));
-#endif
+    LOG_TRACE(fmt("writing %1% edges") % num);
+    outFile.write((char*)edgesAsUint, num * sizeof(DicEdge));
 }
 
 #define MAX_EDGES 2000
@@ -313,8 +325,7 @@ unsigned int CompDic::makeNode(ostream &outFile, const Header &iHeader,
         m_hashMap[edges] = m_headerInfo.edgesused;
         m_headerInfo.edgesused += numedges;
         m_headerInfo.nodesused++;
-        writeNode(reinterpret_cast<uint32_t*>(&edges.front()),
-                   numedges, outFile);
+        writeNode(&edges.front(), numedges, outFile);
 
         return node_pos;
     }
@@ -325,7 +336,7 @@ Header CompDic::generateDawg(const string &iWordListFile,
                              const string &iDawgFile,
                              const string &iDicName)
 {
-    m_headerInfo.dicName = convertToWc(iDicName);
+    m_headerInfo.dicName = wfl(iDicName);
     // We are not (yet) able to build the GADDAG format
     m_headerInfo.dawg = true;
 
@@ -356,8 +367,7 @@ Header CompDic::generateDawg(const string &iWordListFile,
     specialNode.last = 1;
     // Temporary variable to avoid a warning when compiling with -O2
     // (there is no warning with -O0... g++ bug?)
-    DicEdge *tmpPtr = &specialNode;
-    writeNode(reinterpret_cast<uint32_t*>(tmpPtr), 1, outFile);
+    writeNode(&specialNode, 1, outFile);
 
     vector<wstring>::const_iterator firstWord = wordList.begin();
     wstring::const_iterator initialPos = firstWord->begin();
@@ -371,8 +381,7 @@ Header CompDic::generateDawg(const string &iWordListFile,
                             firstWord, wordList.end(),
                             initialPos, m_endString);
     // Reuse the temporary variable
-    tmpPtr = &rootNode;
-    writeNode(reinterpret_cast<uint32_t*>(tmpPtr), 1, outFile);
+    writeNode(&rootNode, 1, outFile);
     const clock_t endBuildTime = clock();
     m_buildTime = 1.0 * (endBuildTime - startBuildTime) / CLOCKS_PER_SEC;
 
