@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2002-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2002-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -48,6 +48,9 @@
 #include "tile.h"
 
 
+INIT_LOGGER(dic, Dictionary);
+
+
 const Dictionary *Dictionary::m_dic = NULL;
 
 
@@ -137,13 +140,14 @@ void Dictionary::initializeTiles()
     // "Activate" the dictionary by giving the header to the Tile class
     Tile::SetHeader(*m_header);
 
-    // XXX: temp
-    Tile::m_TheJoker = Tile(Tile::kTILE_JOKER);
-
     m_tilesVect.reserve(m_header->getLetters().size() + 1);
     // Create a tile for each letter in the dictionary header
     for (unsigned int i = 0; i < m_header->getLetters().size(); ++i)
-        m_tilesVect.push_back(Tile(m_header->getLetters()[i]));
+    {
+        wchar_t chr = m_header->getLetters()[i];
+        unsigned int code = m_header->getCodeFromChar(chr);
+        m_tilesVect.push_back(Tile(code, chr == Tile::kTILE_JOKER));
+    }
 }
 
 
