@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2002-2009 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2002-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -19,14 +19,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#ifndef _DIC_H_
-#define _DIC_H_
+#ifndef DIC_H_
+#define DIC_H_
 
+#include <stdint.h>
 #include <string>
 #include <vector>
 #include <map>
 
 #include "tile.h"
+#include "logging.h"
 
 using namespace std;
 
@@ -58,6 +60,7 @@ typedef wstring wistring;
 
 class Dictionary
 {
+    DEFINE_LOGGER();
 public:
     /**
      * Dictionary creation and loading from a file
