@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2007-2009 Olivier Teulière
+ * Copyright (C) 2007-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -56,6 +56,9 @@ using boost::format;
 using boost::wformat;
 
 
+INIT_LOGGER(dic, Header);
+
+
 #if defined(WORDS_BIGENDIAN)
 // Nothing to do on big-endian machines
 #   define ntohll(x) (x)
@@ -110,6 +113,12 @@ struct Dict_header_old
 /** Extension of the old format (used in version 1) */
 struct Dict_header_ext
 {
+    Dict_header_ext()
+    {
+        // Make sure we won't write uninitialized bytes
+        memset(this, 0, sizeof(*this));
+    }
+
     // Time when the dictionary was compressed
     // (number of seconds since the Epoch)
     uint64_t compressDate;
@@ -163,6 +172,12 @@ struct Dict_header_ext
 /** Extension of the extension :-) (used in version 2) */
 struct Dict_header_ext_2
 {
+    Dict_header_ext_2()
+    {
+        // Make sure we won't write uninitialized bytes
+        memset(this, 0, sizeof(*this));
+    }
+
     // Additional information concerning the display strings and the
     // alternative input strings of the letters
     char displayAndInput[_MAX_DISPLAY_INPUT_SIZE_];
@@ -220,7 +235,7 @@ Header::Header(const DictHeaderInfo &iInfo)
     }
 
     m_compressDate = time(NULL);
-    m_userHost = convertToWc(ELIOT_COMPILE_BY + string("@") + ELIOT_COMPILE_HOST);
+    m_userHost = wfl(ELIOT_COMPILE_BY + string("@") + ELIOT_COMPILE_HOST);
     m_root = iInfo.root;
     m_nbWords = iInfo.nwords;
     m_nodesUsed = iInfo.nodesused;
@@ -262,7 +277,8 @@ void Header::buildCaches()
             m_displayCache[i + 1] = it->second[0];
     }
 
-    // Create a string with all the characters possibly used
+    // Create a string with all the characters possibly used,
+    // and another one with the characters used in multi-char input strings
     m_inputChars.reserve(m_letters.size());
     BOOST_FOREACH(wchar_t wch, m_letters)
     {
@@ -276,8 +292,14 @@ void Header::buildCaches()
             {
                 BOOST_FOREACH(wchar_t chr, str)
                 {
-                    if (m_inputChars.find(towupper(chr)) == string::npos)
-                        m_inputChars.append(1, towupper(chr));
+                    wchar_t upChr = towupper(chr);
+                    if (m_inputChars.find(upChr) == string::npos)
+                        m_inputChars.append(1, upChr);
+                    if (str.size() > 1 &&
+                        m_multiCharInputChars.find(upChr) == wstring::npos)
+                    {
+                        m_multiCharInputChars.append(1, upChr);
+                    }
                 }
             }
         }
@@ -285,6 +307,12 @@ void Header::buildCaches()
 }
 
 
+bool Header::isMultiCharPart(wchar_t iChar) const
+{
+    return m_multiCharInputChars.find(towupper(iChar)) != wstring::npos;
+}
+
+
 wchar_t Header::getCharFromCode(unsigned int iCode) const
 {
     // Safety check
@@ -307,7 +335,7 @@ unsigned int Header::getCodeFromChar(wchar_t iChar) const
         char s[5];
         sprintf(s, "%d", iChar);
         format fmt(_("Header::getCodeFromChar: No code for letter '%1%' (val=%2%)"));
-        fmt % convertToMb(iChar) % s;
+        fmt % lfw(iChar) % s;
         throw DicException(fmt.str());
     }
     return pair->second;
@@ -406,15 +434,18 @@ void Header::read(istream &iStream)
     else
         throw DicException("Header::read: unrecognized algorithm type");
 
-    m_userHost = readFromUTF8(aHeaderExt.userHost, aHeaderExt.userHostSize,
+    m_userHost = readFromUTF8(string(aHeaderExt.userHost,
+                                     aHeaderExt.userHostSize),
                               "user and host information");
 
     // Convert the dictionary letters from UTF-8 to wchar_t*
-    m_dicName = readFromUTF8(aHeaderExt.dicName, aHeaderExt.dicNameSize,
+    m_dicName = readFromUTF8(string(aHeaderExt.dicName,
+                                    aHeaderExt.dicNameSize),
                              "dictionary name");
 
     // Convert the dictionary letters from UTF-8 to wchar_t*
-    m_letters = readFromUTF8(aHeaderExt.letters, aHeaderExt.lettersSize,
+    m_letters = readFromUTF8(string(aHeaderExt.letters,
+                                    aHeaderExt.lettersSize),
                              "dictionary letters");
     // Safety check: correct number of letters?
     if (m_letters.size() != aHeaderExt.nbLetters)
@@ -449,8 +480,8 @@ void Header::read(istream &iStream)
         aHeaderExt2.displayAndInputSize = ntohs(aHeaderExt2.displayAndInputSize);
 
         // Convert the dictionary letters from UTF-8 to wchar_t*
-        wstring serialized = readFromUTF8(aHeaderExt2.displayAndInput,
-                                          aHeaderExt2.displayAndInputSize,
+        wstring serialized = readFromUTF8(string(aHeaderExt2.displayAndInput,
+                                                 aHeaderExt2.displayAndInputSize),
                                           "display and input data");
         // Parse this string and structure the data
         readDisplayAndInput(serialized);
@@ -621,13 +652,13 @@ wstring Header::writeDisplayAndInput() const
 void Header::print() const
 {
 #define fmt(x) boost::format(x)
-    cout << fmt(_("Dictionary name: %1%")) % convertToMb(m_dicName) << endl;
+    cout << fmt(_("Dictionary name: %1%")) % lfw(m_dicName) << endl;
     char buf[150];
     strftime(buf, sizeof(buf), "%c", gmtime(&m_compressDate));
     cout << fmt(_("Compressed on: %1%")) % buf << endl;
-    cout << fmt(_("Compressed using a binary compiled by: %1%")) % convertToMb(m_userHost) << endl;
+    cout << fmt(_("Compressed using a binary compiled by: %1%")) % lfw(m_userHost) << endl;
     cout << fmt(_("Dictionary type: %1%")) % (m_type == kDAWG ? "DAWG" : "GADDAG") << endl;
-    cout << fmt(_("Letters: %1%")) % convertToMb(m_letters) << endl;
+    cout << fmt(_("Letters: %1%")) % lfw(m_letters) << endl;
     cout << fmt(_("Number of letters: %1%")) % m_letters.size() << endl;
     cout << fmt(_("Number of words: %1%")) % m_nbWords << endl;
     long unsigned int size = sizeof(Dict_header_old) +
@@ -665,7 +696,7 @@ void Header::print() const
                     first = false;
                 else
                     s += " ";
-                s += convertToMb(inputs[j]);
+                s += lfw(inputs[j]);
             }
             fmter % s;
         }
