@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2007-2009 Olivier Teulière
+ * Copyright (C) 2007-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,8 @@
 #include <time.h>
 #include <stdint.h>
 
+#include "logging.h"
+
 using namespace std;
 
 // XXX: duplicated typedef (also present in dic.h)
@@ -76,6 +78,7 @@ struct DictHeaderInfo
  */
 class Header
 {
+    DEFINE_LOGGER();
 public:
 
     /// Dictionary type
@@ -122,6 +125,11 @@ public:
 
     const map<wchar_t, vector<wstring> > & getDisplayInputData() const { return m_displayAndInputData; }
 
+    /**
+     * Return true if the given char is part of a multi-char input string, false otherwise
+     */
+    bool isMultiCharPart(wchar_t iChar) const;
+
     /**
      * Return the letter corresponding to the given code
      */
@@ -180,6 +188,9 @@ private:
     /// Characters usable to input the dictionary letters
     wstring m_inputChars;
 
+    /// Characters part of a multi-char input string
+    wstring m_multiCharInputChars;
+
     /// Points of the letters
     vector<uint8_t> m_points;
 
