@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2007 Olivier Teulière & Antoine Fraboulet
+ * Copyright (C) 2005-2012 Olivier Teulière & Antoine Fraboulet & Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *          Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *
@@ -27,6 +27,10 @@
 #include "header.h"
 #include "encoding.h"
 #include "dic_exception.h"
+#include "debug.h"
+
+
+INIT_LOGGER(dic, Tile);
 
 
 const Header * Tile::m_header = NULL;
@@ -58,7 +62,7 @@ Tile::Tile(wchar_t c)
     else
     {
         ostringstream ss;
-        ss << "Tile::Tile: Unknown character: " << convertToMb(c);
+        ss << "Tile::Tile: Unknown character: " << lfw(c);
         throw DicException(ss.str());
     }
 }
@@ -146,6 +150,22 @@ unsigned int Tile::toCode() const
 }
 
 
+Tile Tile::toLower() const
+{
+    ASSERT(iswalpha(m_char),
+           "toLower() should be called on alphabetical tiles");
+    return Tile(towlower(m_char));
+}
+
+
+Tile Tile::toUpper() const
+{
+    ASSERT(iswalpha(m_char),
+           "toUpper() should be called on alphabetical tiles");
+    return Tile(towupper(m_char));
+}
+
+
 bool Tile::operator<(const Tile &iOther) const
 {
     if (m_joker)
@@ -159,13 +179,8 @@ bool Tile::operator<(const Tile &iOther) const
 
 bool Tile::operator==(const Tile &iOther) const
 {
-    if (m_joker || iOther.m_joker)
-    {
-        if (m_joker != iOther.m_joker)
-            return false;
-        return m_char == iOther.m_char;
-    }
-    return m_char == iOther.m_char;
+    return m_char == iOther.m_char
+        && m_joker == iOther.m_joker;
 }
 
 
