@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2009 Olivier Teulière
+ * Copyright (C) 2005-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,9 @@
 #include "ai_percent.h"
 
 
+INIT_LOGGER(game, AIPercent);
+
+
 AIPercent::AIPercent(float iPercent)
 {
     if (iPercent < 0)
@@ -35,6 +38,8 @@ AIPercent::AIPercent(float iPercent)
     if (iPercent > 1)
         iPercent = 1;
 
+    m_percent = iPercent;
+
     // Use BestResults to be slightly faster when the percentage is 100%
     if (iPercent == 1)
         m_results = new BestResults;
@@ -53,8 +58,7 @@ void AIPercent::compute(const Dictionary &iDic, const Board &iBoard, bool iFirst
 {
     m_results->clear();
 
-    Rack rack;
-    getCurrentRack().getRack(rack);
+    const Rack &rack = getCurrentRack().getRack();
     m_results->search(iDic, iBoard, rack, iFirstWord);
 }
 
@@ -64,8 +68,8 @@ Move AIPercent::getMove() const
     if (m_results->size() == 0)
     {
         // If there is no result, pass the turn.
-        // XXX: it is forbidden in duplicate mode (even passing is forbidden),
-        // but well, what else to do?
+        // FIXME: in duplicate mode, we should return a move of type NO_MOVE
+        // instead of one of type PASS
         return Move(L"");
     }
     else
