@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -24,6 +24,7 @@
 
 #include <map>
 #include "tile.h"
+#include "logging.h"
 
 using std::map;
 
@@ -35,6 +36,7 @@ class Dictionary;
  */
 class Bag
 {
+    DEFINE_LOGGER();
 public:
     explicit Bag(const Dictionary &iDic);
 
