@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -42,6 +42,9 @@
 #define W3 3
 
 
+INIT_LOGGER(game, Board);
+
+
 const int Board::m_tileMultipliers[BOARD_REALDIM][BOARD_REALDIM] =
 {
     { oo,oo,oo,oo,oo,oo,oo,oo,oo,oo,oo,oo,oo,oo,oo,oo,oo },
@@ -96,7 +99,7 @@ Board::Board(const GameParams &iParams):
     m_crossCol(BOARD_REALDIM, Cross()),
     m_pointRow(BOARD_REALDIM, -1),
     m_pointCol(BOARD_REALDIM, -1),
-    m_testsRow(BOARD_REALDIM, false),
+    m_testsRow(BOARD_REALDIM, Tile()),
     m_isEmpty(true)
 {
     // No cross check allowed around the board
@@ -139,11 +142,8 @@ bool Board::isJoker(int iRow, int iCol) const
 
 bool Board::isVacant(int iRow, int iCol) const
 {
-    if (iRow < 1 || iRow > BOARD_DIM ||
-        iCol < 1 || iCol > BOARD_DIM)
-    {
-        return false;
-    }
+    ASSERT(iRow >= BOARD_MIN && iRow <= BOARD_MAX &&
+           iCol >= BOARD_MIN && iCol <= BOARD_MAX, "Invalid coordinates");
     return m_tilesRow[iRow][iCol].isEmpty();
 }
 
@@ -154,39 +154,52 @@ void Board::addRound(const Dictionary &iDic, const Round &iRound)
     int col = iRound.getCoord().getCol();
     if (iRound.getCoord().getDir() == Coord::HORIZONTAL)
     {
-        Tile t;
         for (unsigned int i = 0; i < iRound.getWordLen(); i++)
         {
-            if (m_tilesRow[row][col + i].isEmpty())
+            const Tile &t = iRound.getTile(i);
+            if (isVacant(row, col + i))
             {
-                t = iRound.getTile(i);
+                ASSERT(iRound.isPlayedFromRack(i), "Invalid round (1)");
                 m_tilesRow[row][col + i] = t;
                 m_jokerRow[row][col + i] = iRound.isJoker(i);
                 m_tilesCol[col + i][row] = t;
                 m_jokerCol[col + i][row] = iRound.isJoker(i);
             }
+            else
+            {
+                ASSERT(!iRound.isPlayedFromRack(i), "Invalid round (2)");
+                ASSERT(t == m_tilesRow[row][col + i].toUpper(), "Invalid round (3)");
+            }
         }
     }
     else
     {
-        Tile t;
         for (unsigned int i = 0; i < iRound.getWordLen(); i++)
         {
-            if (m_tilesRow[row + i][col].isEmpty())
+            const Tile &t = iRound.getTile(i);
+            if (isVacant(row + i, col))
             {
-                t = iRound.getTile(i);
+                ASSERT(iRound.isPlayedFromRack(i), "Invalid round (1)");
                 m_tilesRow[row + i][col] = t;
                 m_jokerRow[row + i][col] = iRound.isJoker(i);
                 m_tilesCol[col][row + i] = t;
                 m_jokerCol[col][row + i] = iRound.isJoker(i);
             }
+            else
+            {
+                ASSERT(!iRound.isPlayedFromRack(i), "Invalid round (2)");
+                ASSERT(t == m_tilesRow[row + i][col].toUpper(), "Invalid round (3)");
+            }
         }
     }
     buildCross(iDic);
-    m_isEmpty = false;
 #ifdef DEBUG
     checkDouble();
 #endif
+
+    removeTestRound();
+
+    m_isEmpty = false;
 }
 
 
@@ -203,12 +216,12 @@ void Board::removeRound(const Dictionary &iDic, const Round &iRound)
                    "Invalid round removal");
             if (iRound.isPlayedFromRack(i))
             {
+                ASSERT(iRound.isJoker(i) == m_jokerRow[row][col + i],
+                       "Invalid round removal");
                 m_tilesRow[row][col + i] = Tile();
                 m_jokerRow[row][col + i] = false;
-                m_crossRow[row][col + i].setAny();
                 m_tilesCol[col + i][row] = Tile();
                 m_jokerCol[col + i][row] = false;
-                m_crossCol[col + i][row].setAny();
             }
         }
     }
@@ -220,26 +233,30 @@ void Board::removeRound(const Dictionary &iDic, const Round &iRound)
                    "Invalid round removal");
             if (iRound.isPlayedFromRack(i))
             {
+                ASSERT(iRound.isJoker(i) == m_jokerRow[row + i][col],
+                       "Invalid round removal");
                 m_tilesRow[row + i][col] = Tile();
                 m_jokerRow[row + i][col] = false;
-                m_crossRow[row + i][col].setAny();
                 m_tilesCol[col][row + i] = Tile();
                 m_jokerCol[col][row + i] = false;
-                m_crossCol[col][row + i].setAny();
             }
         }
     }
+
+    // Rebuild all the cross checks, because they are now invalid
     buildCross(iDic);
 #ifdef DEBUG
     checkDouble();
 #endif
 
+    removeTestRound();
+
     // Update the m_isEmpty flag
     for (int i = 1; i <= BOARD_DIM; i++)
     {
         for (int j = 1; j <= BOARD_DIM; j++)
         {
-            if (!m_tilesRow[i][j].isEmpty())
+            if (!isVacant(i, j))
                 return;
         }
     }
@@ -255,8 +272,6 @@ int Board::checkRoundAux(const Matrix<Tile> &iTilesMx,
                          const Matrix<bool> &iJokerMx,
                          Round &iRound) const
 {
-    Tile t;
-    int l, p;
     bool isolated = true;
 
     int fromrack = 0;
@@ -279,23 +294,16 @@ int Board::checkRoundAux(const Matrix<Tile> &iTilesMx,
 
     for (unsigned int i = 0; i < iRound.getWordLen(); i++)
     {
-        t = iRound.getTile(i);
+        const Tile &t = iRound.getTile(i);
         if (!iTilesMx[row][col + i].isEmpty())
         {
-            // There is already a letter on the board
-            if (iTilesMx[row][col + i] != t)
+            // Using a joker tile to emulate the letter on the board is not allowed,
+            // the plain letter should be used instead.
+            // Also, make sure the played letter is the same as the one on the board
+            if (iRound.isJoker(i) || iTilesMx[row][col+i].toCode() != t.toCode())
             {
-                // Check if it is only a joker
-                if ((iTilesMx[row][col+i].toCode() == t.toCode()) && iTilesMx[row][col+i].isJoker())
-                {
-                    // Do nothing, we don't need to change the tile in the round
-                    //iRound.setJoker(i,true);
-                }
-                else
-                {
-                    // Trying to overwrite a placed letter
-                    return 2;
-                }
+                // Trying to overwrite a placed letter
+                return 2;
             }
 
             isolated = false;
@@ -313,6 +321,7 @@ int Board::checkRoundAux(const Matrix<Tile> &iTilesMx,
                 if (!iCrossMx[row][col + i].isAny())
                     isolated = false;
 
+                int l;
                 if (!iRound.isJoker(i))
                     l = t.getPoints() * m_tileMultipliers[row][col + i];
                 else
@@ -320,7 +329,7 @@ int Board::checkRoundAux(const Matrix<Tile> &iTilesMx,
                 pts += l;
                 wordmul *= m_wordMultipliers[row][col + i];
 
-                p = iPointsMx[row][col + i];
+                int p = iPointsMx[row][col + i];
                 if (p >= 0)
                 {
                     ptscross += (p + l) * m_wordMultipliers[row][col + i];
@@ -402,7 +411,7 @@ int Board::checkRound(Round &iRound) const
 
 void Board::testRound(const Round &iRound)
 {
-    Tile t;
+    removeTestRound();
 
     int row = iRound.getCoord().getRow();
     int col = iRound.getCoord().getCol();
@@ -410,15 +419,13 @@ void Board::testRound(const Round &iRound)
     {
         for (unsigned int i = 0; i < iRound.getWordLen(); i++)
         {
-            if (m_tilesRow[row][col + i].isEmpty())
+            if (isVacant(row, col + i))
             {
-                t = iRound.getTile(i);
-                m_tilesRow[row][col + i] = t;
-                m_jokerRow[row][col + i] = iRound.isJoker(i);
-                m_testsRow[row][col + i] = true;
-
-                m_tilesCol[col + i][row] = t;
-                m_jokerCol[col + i][row] = iRound.isJoker(i);
+                const Tile &t = iRound.getTile(i);
+                if (iRound.isJoker(i))
+                    m_testsRow[row][col + i] = t.toLower();
+                else
+                    m_testsRow[row][col + i] = t;
             }
         }
     }
@@ -426,15 +433,13 @@ void Board::testRound(const Round &iRound)
     {
         for (unsigned int i = 0; i < iRound.getWordLen(); i++)
         {
-            if (m_tilesRow[row + i][col].isEmpty())
+            if (isVacant(row + i, col))
             {
-                t = iRound.getTile(i);
-                m_tilesRow[row + i][col] = t;
-                m_jokerRow[row + i][col] = iRound.isJoker(i);
-                m_testsRow[row + i][col] = true;
-
-                m_tilesCol[col][row + i] = t;
-                m_jokerCol[col][row + i] = iRound.isJoker(i);
+                const Tile &t = iRound.getTile(i);
+                if (iRound.isJoker(i))
+                    m_testsRow[row + i][col] = t.toLower();
+                else
+                    m_testsRow[row + i][col] = t;
             }
         }
     }
@@ -443,26 +448,19 @@ void Board::testRound(const Round &iRound)
 
 void Board::removeTestRound()
 {
-    for (int row = 1; row <= BOARD_DIM; row++)
-    {
-        for (int col = 1; col <= BOARD_DIM; col++)
-        {
-            if (m_testsRow[row][col])
-            {
-                m_tilesRow[row][col] = Tile();
-                m_testsRow[row][col] = false;
-                m_jokerRow[row][col] = false;
-
-                m_tilesCol[col][row] = Tile();
-                m_jokerCol[col][row] = false;
-            }
-        }
-    }
+    m_testsRow = m_tilesRow;
 }
 
 
 bool Board::isTestChar(int iRow, int iCol) const
 {
+    return !m_testsRow[iRow][iCol].isEmpty() && isVacant(iRow, iCol);
+}
+
+
+const Tile& Board::getTestTile(int iRow, int iCol) const
+{
+    ASSERT(isTestChar(iRow, iCol), "The requested tile is not a test tile");
     return m_testsRow[iRow][iCol];
 }
 
@@ -516,10 +514,10 @@ void Board::checkDouble()
     {
         for (int col = BOARD_MIN; col <= BOARD_MAX; col++)
         {
-            if (m_tilesRow[row][col] != m_tilesCol[col][row])
-                printf("tiles diff %d %d\n", row, col);
-            if (m_jokerRow[row][col] != m_jokerCol[col][row])
-                printf("joker diff %d %d\n", row, col);
+            ASSERT(m_tilesRow[row][col] == m_tilesCol[col][row],
+                   "Tiles inconsistency at " << row << "x" << col);
+            ASSERT(m_jokerRow[row][col] == m_jokerCol[col][row],
+                   "Jokers inconsistency at " << row << "x" << col);
             // The crossckecks and the points have no reason to be the same
             // in both directions
         }
