@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -27,6 +27,7 @@
 #include "matrix.h"
 #include "tile.h"
 #include "cross.h"
+#include "logging.h"
 
 class GameParams;
 class Dictionary;
@@ -50,6 +51,7 @@ using namespace std;
  */
 class Board
 {
+    DEFINE_LOGGER();
 public:
     Board(const GameParams &iParams);
 
@@ -64,20 +66,16 @@ public:
     int  checkRound(Round &iRound) const;
 
     /**
-     *
+     * Preview
      */
     bool isTestChar(int iRow, int iCol) const;
+    const Tile& getTestTile(int iRow, int iCol) const;
     void testRound(const Round &iRound);
     void removeTestRound();
 
     void search(const Dictionary &iDic, const Rack &iRack, Results &oResults) const;
     void searchFirst(const Dictionary &iDic, const Rack &iRack, Results &oResults) const;
 
-    /**
-     * board_cross.c
-     */
-    void buildCross(const Dictionary &iDic);
-
     /**
      *
      */
@@ -106,7 +104,7 @@ private:
     Matrix<int> m_pointRow;
     Matrix<int> m_pointCol;
 
-    Matrix<bool> m_testsRow;
+    Matrix<Tile> m_testsRow;
 
     /// Flag indicating if the board is empty or if it has letters
     bool m_isEmpty;
@@ -114,6 +112,11 @@ private:
     static const int m_tileMultipliers[BOARD_REALDIM][BOARD_REALDIM];
     static const int m_wordMultipliers[BOARD_REALDIM][BOARD_REALDIM];
 
+    /**
+     * board_cross.c
+     */
+    void buildCross(const Dictionary &iDic);
+
     int checkRoundAux(const Matrix<Tile> &iTilesMx,
                       const Matrix<Cross> &iCrossMx,
                       const Matrix<int> &iPointsMx,
