@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2009 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -55,7 +55,7 @@ void BoardSearch::search(Rack &iRack, Results &oResults, Coord::Direction iDir)
         tmpRound.accessCoord().setCol(col);
         tmpRound.accessCoord().setDir(Coord::HORIZONTAL);
         leftPart(iRack, tmpRound, oResults, m_dic.getRoot(),
-                 row, col, iRack.getNbTiles() - 1);
+                 row, col, std::min(iRack.getNbTiles(), (unsigned)col) - 1);
         return;
     }
 
@@ -63,11 +63,10 @@ void BoardSearch::search(Rack &iRack, Results &oResults, Coord::Direction iDir)
     vector<Tile> rackTiles;
     iRack.getTiles(rackTiles);
     vector<Tile>::const_iterator it;
-    Round partialWord;
 
     for (int row = 1; row <= BOARD_DIM; row++)
     {
-        partialWord.init();
+        Round partialWord;
         partialWord.accessCoord().setDir(iDir);
         partialWord.accessCoord().setRow(row);
         int lastanchor = 0;
@@ -84,13 +83,13 @@ void BoardSearch::search(Rack &iRack, Results &oResults, Coord::Direction iDir)
                 {
                     partialWord.accessCoord().setCol(lastanchor + 1);
                     extendRight(iRack, partialWord, oResults,
-                                Dic_root(m_dic), row, lastanchor + 1, col);
+                                m_dic.getRoot(), row, lastanchor + 1, col);
                 }
                 else
                 {
                     partialWord.accessCoord().setCol(col);
                     leftPart(iRack, partialWord, oResults,
-                             Dic_root(m_dic), row, col, col - lastanchor - 1);
+                             m_dic.getRoot(), row, col, col - lastanchor - 1);
                 }
                 lastanchor = col;
 #else
@@ -137,11 +136,10 @@ void BoardSearch::leftPart(Rack &iRack, Round &ioPartialWord,
 
     if (iLimit > 0)
     {
-        Tile l;
         bool hasJokerInRack = iRack.in(Tile::Joker());
         for (unsigned int succ = m_dic.getSucc(n); succ; succ = m_dic.getNext(succ))
         {
-            l = Tile(m_dic.getChar(succ));
+            const Tile &l = Tile(m_dic.getChar(succ));
             if (iRack.in(l))
             {
                 iRack.remove(l);
@@ -150,7 +148,7 @@ void BoardSearch::leftPart(Rack &iRack, Round &ioPartialWord,
                 leftPart(iRack, ioPartialWord, oResults,
                          succ, iRow, iAnchor, iLimit - 1);
                 ioPartialWord.accessCoord().setCol(ioPartialWord.getCoord().getCol() + 1);
-                ioPartialWord.removeRightToRack(l, false);
+                ioPartialWord.removeRight();
                 iRack.add(l);
             }
             if (hasJokerInRack)
@@ -161,7 +159,7 @@ void BoardSearch::leftPart(Rack &iRack, Round &ioPartialWord,
                 leftPart(iRack, ioPartialWord, oResults,
                          succ, iRow, iAnchor, iLimit - 1);
                 ioPartialWord.accessCoord().setCol(ioPartialWord.getCoord().getCol() + 1);
-                ioPartialWord.removeRightToRack(l, true);
+                ioPartialWord.removeRight();
                 iRack.add(Tile::Joker());
             }
         }
@@ -173,8 +171,6 @@ void BoardSearch::extendRight(Rack &iRack, Round &ioPartialWord,
                               Results &oResults, unsigned int iNode,
                               int iRow, int iCol, int iAnchor) const
 {
-    Tile l;
-
     if (m_tilesMx[iRow][iCol].isEmpty())
     {
         if (m_dic.isEndOfWord(iNode) && iCol > iAnchor)
@@ -189,7 +185,7 @@ void BoardSearch::extendRight(Rack &iRack, Round &ioPartialWord,
         bool hasJokerInRack = iRack.in(Tile::Joker());
         for (unsigned int succ = m_dic.getSucc(iNode); succ; succ = m_dic.getNext(succ))
         {
-            l = Tile(m_dic.getChar(succ));
+            const Tile &l = Tile(m_dic.getChar(succ));
             if (m_crossMx[iRow][iCol].check(l))
             {
                 if (iRack.in(l))
@@ -198,7 +194,7 @@ void BoardSearch::extendRight(Rack &iRack, Round &ioPartialWord,
                     ioPartialWord.addRightFromRack(l, false);
                     extendRight(iRack, ioPartialWord, oResults,
                                 succ, iRow, iCol + 1, iAnchor);
-                    ioPartialWord.removeRightToRack(l, false);
+                    ioPartialWord.removeRight();
                     iRack.add(l);
                 }
                 if (hasJokerInRack)
@@ -207,7 +203,7 @@ void BoardSearch::extendRight(Rack &iRack, Round &ioPartialWord,
                     ioPartialWord.addRightFromRack(l, true);
                     extendRight(iRack, ioPartialWord, oResults,
                                 succ, iRow, iCol + 1, iAnchor);
-                    ioPartialWord.removeRightToRack(l, true);
+                    ioPartialWord.removeRight();
                     iRack.add(Tile::Joker());
                 }
             }
@@ -215,7 +211,7 @@ void BoardSearch::extendRight(Rack &iRack, Round &ioPartialWord,
     }
     else
     {
-        l = m_tilesMx[iRow][iCol];
+        const Tile &l = m_tilesMx[iRow][iCol];
         wint_t upperChar = towupper(l.toChar());
         for (unsigned int succ = m_dic.getSucc(iNode); succ ; succ = m_dic.getNext(succ))
         {
@@ -224,7 +220,7 @@ void BoardSearch::extendRight(Rack &iRack, Round &ioPartialWord,
                 ioPartialWord.addRightFromBoard(l);
                 extendRight(iRack, ioPartialWord,
                             oResults, succ, iRow, iCol + 1, iAnchor);
-                ioPartialWord.removeRightToBoard(l);
+                ioPartialWord.removeRight();
                 // The letter will be present only once in the dictionary,
                 // so we can stop looping
                 break;
