@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2005-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -19,11 +19,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#ifndef _COORD_H
-#define _COORD_H
+#ifndef COORD_H_
+#define COORD_H_
 
 #include <string>
 
+#include "logging.h"
+
 using std::wstring;
 
 
@@ -33,6 +35,7 @@ using std::wstring;
  */
 class Coord
 {
+    DEFINE_LOGGER();
 public:
 
     enum Direction {VERTICAL, HORIZONTAL};
