@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2007 Olivier Teulière & Antoine Fraboulet
+ * Copyright (C) 2005-2012 Olivier Teulière & Antoine Fraboulet
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *          Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *
@@ -20,11 +20,15 @@
  *****************************************************************************/
 
 #include <string>
+#include <cstdio>
 #include "cross.h"
 
 #define CROSS_MASK 0xFFFFFFFF
 
 
+INIT_LOGGER(game, Cross);
+
+
 Cross::Cross()
 {
     // The default behaviour is to match everything
