@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2009 Olivier Teulière
+ * Copyright (C) 2005-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,10 +22,12 @@
 #define DUPLICATE_H_
 
 #include "game.h"
-#include "command.h"
+#include "move.h"
 #include "logging.h"
 
 class Player;
+class PlayedRack;
+class Move;
 
 using std::string;
 using std::wstring;
@@ -57,6 +59,7 @@ class Duplicate: public Game
     DEFINE_LOGGER();
     friend class GameFactory;
     friend class MarkPlayedCmd;
+    friend class MasterMoveCmd;
 public:
 
     /*************************
@@ -90,21 +93,27 @@ public:
      */
     void setPlayer(unsigned int p);
 
+    /**
+     * Set the master move.
+     * Two move types are possible: VALID_ROUND and NO_MOVE.
+     * Setting a master move to NO_MOVE means reseting it.
+     */
+    void setMasterMove(const Move &iMove);
+
+    const Move &getMasterMove() const { return m_masterMove; }
+
     /// Return true if the player has played for the current turn
     virtual bool hasPlayed(unsigned int iPlayerId) const;
 
-private:
-    // Private constructor to force using the GameFactory class
+protected:
+    // Protected constructor to force using the GameFactory class
     Duplicate(const GameParams &iParams);
 
     /// Record a player move
-    void recordPlayerMove(const Move &iMove, Player &ioPlayer, bool isForHuman);
+    void recordPlayerMove(Player &ioPlayer, const Move &iMove);
 
-    /// Make the AI player whose ID is p play its turn
-    void playAI(unsigned int p);
-
-    /// Change the "has played" status of the given player to the given status
-    void setPlayedFlag(unsigned int iPlayerId, bool iNewFlag);
+    /// Helper function to set the game rack and the players rack at the same time
+    void setGameAndPlayersRack(const PlayedRack &iRack);
 
     /**
      * This function does not terminate the turn itself, but performs some
@@ -120,6 +129,24 @@ private:
      */
     void tryEndTurn();
 
+    /// Finish the game
+    void endGame();
+
+private: // Used by friend classes
+    void innerSetMasterMove(const Move &iMove);
+    bool isArbitrationGame() const;
+
+private:
+    /// Make the AI player whose ID is p play its turn
+    void playAI(unsigned int p);
+
+    /**
+     * Find the player who scored the most  (with a valid move) at this turn.
+     * If several players have the same score, one is returned arbitrarily.
+     * If nobody played a valid move, the method returns a null pointer.
+     */
+    Player * findBestPlayer() const;
+
     /**
      * This function really changes the turn, i.e. the best word is played,
      * the game history is updated, a "solo" bonus is given if needed, and
@@ -130,11 +157,13 @@ private:
      */
     void endTurn();
 
-    /// Finish the game
-    void endGame();
-
-    /// m_hasPlayed[p] is true iff player p has played for this turn
-    map<unsigned int, bool> m_hasPlayed;
+    /**
+     * Master move, i.e. the move that will be played on the board
+     * at this turn (even if no player actually played it).
+     * This is particularly useful for arbitration mode, but could also
+     * be used in normal Duplicate games.
+     */
+    Move m_masterMove;
 };
 
 #endif /* _DUPLICATE_H_ */
