@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2008 Olivier Teulière
+ * Copyright (C) 2005-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,13 +33,14 @@
 #include "pldrack.h"
 #include "results.h"
 #include "player.h"
-#include "player_points_cmd.h"
+#include "player_event_cmd.h"
 #include "player_move_cmd.h"
 #include "player_rack_cmd.h"
 #include "game_move_cmd.h"
+#include "game_rack_cmd.h"
 #include "ai_player.h"
 #include "settings.h"
-#include "turn.h"
+#include "turn_data.h"
 #include "encoding.h"
 
 #include "debug.h"
@@ -48,7 +49,7 @@ INIT_LOGGER(game, FreeGame);
 
 
 FreeGame::FreeGame(const GameParams &iParams)
-    : Game(iParams)
+    : Game(iParams), m_finished(false)
 {
 }
 
@@ -70,7 +71,7 @@ int FreeGame::play(const wstring &iCoord, const wstring &iWord)
         Move move(round);
 
         // Update the rack and the score of the current player
-        recordPlayerMove(move, *m_players[m_currPlayer], true);
+        recordPlayerMove(move, *m_players[m_currPlayer]);
     }
     else
     {
@@ -80,7 +81,7 @@ int FreeGame::play(const wstring &iCoord, const wstring &iWord)
         Move move(dispWord, iCoord);
 
         // Record the invalid move of the player
-        recordPlayerMove(move, *m_players[m_currPlayer], true);
+        recordPlayerMove(move, *m_players[m_currPlayer]);
     }
 
     // Next turn
@@ -99,26 +100,23 @@ void FreeGame::playAI(unsigned int p)
 
     player->compute(getDic(), getBoard(), getHistory().beforeFirstRound());
     const Move &move = player->getMove();
-    if (move.getType() == Move::CHANGE_LETTERS ||
-        move.getType() == Move::PASS)
+    if (move.isChangeLetters() || move.isPass())
     {
         ASSERT(checkPass(*player, move.getChangedLetters()) == 0,
                "AI tried to cheat!");
     }
 
     // Update the rack and the score of the current player
-    recordPlayerMove(move, *player, false);
+    recordPlayerMove(move, *player);
 
     endTurn();
 }
 
 
-void FreeGame::recordPlayerMove(const Move &iMove, Player &ioPlayer,
-                                bool isForHuman)
+void FreeGame::recordPlayerMove(const Move &iMove, Player &ioPlayer)
 {
     LOG_INFO("Player " << ioPlayer.getId() << " plays: " << lfw(iMove.toString()));
     Command *pCmd = new PlayerMoveCmd(ioPlayer, iMove);
-    pCmd->setAutoExecution(!isForHuman);
     accessNavigation().addAndExecute(pCmd);
 }
 
@@ -136,9 +134,9 @@ void FreeGame::start()
         accessNavigation().addAndExecute(pCmd);
     }
 
-    firstPlayer();
-
-    accessNavigation().newTurn();
+    // Set the game rack to the rack of the current player
+    Command *pCmd = new GameRackCmd(*this, getPlayer(0).getCurrentRack());
+    accessNavigation().addAndExecute(pCmd);
 
     // If the first player is an AI, make it play now
     if (!m_players[m_currPlayer]->isHuman())
@@ -150,28 +148,39 @@ void FreeGame::start()
 
 bool FreeGame::isFinished() const
 {
+    // FIXME: the flag is never reset to false!
     return m_finished;
 }
 
 
 int FreeGame::endTurn()
 {
+    // Give a "no move" pseudo-move to all the players
+    // who didn't play at this turn
+    for (unsigned int i = 0; i < getNPlayers(); ++i)
+    {
+        if (i == m_currPlayer)
+            continue;
+        Command *pCmd = new PlayerMoveCmd(*m_players[i], Move());
+        // The pseudo-moves should be completely transparent
+        pCmd->setHumanIndependent(true);
+        accessNavigation().addAndExecute(pCmd);
+    }
+
     const Move &move = getCurrentPlayer().getLastMove();
     // Update the game
     Command *pCmd = new GameMoveCmd(*this, move, m_currPlayer);
     accessNavigation().addAndExecute(pCmd);
 
     // Complete the rack for the player that just played
-    if (move.getType() == Move::VALID_ROUND ||
-        move.getType() == Move::CHANGE_LETTERS)
+    if (move.isValid() || move.isChangeLetters())
     {
         try
         {
             const PlayedRack &newRack =
                 helperSetRackRandom(getCurrentPlayer().getCurrentRack(), false, RACK_NEW);
-            Command *pCmd = new PlayerRackCmd(*m_players[m_currPlayer],
-                                              newRack);
-            accessNavigation().addAndExecute(pCmd);
+            Command *pCmd2 = new PlayerRackCmd(*m_players[m_currPlayer], newRack);
+            accessNavigation().addAndExecute(pCmd2);
         }
         catch (EndGameException &e)
         {
@@ -184,6 +193,10 @@ int FreeGame::endTurn()
     // Next player
     nextPlayer();
 
+    // Set the game rack to the rack of the current player
+    Command *pCmd3 = new GameRackCmd(*this, getCurrentPlayer().getCurrentRack());
+    accessNavigation().addAndExecute(pCmd3);
+
     accessNavigation().newTurn();
 
     // If this player is an AI, make it play now
@@ -214,9 +227,10 @@ void FreeGame::endGame()
     // For case 2, we need both to detect a blocked situation (not easy...) and
     // to handle it in the endGame() method (very easy).
 
-    /* Add the points of the remaining tiles to the score of the current
-     * player (i.e. the first player with an empty rack), and remove them
-     * from the score of the players who still have tiles */
+    // Add the points of the remaining tiles to the score of the current
+    // player (i.e. the first player with an empty rack), and remove them
+    // from the score of the players who still have tiles
+    int addedPoints = 0;
     for (unsigned int i = 0; i < getNPlayers(); i++)
     {
         if (i != m_currPlayer)
@@ -228,14 +242,17 @@ void FreeGame::endGame()
             {
                 points += tile.getPoints();
             }
-            // Add the points to the current player...
-            Command *pCmd = new PlayerPointsCmd(*m_players[m_currPlayer], points);
+            addedPoints += points;
+            // Remove the points from the "losing" player
+            Command *pCmd = new PlayerEventCmd(*m_players[i],
+                                               PlayerEventCmd::END_GAME, -points);
             accessNavigation().addAndExecute(pCmd);
-            // ... and remove them from the other player
-            Command *pCmd2 = new PlayerPointsCmd(*m_players[i], -points);
-            accessNavigation().addAndExecute(pCmd2);
         }
     }
+    // Add all the points to the current player
+    Command *pCmd = new PlayerEventCmd(*m_players[m_currPlayer],
+                                       PlayerEventCmd::END_GAME, addedPoints);
+    accessNavigation().addAndExecute(pCmd);
 
     // Lock game
     m_finished = true;
@@ -246,7 +263,7 @@ int FreeGame::checkPass(const Player &iPlayer,
                         const wstring &iToChange) const
 {
     // Check that the game is not finished
-    if (m_finished)
+    if (isFinished())
         return 3;
 
     // Check that the letters are valid for the current dictionary
@@ -265,8 +282,7 @@ int FreeGame::checkPass(const Player &iPlayer,
 
     // Check that the letters are all present in the player's rack
     const PlayedRack &pld = iPlayer.getCurrentRack();
-    Rack rack;
-    pld.getRack(rack);
+    Rack rack = pld.getRack();
     BOOST_FOREACH(wchar_t wch, iToChange)
     {
         // Remove the letter from the rack
@@ -295,9 +311,11 @@ int FreeGame::pass(const wstring &iToChange)
     if (res != 0)
         return res;
 
+    ASSERT(player.isHuman(), "AI tried to pass using the wrong method");
+
     Move move(iToChange);
     // End the player's turn
-    recordPlayerMove(move, player, true);
+    recordPlayerMove(move, player);
 
     // Next game turn
     endTurn();
