@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2009 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -39,10 +39,10 @@
 #include "results.h"
 #include "player.h"
 #include "game.h"
-#include "turn.h"
+#include "turn_data.h"
 #include "encoding.h"
 #include "game_exception.h"
-#include "turn_cmd.h"
+#include "turn.h"
 
 #include "debug.h"
 
@@ -54,7 +54,6 @@ Game::Game(const GameParams &iParams):
 {
     m_points = 0;
     m_currPlayer = 0;
-    m_finished = false;
 }
 
 
@@ -67,6 +66,13 @@ Game::~Game()
 }
 
 
+Player& Game::accessPlayer(unsigned int iNum)
+{
+    ASSERT(iNum < m_players.size(), "Wrong player number");
+    return *(m_players[iNum]);
+}
+
+
 const Player& Game::getPlayer(unsigned int iNum) const
 {
     ASSERT(iNum < m_players.size(), "Wrong player number");
@@ -116,9 +122,79 @@ void Game::realBag(Bag &ioBag) const
 }
 
 
+bool Game::canDrawRack(const PlayedRack &iPld, bool iCheck, int *reason) const
+{
+    // When iCheck is true, we must make sure that there are at least 2 vowels
+    // and 2 consonants in the rack up to the 15th turn, and at least one of
+    // each starting from the 16th turn.
+    // So before trying to fill the rack, we'd better make sure there is a way
+    // to complete the rack with these constraints...
+    unsigned int min = 0;
+    if (iCheck)
+    {
+        // 2 vowels and 2 consonants are needed up to the 15th turn
+        if (m_history.getSize() < 15)
+            min = 2;
+        else
+            min = 1;
+    }
+
+    // Create a copy of the bag in which we can do everything we want,
+    // and take from it the tiles of the players rack so that "bag"
+    // contains the right number of tiles.
+    Bag bag(getDic());
+    realBag(bag);
+    // Replace all the tiles of the given rack into the bag
+    vector<Tile> tiles;
+    iPld.getAllTiles(tiles);
+    BOOST_FOREACH(const Tile &tile, tiles)
+    {
+        bag.replaceTile(tile);
+    }
+
+    // Nothing in the rack, nothing in the bag --> end of the (free)game
+    if (bag.getNbTiles() == 0)
+    {
+        if (reason)
+            *reason = 1;
+        return false;
+    }
+
+    // Check whether it is possible to complete the rack properly
+    if (bag.getNbVowels() < min ||
+        bag.getNbConsonants() < min)
+    {
+        if (reason)
+            *reason = 2;
+        return false;
+    }
+
+    // In a duplicate game, we need at least 2 letters, even if we have
+    // one letter which can be considered both as a consonant and as a vowel
+    if (iCheck && bag.getNbTiles() < 2)
+    {
+        if (reason)
+            *reason = 2;
+        return false;
+    }
+
+    return true;
+}
+
+
 PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
                                      bool iCheck, set_rack_mode mode) const
 {
+    int reason = 0;
+    if (!canDrawRack(iPld, iCheck, &reason))
+    {
+        if (reason == 1)
+            throw EndGameException(_("The bag is empty"));
+        else if (reason == 2)
+            throw EndGameException(_("Not enough vowels or consonants to complete the rack"));
+        ASSERT(false, "Error code not handled")
+    }
+
     // When iCheck is true, we must make sure that there are at least 2 vowels
     // and 2 consonants in the rack up to the 15th turn, and at least one of
     // each starting from the 16th turn.
@@ -215,42 +291,6 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
         }
     }
 
-    // Count the needed consonants and vowels in the rack
-    // (i.e. minimum required, minus what we already have in the rack)
-    unsigned int neededVowels = min;
-    unsigned int neededConsonants = min;
-    BOOST_FOREACH(const Tile &tile, tiles)
-    {
-        if (neededVowels > 0 && tile.isVowel())
-            neededVowels--;
-        if (neededConsonants > 0 && tile.isConsonant())
-            neededConsonants--;
-    }
-
-    // Nothing in the rack, nothing in the bag --> end of the (free)game
-    if (bag.getNbTiles() == 0 && pld.getNbTiles() == 0)
-    {
-        throw EndGameException(_("The bag is empty"));
-    }
-
-    // Check whether it is possible to complete the rack properly
-    if (bag.getNbVowels() < neededVowels ||
-        bag.getNbConsonants() < neededConsonants)
-    {
-        throw EndGameException(_("Not enough vowels or consonants to complete the rack"));
-    }
-    // End of game condition
-    if (iCheck)
-    {
-        // FIXME: redundant checks?
-        if (bag.getNbVowels() < neededVowels ||
-            bag.getNbConsonants() < neededConsonants ||
-            (bag.getNbTiles() + tiles.size()) == 1)
-        {
-            throw EndGameException(_("Not enough vowels or consonants to complete the rack"));
-        }
-    }
-
     // Handle reject:
     // Now that the joker has been dealt with, we try to complete the rack
     // with truly random tiles. If it meets the requirements (i.e. if there
@@ -278,13 +318,20 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
         // Do not mark the rack as rejected if it was empty
         if (nold > 0)
             pld.setReject();
-        // Reset the number of required vowels and consonants
-        neededVowels = min;
-        neededConsonants = min;
+
+        // Keep track of the needed consonants and vowels in the rack
+        unsigned int neededVowels = min;
+        unsigned int neededConsonants = min;
 
         // Restore the joker if we are in a joker game
         if (jokerAdded)
+        {
             pld.addNew(Tile::Joker());
+            if (neededVowels > 0)
+                --neededVowels;
+            if (neededConsonants > 0)
+                --neededConsonants;
+        }
 
         // RACK_SIZE - tiles.size() is the number of letters to add to the rack
         if (neededVowels > RACK_SIZE - tiles.size() ||
@@ -306,7 +353,7 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
             // Handle the case where the vowel can also be considered
             // as a consonant
             if (l.isConsonant() && neededConsonants > 0)
-                neededConsonants--;
+                --neededConsonants;
         }
         for (unsigned int i = 0; i < neededConsonants; ++i)
         {
@@ -332,8 +379,7 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
     // A joker coming from a previous rack is not replaced
     if (m_params.hasVariant(GameParams::kEXPLOSIVE) && jokerAdded)
     {
-        Rack rack;
-        pld.getRack(rack);
+        const Rack &rack = pld.getRack();
 
         BestResults res;
         res.search(getDic(), getBoard(), rack,  getHistory().beforeFirstRound());
@@ -353,7 +399,7 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
                 if (bestRound.isJoker(i) && bestRound.isPlayedFromRack(i))
                 {
                     const Tile &jokerTile = bestRound.getTile(i);
-                    Tile replacingTile(towupper(jokerTile.toChar()));
+                    const Tile &replacingTile = jokerTile.toUpper();
                     LOG_DEBUG("helperSetRackRandom(): replacing Joker with "
                               << lfw(replacingTile.toChar()));
 
@@ -364,8 +410,7 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
                         // The bag contains the replacing letter
                         // We need to swap the joker (it is necessarily in the
                         // new tiles, because jokerAdded is true)
-                        Rack tmpRack;
-                        pld.getNew(tmpRack);
+                        Rack tmpRack = pld.getNew();
                         ASSERT(tmpRack.in(Tile::Joker()),
                                "No joker found in the new tiles!");
                         tmpRack.remove(Tile::Joker());
@@ -413,8 +458,7 @@ PlayedRack Game::helperSetRackManual(bool iCheck, const wstring &iLetters) const
     PlayedRack pld;
     pld.setManual(iLetters);
 
-    Rack rack;
-    pld.getRack(rack);
+    const Rack &rack = pld.getRack();
     if (!rackInBag(rack, m_bag))
     {
         throw GameException(_("The bag does not contain all these letters"));
@@ -436,6 +480,7 @@ PlayedRack Game::helperSetRackManual(bool iCheck, const wstring &iLetters) const
     return pld;
 }
 
+
 /*********************************************************
  *********************************************************/
 
@@ -465,32 +510,6 @@ void Game::addPlayer(Player *iPlayer)
 }
 
 
-void Game::firstPlayer()
-{
-    ASSERT(getNPlayers() != 0, "Expected at least one player");
-    // Make sure there is something to do
-    if (m_currPlayer == 0)
-        return;
-
-    Command *pCmd = new CurrentPlayerCmd(*this, 0);
-    accessNavigation().addAndExecute(pCmd);
-}
-
-
-void Game::prevPlayer()
-{
-    ASSERT(getNPlayers() != 0, "Expected at least one player");
-
-    unsigned int newPlayerId;
-    if (m_currPlayer == 0)
-        newPlayerId = getNPlayers() - 1;
-    else
-        newPlayerId = m_currPlayer - 1;
-    Command *pCmd = new CurrentPlayerCmd(*this, newPlayerId);
-    accessNavigation().addAndExecute(pCmd);
-}
-
-
 void Game::nextPlayer()
 {
     ASSERT(getNPlayers() != 0, "Expected at least one player");
@@ -515,7 +534,7 @@ int Game::checkPlayedWord(const wstring &iCoord,
         return 1;
 
     // Init the round with the given coordinates
-    oRound.init();
+    oRound = Round();
     oRound.accessCoord().setFromString(iCoord);
     if (!oRound.getCoord().isValid())
     {
@@ -538,11 +557,6 @@ int Game::checkPlayedWord(const wstring &iCoord,
         tiles.push_back(Tile(iWord[i]));
     }
     oRound.setWord(tiles);
-    for (unsigned int i = 0; i < iWord.size(); i++)
-    {
-        if (islower(iWord[i]))
-            oRound.setJoker(i);
-    }
 
     // Check the word position, compute its points,
     // and specify the origin of each letter (board or rack)
@@ -550,7 +564,9 @@ int Game::checkPlayedWord(const wstring &iCoord,
     if (res != 0)
         return res + 4;
     // In duplicate mode, the first word must be horizontal
-    if (getMode() == GameParams::kDUPLICATE && m_board.isVacant(8, 8))
+    if (m_board.isVacant(8, 8) &&
+        (getMode() == GameParams::kDUPLICATE ||
+         getMode() == GameParams::kARBITRATION))
     {
         if (oRound.getCoord().getDir() == Coord::VERTICAL)
             return 10;
@@ -561,9 +577,8 @@ int Game::checkPlayedWord(const wstring &iCoord,
         // Check that the word can be formed with the tiles in the rack:
         // we first create a copy of the rack, then we remove the tiles
         // one by one
-        Rack rack;
         Player *player = m_players[m_currPlayer];
-        player->getCurrentRack().getRack(rack);
+        Rack rack = player->getCurrentRack().getRack();
 
         Tile t;
         for (unsigned int i = 0; i < oRound.getWordLen(); i++)
