@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2009 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -37,7 +37,7 @@ class Player;
 class PlayedRack;
 class Round;
 class Rack;
-class Turn;
+class TurnData;
 
 using namespace std;
 
@@ -99,6 +99,7 @@ public:
      * Methods to access players.
      ***************/
 
+    Player & accessPlayer(unsigned int iNum);
     const Player& getPlayer(unsigned int iNum) const;
     const Player& getCurrentPlayer() const { return getPlayer(currPlayer()); };
     unsigned int getNPlayers() const { return m_players.size(); }
@@ -157,19 +158,6 @@ public:
     // XXX: not very nice API, should be a player property...
     virtual bool hasPlayed(unsigned int player) const { return player != currPlayer(); }
 
-    /***************
-     * Saved games handling
-     ***************/
-
-    /**
-     * XXX FIXME XXX: these methods are deprecated, don't use them anymore.
-     * load() returns the loaded game, or NULL if there was a problem
-     * load() does need some more work to be robust enough to
-     * handle "hand written" files
-     */
-    static Game * load(FILE *fin, const Dictionary &iDic);
-    static Game * load(const string &iFileName, const Dictionary &iDic);
-
     /***************
      * Setting the rack
      ***************/
@@ -242,12 +230,23 @@ protected:
     /// Bag
     Bag m_bag;
 
-    bool m_finished;
-
     /*********************************************************
      * Helper functions
      *********************************************************/
 
+    /**
+     * Check if it is possible to draw a new rack from the bag,
+     * after we put back the contents of iPld.
+     * @param iCheck if true, the number of consonants and vowels will
+     * be checked.
+     * @param oReason if the pointer is valid, and the method returns false,
+     *      oReason will point to the error code. Possible values:
+     *          1: The bag is empty
+     *          2: Not enough vowels or consonants to complete the rack
+     * @return true if a rack can be drawn, false otherwise
+     */
+    bool canDrawRack(const PlayedRack &iPld, bool iCheck, int *oReason = NULL) const;
+
     /**
      * Complete the given rack randomly.
      *
@@ -278,8 +277,6 @@ protected:
      */
     PlayedRack helperSetRackManual(bool iCheck, const wstring &iLetters) const;
 
-    void firstPlayer();
-    void prevPlayer();
     void nextPlayer();
 
     /**
@@ -294,30 +291,6 @@ protected:
      */
     bool rackInBag(const Rack &iRack, const Bag &iBag) const;
 
-    /**
-     * load games from File using the first format.
-     * This format is used for Training games
-     */
-    static Game* gameLoadFormat_14(FILE *fin, const Dictionary& iDic);
-
-    /**
-     * load games from File using advanced format (since Eliot 1.5)
-     * This format is used for Duplicate, FreeGame, ...
-     */
-    static Game* gameLoadFormat_15(FILE *fin, const Dictionary& iDic);
-
-#if 0
-    /**
-     * Training games ares saved using the initial Eliot format
-     */
-    void gameSaveFormat_14(ostream &out) const;
-
-    /**
-     * Advanced game file format output
-     */
-    void gameSaveFormat_15(ostream &out) const;
-#endif
-
 };
 
 #endif /* _GAME_H_ */
