@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2007 Olivier Teulière & Antoine Fraboulet
+ * Copyright (C) 2005-2012 Olivier Teulière & Antoine Fraboulet
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *          Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *
@@ -40,6 +40,7 @@
 #include "training.h"
 #include "freegame.h"
 #include "duplicate.h"
+#include "arbitration.h"
 #include "player.h"
 #include "ai_percent.h"
 #include "dic.h"
@@ -98,6 +99,12 @@ Game *GameFactory::createGame(const GameParams &iParams)
         Duplicate *game = new Duplicate(iParams);
         return game;
     }
+    if (iParams.getMode() == GameParams::kARBITRATION)
+    {
+        LOG_INFO("Creating an arbitration game");
+        Arbitration *game = new Arbitration(iParams);
+        return game;
+    }
     throw GameException("Unknown game type");
 }
 
@@ -156,7 +163,7 @@ Game *GameFactory::createFromCmdLine(int argc, char **argv)
                 }
                 else
                     name = wfl(optarg);
-                m_players.push_back(make_pair<bool, wstring>(res == 'u', name));
+                m_players.push_back(make_pair(res == 'u', name));
             }
             break;
         case 500:
