@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2005-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -19,31 +19,24 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-/**
- *  \file   history.cpp
- *  \brief  Game history  system
- *  \author Antoine Fraboulet
- *  \date   2005
- */
+#include <boost/foreach.hpp>
 
 #include <string>
 #include "rack.h"
 #include "pldrack.h"
 #include "move.h"
-#include "turn.h"
+#include "turn_data.h"
 #include "history.h"
 #include "encoding.h"
 #include "debug.h"
 
 
-/* ******************************************************** */
-/* ******************************************************** */
-/* ******************************************************** */
+INIT_LOGGER(game, History);
 
 
 History::History()
 {
-    Turn* t = new Turn();
+    TurnData *t = new TurnData();
     m_history.clear();
     m_history.push_back(t);
 }
@@ -51,9 +44,9 @@ History::History()
 
 History::~History()
 {
-    for (unsigned int i = 0; i < m_history.size(); i++)
+    BOOST_FOREACH(TurnData *turn, m_history)
     {
-        delete m_history[i];
+        delete turn;
     }
 }
 
@@ -77,7 +70,7 @@ void History::setCurrentRack(const PlayedRack &iPld)
 }
 
 
-const Turn& History::getPreviousTurn() const
+const TurnData& History::getPreviousTurn() const
 {
     int idx = m_history.size() - 2;
     ASSERT(0 <= idx , "No previous turn");
@@ -85,7 +78,7 @@ const Turn& History::getPreviousTurn() const
 }
 
 
-const Turn& History::getTurn(unsigned int n) const
+const TurnData& History::getTurn(unsigned int n) const
 {
     ASSERT(n < m_history.size(), "Wrong turn number");
     return *(m_history[n]);
@@ -96,56 +89,26 @@ bool History::beforeFirstRound() const
 {
     for (unsigned int i = 0; i < m_history.size() - 1; i++)
     {
-        if (m_history[i]->getMove().getType() == Move::VALID_ROUND)
+        if (m_history[i]->getMove().isValid())
             return false;
     }
     return true;
 }
 
 
-void History::playMove(unsigned int iPlayer, unsigned int iTurn, const Move &iMove)
+void History::playMove(unsigned int iPlayer,
+                       const Move &iMove,
+                       const PlayedRack &iNewRack)
 {
-    Turn * current_turn = m_history.back();
+    TurnData * current_turn = m_history.back();
 
     // Set the number and the round
-    current_turn->setNum(iTurn);
     current_turn->setPlayer(iPlayer);
     current_turn->setMove(iMove);
 
-    // Get what was the rack for the current turn
-    Rack rack;
-    current_turn->getPlayedRack().getRack(rack);
-
-    if (iMove.getType() == Move::VALID_ROUND)
-    {
-        // Remove the played tiles from the rack
-        const Round &round = iMove.getRound();
-        for (unsigned int i = 0; i < round.getWordLen(); i++)
-        {
-            if (round.isPlayedFromRack(i))
-            {
-                if (round.isJoker(i))
-                    rack.remove(Tile::Joker());
-                else
-                    rack.remove(round.getTile(i));
-            }
-        }
-    }
-    else if (iMove.getType() == Move::CHANGE_LETTERS)
-    {
-        // Remove the changed tiles from the rack
-        const wstring & changed = iMove.getChangedLetters();
-        for (unsigned int i = 0; i < changed.size(); ++i)
-        {
-            rack.remove(Tile(changed[i]));
-        }
-    }
-
     // Create a new turn
-    Turn * next_turn = new Turn();
-    PlayedRack pldrack;
-    pldrack.setOld(rack);
-    next_turn->setPlayedRack(pldrack);
+    TurnData * next_turn = new TurnData();
+    next_turn->setPlayedRack(iNewRack);
     m_history.push_back(next_turn);
 }
 
@@ -157,14 +120,13 @@ void History::removeLastTurn()
 
     if (idx > 1)
     {
-        Turn *t = m_history.back();
+        TurnData *t = m_history.back();
         m_history.pop_back();
         delete t;
     }
 
     // Now we have the previous played round in back()
-    Turn *t = m_history.back();
-    t->setNum(0);
+    TurnData *t = m_history.back();
     t->setPlayer(0);
     //t->setRound(Round());
 #ifdef BACK_REMOVE_RACK_NEW_PART
@@ -173,6 +135,41 @@ void History::removeLastTurn()
 }
 
 
+void History::addWarning()
+{
+    ASSERT(m_history.size() > 1, "Too short history");
+    m_history[m_history.size() - 2]->addWarning();
+}
+
+
+void History::removeWarning()
+{
+    ASSERT(m_history.size() > 1, "Too short history");
+    m_history[m_history.size() - 2]->addWarning(-1);
+}
+
+
+void History::addPenaltyPoints(int iPoints)
+{
+    ASSERT(m_history.size() > 1, "Too short history");
+    m_history[m_history.size() - 2]->addPenaltyPoints(iPoints);
+}
+
+
+void History::addSoloPoints(int iPoints)
+{
+    ASSERT(m_history.size() > 1, "Too short history");
+    m_history[m_history.size() - 2]->addSoloPoints(iPoints);
+}
+
+
+void History::addEndGamePoints(int iPoints)
+{
+    ASSERT(m_history.size() > 1, "Too short history");
+    m_history[m_history.size() - 2]->addEndGamePoints(iPoints);
+}
+
+
 wstring History::toString() const
 {
     wstring rs;
@@ -181,22 +178,10 @@ wstring History::toString() const
     _swprintf(buff, 4, L"%ld", m_history.size());
     rs = L"history size = " + wstring(buff) + L"\n\n";
 #endif
-    for (unsigned int i = 0; i < m_history.size(); i++)
+    BOOST_FOREACH(const TurnData *turn, m_history)
     {
-        Turn *t = m_history[i];
-        rs += t->toString() + L"\n";
+        rs += turn->toString() + L"\n";
     }
     return rs;
 }
 
-/* ******************************************************** */
-/* ******************************************************** */
-/* ******************************************************** */
-
-
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
