@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2002-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2002-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -25,6 +25,9 @@
 #include "debug.h"
 
 
+INIT_LOGGER(game, Rack);
+
+
 Rack::Rack()
     : m_tiles(Dictionary::GetDic().getTileNumber() + 1, 0), m_ntiles(0)
 {
@@ -34,7 +37,7 @@ Rack::Rack()
 void Rack::remove(const Tile &t)
 {
     ASSERT(in(t),
-           "The rack does not contain the letter " + convertToMb(t.getDisplayStr()));
+           "The rack does not contain the letter " + lfw(t.getDisplayStr()));
     m_tiles[t.toCode()]--;
     m_ntiles--;
 }
