@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2002-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2002-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -26,6 +26,7 @@
 #include <string>
 
 #include "tile.h"
+#include "logging.h"
 
 using namespace std;
 
@@ -36,6 +37,7 @@ using namespace std;
  */
 class Rack
 {
+    DEFINE_LOGGER();
 public:
     Rack();
 
