@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2009 Antoine Fraboulet & Olivier Teulire
+ * Copyright (C) 2005-2012 Antoine Fraboulet & Olivier Teulire
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulire <ipkiss @@ gmail.com>
  *
@@ -25,6 +25,7 @@
 #include <vector>
 #include <map>
 #include "round.h"
+#include "logging.h"
 
 using namespace std;
 
@@ -46,7 +47,9 @@ class Rack;
  */
 class Results
 {
+    DEFINE_LOGGER();
 public:
+    virtual ~Results() {}
     unsigned int size() const { return m_rounds.size(); }
     const Round & get(unsigned int) const;
 
