@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -26,30 +26,15 @@
 #include "tile.h"
 #include "round.h"
 #include "encoding.h"
+#include "debug.h"
 
 
-#define FROMBOARD 0x1
-#define FROMRACK  0x2
-#define JOKER     0x4
-
-#define __UNUSED__ __attribute__((unused))
+INIT_LOGGER(game, Round);
 
 
 Round::Round()
+    : m_coord(1, 1, Coord::HORIZONTAL), m_points(0), m_bonus(false)
 {
-    init();
-}
-
-
-void Round::init()
-{
-    m_word.clear();
-    m_tileOrigin.clear();
-    m_coord.setRow(1);
-    m_coord.setCol(1);
-    m_coord.setDir(Coord::HORIZONTAL);
-    m_points = 0;
-    m_bonus  = false;
 }
 
 
@@ -57,85 +42,80 @@ void Round::setWord(const vector<Tile> &iTiles)
 {
     m_word = iTiles;
     // XXX: always from rack?
-    m_tileOrigin = vector<char>(iTiles.size(), FROMRACK);
+    m_rackOrigin = vector<bool>(iTiles.size(), true);
+}
+
+
+void Round::setTile(unsigned int iIndex, const Tile &iTile)
+{
+    ASSERT(iIndex < m_word.size(), "Invalid index");
+    m_word[iIndex] = iTile;
 }
 
 
 void Round::setFromRack(unsigned int iIndex)
 {
-    m_tileOrigin[iIndex] &= ~FROMBOARD;
-    m_tileOrigin[iIndex] |= FROMRACK;
+    ASSERT(iIndex < m_word.size(), "Invalid index");
+    m_rackOrigin[iIndex] = true;
 }
 
 
 void Round::setFromBoard(unsigned int iIndex)
 {
-    m_tileOrigin[iIndex] &= ~FROMRACK;
-    m_tileOrigin[iIndex] |= FROMBOARD;
-}
-
-
-void Round::setJoker(unsigned int iIndex, bool value)
-{
-    if (value)
-        m_tileOrigin[iIndex] |= JOKER;
-    else
-        m_tileOrigin[iIndex] &= ~JOKER;
+    ASSERT(iIndex < m_word.size(), "Invalid index");
+    m_rackOrigin[iIndex] = false;
 }
 
 
 bool Round::isJoker(unsigned int iIndex) const
 {
-     return m_tileOrigin[iIndex] & JOKER;
+    ASSERT(iIndex < m_word.size(), "Invalid index");
+     return m_word[iIndex].isJoker();
 }
 
 
 const Tile& Round::getTile(unsigned int iIndex) const
 {
+    ASSERT(iIndex < m_word.size(), "Invalid index");
      return m_word[iIndex];
 }
 
 
 bool Round::isPlayedFromRack(unsigned int iIndex) const
 {
-     return m_tileOrigin[iIndex] & FROMRACK;
+    ASSERT(iIndex < m_word.size(), "Invalid index");
+     return m_rackOrigin[iIndex];
 }
 
 
-void Round::addRightFromBoard(Tile c)
+void Round::addRightFromBoard(const Tile &iTile)
 {
-    m_word.push_back(c);
-    m_tileOrigin.push_back(FROMBOARD);
+    // The call to toUpper() is necessary to avoid that a joker
+    // on the board appears as a joker in the Round
+    m_word.push_back(iTile.toUpper());
+    m_rackOrigin.push_back(false);
 }
 
 
-void Round::removeRightToBoard(Tile __UNUSED__ c)
+void Round::addRightFromRack(const Tile &iTile, bool iJoker)
 {
-    // c is unused.
-    m_word.pop_back();
-    m_tileOrigin.pop_back();
-}
-
-
-void Round::addRightFromRack(Tile c, bool iJoker)
-{
-    m_word.push_back(c);
-    char origin = FROMRACK;
     if (iJoker)
-    {
-        origin |= JOKER;
-    }
-    m_tileOrigin.push_back(origin);
+        m_word.push_back(iTile.toLower());
+    else
+        m_word.push_back(iTile);
+    m_rackOrigin.push_back(true);
 }
 
 
-void Round::removeRightToRack(Tile __UNUSED__ c, bool __UNUSED__ iJoker)
+void Round::removeRight()
 {
-    // c is unused.
+    ASSERT(!m_word.empty() && !m_rackOrigin.empty(),
+           "Trying to remove tiles that were never added");
     m_word.pop_back();
-    m_tileOrigin.pop_back();
+    m_rackOrigin.pop_back();
 }
 
+
 wstring Round::getWord() const
 {
     wstring s;
@@ -150,6 +130,19 @@ wstring Round::getWord() const
     return s;
 }
 
+
+unsigned Round::countJokersFromRack() const
+{
+    unsigned count = 0;
+    for (unsigned int i = 0; i < getWordLen(); i++)
+    {
+        if (isJoker(i) && isPlayedFromRack(i))
+            ++count;
+    }
+    return count;
+}
+
+
 wstring Round::toString() const
 {
     wostringstream oss;
