@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -25,6 +25,7 @@
 #include <vector>
 #include "tile.h"
 #include "coord.h"
+#include "logging.h"
 
 using namespace std;
 
@@ -37,32 +38,27 @@ using namespace std;
  */
 class Round
 {
+    DEFINE_LOGGER();
 public:
 
-    /*************************
-     *
-     *************************/
     Round();
-    void init();
 
     /*************************
      *
      *************************/
-    void addRightFromBoard(Tile);
-    void removeRightToBoard(Tile);
-    void addRightFromRack(Tile, bool);
-    void removeRightToRack(Tile, bool);
+    void addRightFromBoard(const Tile &iTile);
+    void addRightFromRack(const Tile &iTile, bool iJoker);
+    void removeRight();
 
     /*************************
      * General setters
      *************************/
     void setPoints(int iPoints)    { m_points = iPoints; }
     void setBonus(bool iBonus)     { m_bonus = iBonus; }
-    void setTile(unsigned int iIndex, const Tile &iTile) { m_word[iIndex] = iTile; }
+    void setTile(unsigned int iIndex, const Tile &iTile);
     void setWord(const vector<Tile> &iTiles);
     void setFromRack(unsigned int iIndex);
     void setFromBoard(unsigned int iIndex);
-    void setJoker(unsigned int iIndex, bool value = true);
 
     /*************************
      * General getters
@@ -77,6 +73,8 @@ public:
     bool getBonus() const           { return m_bonus; }
     const vector<Tile> &getTiles() const { return m_word; }
 
+    unsigned countJokersFromRack() const;
+
     /*************************
      * Coordinates
      *************************/
@@ -88,7 +86,7 @@ public:
 
 private:
     vector<Tile> m_word;
-    vector<char> m_tileOrigin;
+    vector<bool> m_rackOrigin;
     Coord m_coord;
     int m_points;
     bool m_bonus;
