@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2007 Olivier Teulière
+ * Copyright (C) 2007-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,6 +41,9 @@
 using namespace libconfig;
 
 
+INIT_LOGGER(game, Settings);
+
+
 Settings *Settings::m_instance = NULL;
 
 
@@ -61,66 +64,116 @@ void Settings::Destroy()
 }
 
 
-namespace
+// Return true if the given path exists and is a directory)
+static bool is_directory(const string &path)
 {
-    string getConfigFileName()
-    {
-        string fileName;
 #ifdef WIN32
-        char szPath[MAX_PATH];
-        // Get the AppData directory
-        if (SHGetFolderPath(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE,
-                            NULL, 0, szPath) == S_OK)
-        {
-            fileName = szPath + string("\\eliot");
-#if 1
-            strncpy(szPath, fileName.c_str(), MAX_PATH);
-            // Try to create the directory
-            // We don't care about the results
-            CreateDirectory(fileName.c_str(), NULL);
-#endif
-        }
-        if (fileName != "")
-            fileName += "\\";
-        fileName += "eliot.cfg";
+    DWORD attrib = GetFileAttributes(path.c_str());
+    return (attrib != INVALID_FILE_ATTRIBUTES &&
+            (attrib & FILE_ATTRIBUTE_DIRECTORY));
 #else
-        // Follow the XDG Base Directory Specification (from freedesktop.org)
-        // XXX: In fact we don't follow it to the letter, because the location
-        // of the config file could be different when reading and writing.
-        // But in the case of Eliot it's not very important (we don't try to
-        // merge config files)...
-        const char *configDir = getenv("XDG_CONFIG_HOME");
-        if (configDir != NULL)
-            fileName = configDir;
-        else
-        {
-            // Fallback to the default value: $HOME/.config
-            configDir = getenv("HOME");
-            if (configDir)
-                fileName = configDir + string("/.config");
-        }
-        fileName += "/eliot";
-
 #if defined(HAVE_SYS_STAT_H) && defined(HAVE_SYS_TYPES_H)
-        // Create the directory if it doesn't exist
-        struct stat sb;
-        if (fileName != "" && stat(fileName.c_str(), &sb) == -1)
-        {
-            // Try to create the directory with mode 0700
-            if (mkdir(fileName.c_str(), S_IRWXU))
-            {
-                // The directory could not be created. Too bad...
-                // Saving the configuration file will definitely fail.
-            }
-        }
+    struct stat sb;
+    int res = stat(path.c_str(), &sb);
+    return res == 0 && S_ISDIR(sb.st_mode);
+#else
+    // Unimplemented
+    return false;
 #endif
+#endif
+}
 
-        fileName += "/eliot.cfg";
+// Create a directory.
+// Return true in case of success, false otherwise.
+static bool my_mkdir(const string &dir)
+{
+#ifdef WIN32
+    // The value '248' comes from MSDN
+    char tmp[248];
+    snprintf(tmp, sizeof(tmp), "%s", dir.c_str());
+    return CreateDirectory(tmp, NULL);
+#else
+#if defined(HAVE_SYS_STAT_H) && defined(HAVE_SYS_TYPES_H)
+    // Create the directory with mode 0700
+    return mkdir(dir.c_str(), S_IRWXU) == 0;
+#else
+    // Unimplemented
+    return false;
 #endif
-        return fileName;
+#endif
+}
+
+// Create a directory, like mkdir -p
+// We ignore potential errors...
+static void full_mkdir(const string &dir)
+{
+    // Remove trailing '/'
+    string copy = dir;
+    string::size_type pos = dir.find_last_not_of('/');
+    if (pos != string::npos && pos != dir.size() - 1)
+        copy.erase(pos + 1, dir.size() - 1 - pos);
+
+    // Create intermediate directories
+    pos = 0;
+    while ((pos = copy.find('/', pos)) != string::npos)
+    {
+        // Ignore potential errors...
+        my_mkdir(copy.substr(0, pos));
+        ++pos;
     }
 
+    // Create the final directory
+    my_mkdir(copy);
+}
 
+
+string Settings::GetConfigFileDir()
+{
+    string dirName;
+#ifdef WIN32
+    char szPath[MAX_PATH];
+    // Get the AppData directory
+    if (SHGetFolderPath(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE,
+                        NULL, 0, szPath) == S_OK)
+    {
+        dirName = szPath + string("/eliot");
+    }
+#else
+    // Follow the XDG Base Directory Specification (from freedesktop.org)
+    // XXX: In fact we don't follow it to the letter, because the location
+    // of the config file could be different when reading and writing.
+    // But in the case of Eliot it's not very important (we don't try to
+    // merge config files)...
+    const char *configDir = getenv("XDG_CONFIG_HOME");
+    if (configDir != NULL)
+        dirName = configDir;
+    else
+    {
+        // Fallback to the default value: $HOME/.config
+        configDir = getenv("HOME");
+        if (configDir)
+            dirName = configDir + string("/.config");
+    }
+    dirName += "/eliot";
+#endif
+
+    if (dirName != "")
+        dirName += "/";
+
+    // Try to create the directory if it doesn't exist.
+    // If the directory cannot be created, saving the
+    // configuration file will definitely fail...
+    if (!is_directory(dirName))
+    {
+        full_mkdir(dirName);
+    }
+
+    return dirName;
+}
+
+
+namespace
+{
 #ifdef HAVE_LIBCONFIG
     template<typename T>
     void copySetting(const Config &srcConf, Config &dstConf, const char *path)
@@ -139,7 +192,7 @@ namespace
 Settings::Settings()
 {
 #ifdef HAVE_LIBCONFIG
-    m_fileName = ::getConfigFileName();
+    m_fileName = GetConfigFileDir() + "eliot.cfg";
     m_conf = new Config;
 
     // ============== General options ==============
@@ -174,6 +227,24 @@ Settings::Settings()
     // be rejected in any case.
     freegame.add("reject-invalid", Setting::TypeBoolean) = true;
 
+    // ============== Arbitration mode options ==============
+    Setting &arbitration = m_conf->getRoot().add("arbitration", Setting::TypeGroup);
+
+    // If true, a random rack is defined, otherwise the rack is left untouched
+    arbitration.add("fill-rack", Setting::TypeBoolean) = true;
+
+    // Number of search results kept in a search
+    arbitration.add("search-limit", Setting::TypeInt) = 100;
+
+    // Number of points granted for a solo (10 is the ODS value)
+    arbitration.add("solo-value", Setting::TypeInt) = 10;
+
+    // Default value of a penalty
+    arbitration.add("penalty-value", Setting::TypeInt) = 5;
+
+    // Maximum number of warnings before getting penalties
+    arbitration.add("warnings-limit", Setting::TypeInt) = 3;
+
     // Try to read the values from the configuration file
     try
     {
@@ -187,6 +258,11 @@ Settings::Settings()
         copySetting<int>(tmpConf, *m_conf, "duplicate.solo-value");
         copySetting<bool>(tmpConf, *m_conf, "duplicate.reject-invalid");
         copySetting<bool>(tmpConf, *m_conf, "freegame.reject-invalid");
+        copySetting<bool>(tmpConf, *m_conf, "arbitration.fill-rack");
+        copySetting<int>(tmpConf, *m_conf, "arbitration.search-limit");
+        copySetting<int>(tmpConf, *m_conf, "arbitration.solo-value");
+        copySetting<int>(tmpConf, *m_conf, "arbitration.penalty-value");
+        copySetting<int>(tmpConf, *m_conf, "arbitration.warnings-limit");
     }
     catch (...)
     {
@@ -269,6 +345,14 @@ int Settings::getInt(const string &iName) const
         return 16;
     else if (iName == "duplicate.solo-value")
         return 10;
+    else if (iName == "arbitration.search-limit")
+        return 100;
+    else if (iName == "arbitration.solo-value")
+        return 5;
+    else if (iName == "arbitration.penalty-value")
+        return 5;
+    else if (iName == "arbitration.warnings-limit")
+        return 3;
     return 0;
 #endif
 }
