@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2009 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -22,6 +22,7 @@
 #include <algorithm>
 #include <cwctype> // For towupper
 
+#include "config.h"
 #if ENABLE_NLS
 #   include <libintl.h>
 #   define _(String) gettext(String)
@@ -41,6 +42,7 @@
 #include "player_move_cmd.h"
 #include "player_rack_cmd.h"
 #include "game_move_cmd.h"
+#include "game_rack_cmd.h"
 #include "encoding.h"
 
 #include "debug.h"
@@ -59,10 +61,13 @@ void Training::setRackRandom(bool iCheck, set_rack_mode mode)
 {
     m_results.clear();
     const PlayedRack &newRack =
-        helperSetRackRandom(getCurrentPlayer().getCurrentRack(), iCheck, mode);
-    Command *pCmd = new PlayerRackCmd(*m_players[m_currPlayer], newRack);
-    pCmd->setAutoExecution(false);
-    accessNavigation().addAndExecute(pCmd);
+        helperSetRackRandom(getHistory().getCurrentRack(), iCheck, mode);
+    Command *pCmd1 = new GameRackCmd(*this, newRack);
+    pCmd1->setHumanIndependent(false);
+    accessNavigation().addAndExecute(pCmd1);
+    Command *pCmd2 = new PlayerRackCmd(*m_players[m_currPlayer], newRack);
+    pCmd2->setHumanIndependent(false);
+    accessNavigation().addAndExecute(pCmd2);
 }
 
 
@@ -76,9 +81,12 @@ void Training::setRackManual(bool iCheck, const wstring &iLetters)
     std::transform(upperLetters.begin(), upperLetters.end(),
                    upperLetters.begin(), towupper);
     const PlayedRack &newRack = helperSetRackManual(iCheck, upperLetters);
-    Command *pCmd = new PlayerRackCmd(*m_players[m_currPlayer], newRack);
-    pCmd->setAutoExecution(false);
-    accessNavigation().addAndExecute(pCmd);
+    Command *pCmd1 = new GameRackCmd(*this, newRack);
+    pCmd1->setHumanIndependent(false);
+    accessNavigation().addAndExecute(pCmd1);
+    Command *pCmd2 = new PlayerRackCmd(*m_players[m_currPlayer], newRack);
+    pCmd2->setHumanIndependent(false);
+    accessNavigation().addAndExecute(pCmd2);
     // Clear the results if everything went well
     m_results.clear();
 }
@@ -89,7 +97,7 @@ int Training::play(const wstring &iCoord, const wstring &iWord)
     // Perform all the validity checks, and fill a round
     Round round;
 
-    removeTestPlay();
+    m_board.removeTestRound();
 
     int res = checkPlayedWord(iCoord, iWord, round);
     if (res != 0)
@@ -115,19 +123,13 @@ void Training::recordPlayerMove(const Move &iMove, Player &ioPlayer)
     // (called in this class in endTurn()).
     // See the big comment in game.cpp, line 96
     Command *pCmd = new PlayerMoveCmd(ioPlayer, iMove);
-    pCmd->setAutoExecution(false);
     accessNavigation().addAndExecute(pCmd);
 }
 
 
 void Training::start()
 {
-    firstPlayer();
-    // Dummy new turn, because the navigation prevents undoing the first turn.
-    // Since in this mode the player can set the rack, we cannot do like in the
-    // duplicate and free game modes, where we change turn just before a move
-    // is played...
-    accessNavigation().newTurn();
+    // Nothing to do
 }
 
 
@@ -154,11 +156,10 @@ void Training::endTurn()
 void Training::search()
 {
     // Search for the current player
-    Rack r;
-    m_players[m_currPlayer]->getCurrentRack().getRack(r);
+    const Rack &rack = getHistory().getCurrentRack().getRack();
     int limit = Settings::Instance().getInt("training.search-limit");
     m_results.setLimit(limit);
-    m_results.search(getDic(), getBoard(), r, getHistory().beforeFirstRound());
+    m_results.search(getDic(), getBoard(), rack, getHistory().beforeFirstRound());
 }
 
 
@@ -187,24 +188,3 @@ void Training::addPlayer(Player *iPlayer)
     Game::addPlayer(iPlayer);
 }
 
-
-void Training::testPlay(unsigned int num)
-{
-    ASSERT(num < m_results.size(), "Wrong result number");
-    m_testRound = m_results.get(num);
-    m_board.testRound(m_results.get(num));
-}
-
-
-void Training::removeTestPlay()
-{
-    m_board.removeTestRound();
-    m_testRound = Round();
-}
-
-
-wstring Training::getTestPlayWord() const
-{
-    return m_testRound.getWord();
-}
-
