@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2009 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -25,7 +25,6 @@
 #include <string>
 
 #include "game.h"
-#include "round.h"
 #include "results.h"
 #include "logging.h"
 
@@ -77,24 +76,12 @@ public:
      */
     void setRackManual(bool iCheck, const wstring &iLetters);
 
-    /*************************
+    /**
      * Override the default behaviour of addPlayer(), because in training
      * mode we only want a human player
-     *************************/
+     */
     virtual void addPlayer(Player *iPlayer);
 
-    /*************************
-     * Functions to access the current search results
-     * The int parameter should be 0 <= int < getNResults
-     *************************/
-
-    /// Place a temporary word on the board for preview purposes
-    void testPlay(unsigned int iResultIndex);
-    /// Remove the temporary word
-    void removeTestPlay();
-    /// Get the temporary word
-    wstring getTestPlayWord() const;
-
 private:
     /// Private constructor and destructor to force using the GameFactory class
     Training(const GameParams &iParams);
@@ -106,9 +93,6 @@ private:
 
     /// Search results, with all the possible rounds up to a predefined limit
     LimitResults m_results;
-
-    /// Round corresponding to the last test play (if any)
-    Round m_testRound;
 };
 
 #endif /* _TRAINING_H_ */
