@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008-2010 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,8 +34,13 @@
 using namespace std;
 
 
+INIT_LOGGER(qt, BoardWidget);
+
+
 BoardWidget::BoardWidget(CoordModel &iCoordModel, QWidget *parent)
-    : QFrame(parent), m_game(NULL), m_coordModel(iCoordModel),
+    : QFrame(parent), m_game(NULL),
+    m_coordModel(iCoordModel), m_showTemporarySigns(true),
+    m_showOnlyLastTurn(false),
     m_widgetsMatrix(BOARD_MAX + 1, BOARD_MAX + 1, 0)
 {
     // Try to have a black background... FIXME: not working well!
@@ -46,21 +51,24 @@ BoardWidget::BoardWidget(CoordModel &iCoordModel, QWidget *parent)
     setForegroundRole(QPalette::Window);
     setBackgroundRole(QPalette::Window);
 
-    TileLayout *layout = new TileLayout(BOARD_MAX + 1, 1);
+    TileLayout *layout = new TileLayout(BOARD_MAX + 1, BOARD_MAX + 1);
+    layout->setSpacing(1);
     // Line full of coordinates
-    layout->addWidget(new BasicTileWidget(this, ""));
+    TileWidget *cornerTile = new TileWidget;
+    cornerTile->setCoordText("");
+    layout->addWidget(cornerTile);
     for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
     {
-        BasicTileWidget *coordTile =
-            new BasicTileWidget(this, QString("%1").arg(col));
+        TileWidget *coordTile = new TileWidget;
+        coordTile->setCoordText(QString("%1").arg(col));
         layout->addWidget(coordTile);
     }
     // Rest of the board
     for (unsigned int row = BOARD_MIN; row <= BOARD_MAX; ++row)
     {
         // Add the coordinate
-        BasicTileWidget *coordTile =
-            new BasicTileWidget(this, QString(QChar('A' + row - BOARD_MIN)));
+        TileWidget *coordTile = new TileWidget;
+        coordTile->setCoordText(QString(QChar('A' + row - BOARD_MIN)));
         layout->addWidget(coordTile);
         // Add the squares
         for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
@@ -110,6 +118,10 @@ void BoardWidget::updateArrow(const Coord &iOldCoord, const Coord &iNewCoord)
         TileWidget *t = m_widgetsMatrix[iOldCoord.getRow()][iOldCoord.getCol()];
         t->arrowChanged(false, false);
     }
+
+    if (!m_showTemporarySigns)
+        return;
+
     if (iNewCoord.isValid())
     {
         TileWidget *t = m_widgetsMatrix[iNewCoord.getRow()][iNewCoord.getCol()];
@@ -122,19 +134,37 @@ void BoardWidget::refresh()
 {
     if (m_game != NULL)
     {
-        // XXX: in the future, this code could be changed to use signals
-        // emitted from the core. This would allow repainting only the needed
-        // tiles (the same performance improvement could be done with caching
-        // in the TileWidget class, though)
+        if (m_showOnlyLastTurn && !m_game->isLastTurn())
+            return;
+        // Note: the TileWidget class will redraw the tile only if something
+        // has changed, to avoid useless repainting.
         const Board &board = m_game->getBoard();
         for (unsigned int row = BOARD_MIN; row <= BOARD_MAX; ++row)
         {
             for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
             {
-                m_widgetsMatrix[row][col]->tileChanged(
-                        board.getTile(row, col),
-                        board.isJoker(row, col),
-                        board.isTestChar(row, col) ?  TileWidget::PREVIEW : TileWidget::NORMAL);
+                if (board.isTestChar(row, col) && m_showTemporarySigns)
+                {
+                    const Tile &tile = board.getTestTile(row, col);
+                    m_widgetsMatrix[row][col]->tileChanged(
+                                TileWidget::PREVIEW,
+                                tile, tile.isJoker());
+                }
+                else
+                {
+                    if (board.isVacant(row, col))
+                    {
+                        // Force an empty square.
+                        m_widgetsMatrix[row][col]->tileChanged(TileWidget::BOARD_EMPTY);
+                    }
+                    else
+                    {
+                        m_widgetsMatrix[row][col]->tileChanged(
+                                TileWidget::NORMAL,
+                                board.getTile(row, col),
+                                board.isJoker(row, col));
+                    }
+                }
             }
         }
     }
@@ -145,8 +175,7 @@ void BoardWidget::refresh()
         {
             for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
             {
-                m_widgetsMatrix[row][col]->tileChanged(
-                        Tile(), false, TileWidget::NORMAL);
+                m_widgetsMatrix[row][col]->tileChanged(TileWidget::BOARD_EMPTY);
             }
         }
     }
@@ -166,7 +195,7 @@ void BoardWidget::paintEvent(QPaintEvent *)
     QPainter painter(this);
     QRect rect = boardLayout->getBoardRect();
     const int size = boardLayout->getSquareSize();
-    const int spacing = boardLayout->getSpacing();
+    const int spacing = boardLayout->spacing();
 
     QLine hLine(0, 0, rect.width() + 1, 0);
     QLine vLine(0, 0, 0, rect.height() + 1);
