@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2009 Olivier Teulière
+ * Copyright (C) 2009-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,11 +21,14 @@
 #include "coord_model.h"
 
 
+INIT_LOGGER(qt, CoordModel);
+
+
 void CoordModel::setCoord(const Coord &iCoord)
 {
     m_prevCoord = m_currCoord;
     m_currCoord = iCoord;
-    emit coordChanged(iCoord);
+    emit coordChanged(m_prevCoord, iCoord);
 }
 
 
