@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2009 Olivier Teulière
+ * Copyright (C) 2009-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,12 +22,15 @@
 #define COORD_MODEL_H_
 
 #include <QObject>
+
 #include "coord.h"
+#include "logging.h"
 
 
 class CoordModel: public QObject
 {
     Q_OBJECT;
+    DEFINE_LOGGER();
 
 public:
     void setCoord(const Coord &iCoord);
