@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008-2009 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,48 +23,28 @@
 #include <map>
 #include <vector>
 #include <algorithm>
+#include <fstream>
 #include <QtGui/QTreeView>
 #include <QtGui/QStandardItemModel>
-#include <QtGui/QVBoxLayout>
 #include <QtGui/QLineEdit>
-#include <QtGui/QToolTip>
+#include <QtGui/QFileDialog>
+#include <QtGui/QMessageBox>
+#include <QtGui/QMenu>
 #include <QtCore/QString>
 
 #include "dic_tools_widget.h"
+#include "custom_popup.h"
+#include "validator_factory.h"
 #include "qtcommon.h"
 #include "dic.h"
 #include "header.h"
+#include "listdic.h"
 #include "dic_exception.h"
 
 using namespace std;
 
 
-/// Validator used for the line edits accepting only dictionary characters
-class DicRackValidator: public QValidator
-{
-public:
-    explicit DicRackValidator(QObject *parent,
-                              const Dictionary *iDic,
-                              bool acceptJoker = false);
-    virtual State validate(QString &input, int &pos) const;
-
-private:
-    const Dictionary *m_dic;
-    const bool m_acceptJoker;
-};
-
-
-/// Validator used for the regexp line edit
-class RegexpValidator: public QValidator
-{
-public:
-    explicit RegexpValidator(QObject *parent,
-                             const Dictionary *iDic);
-    virtual State validate(QString &input, int &pos) const;
-
-private:
-    const Dictionary *m_dic;
-};
+INIT_LOGGER(qt, DicToolsWidget);
 
 
 DicToolsWidget::DicToolsWidget(QWidget *parent)
@@ -82,9 +62,37 @@ DicToolsWidget::DicToolsWidget(QWidget *parent)
     labelLimitReached->hide();
 
     // Create connections
-    QObject::connect(lineEditCheck, SIGNAL(textChanged(const QString&)), this, SLOT(refreshCheck()));
-    QObject::connect(lineEditPlus1, SIGNAL(returnPressed()), this, SLOT(refreshPlus1()));
-    QObject::connect(lineEditRegexp, SIGNAL(returnPressed()), this, SLOT(refreshRegexp()));
+    QObject::connect(lineEditCheck, SIGNAL(textChanged(const QString&)),
+                     this, SLOT(refreshCheck()));
+    QObject::connect(lineEditPlus1, SIGNAL(textChanged(const QString&)),
+                     this, SLOT(enableSearchPlus1()));
+    QObject::connect(lineEditPlus1, SIGNAL(returnPressed()),
+                     this, SLOT(refreshPlus1()));
+    QObject::connect(buttonSearchPlus1, SIGNAL(clicked()),
+                     this, SLOT(refreshPlus1()));
+    QObject::connect(lineEditRegexp, SIGNAL(textChanged(const QString&)),
+                     this, SLOT(enableSearchRegexp()));
+    QObject::connect(lineEditRegexp, SIGNAL(returnPressed()),
+                     this, SLOT(refreshRegexp()));
+    QObject::connect(buttonSearchRegexp, SIGNAL(clicked()),
+                     this, SLOT(refreshRegexp()));
+    QObject::connect(buttonSaveRegexp, SIGNAL(clicked()),
+                     this, SLOT(saveRegexpResults()));
+    QObject::connect(buttonSaveWords, SIGNAL(clicked()),
+                     this, SLOT(exportWordsList()));
+
+    // Add context menus for the results
+    m_customPopupPlus1 = new CustomPopup(treeViewPlus1);
+    QObject::connect(m_customPopupPlus1, SIGNAL(popupCreated(QMenu&, const QPoint&)),
+                     this, SLOT(populateMenuPlus1(QMenu&, const QPoint&)));
+    QObject::connect(m_customPopupPlus1, SIGNAL(requestDefinition(QString)),
+                     this, SIGNAL(requestDefinition(QString)));
+
+    m_customPopupRegexp = new CustomPopup(treeViewRegexp);
+    QObject::connect(m_customPopupRegexp, SIGNAL(popupCreated(QMenu&, const QPoint&)),
+                     this, SLOT(populateMenuRegexp(QMenu&, const QPoint&)));
+    QObject::connect(m_customPopupRegexp, SIGNAL(requestDefinition(QString)),
+                     this, SIGNAL(requestDefinition(QString)));
 
     // Create models
     m_plus1Model = new QStandardItemModel(this);
@@ -119,9 +127,9 @@ void DicToolsWidget::setDic(const Dictionary *iDic)
         lineEditPlus1->clear();
         lineEditRegexp->clear();
         // Create new validators
-        lineEditCheck->setValidator(new DicRackValidator(this, m_dic));
-        lineEditPlus1->setValidator(new DicRackValidator(this, m_dic, true));
-        lineEditRegexp->setValidator(new RegexpValidator(this, m_dic));
+        lineEditCheck->setValidator(ValidatorFactory::newDicRackValidator(this, m_dic));
+        lineEditPlus1->setValidator(ValidatorFactory::newDicRackValidator(this, m_dic, true));
+        lineEditRegexp->setValidator(ValidatorFactory::newRegexpValidator(this, m_dic));
         // Refresh
         refreshCheck();
         refreshPlus1();
@@ -158,7 +166,7 @@ void DicToolsWidget::refreshCheck()
             return;
         }
 
-        wstring input = m_dic->convertFromInput(qtw(rack->text()));
+        wstring input = m_dic->convertFromInput(wfq(rack->text()));
         bool res = m_dic->searchWord(input);
         // Convert the input to uppercase
         std::transform(input.begin(), input.end(), input.begin(), towupper);
@@ -177,6 +185,12 @@ void DicToolsWidget::refreshCheck()
 }
 
 
+void DicToolsWidget::enableSearchPlus1()
+{
+    buttonSearchPlus1->setEnabled(lineEditPlus1->text() != "");
+}
+
+
 void DicToolsWidget::refreshPlus1()
 {
     QStandardItemModel *model = m_plus1Model;
@@ -192,7 +206,7 @@ void DicToolsWidget::refreshPlus1()
         return;
     }
 
-    const wstring &input = m_dic->convertFromInput(qtw(rack->text().toUpper()));
+    const wstring &input = m_dic->convertFromInput(wfq(rack->text().toUpper()));
     const wdstring &disp = m_dic->convertToDisplay(input);
     model->setHeaderData(0, Qt::Horizontal,
                          _q("Rack: %1").arg(qfw(disp)),
@@ -231,6 +245,14 @@ void DicToolsWidget::refreshPlus1()
 }
 
 
+void DicToolsWidget::enableSearchRegexp()
+{
+    bool hasText = lineEditRegexp->text() != "";
+    buttonSearchRegexp->setEnabled(hasText);
+    buttonSaveRegexp->setEnabled(m_dic != NULL && hasText);
+}
+
+
 void DicToolsWidget::refreshRegexp()
 {
     QStandardItemModel *model = m_regexpModel;
@@ -245,7 +267,7 @@ void DicToolsWidget::refreshRegexp()
         return;
     }
 
-    const wstring &input = m_dic->convertFromInput(qtw(rack->text().toUpper()));
+    const wstring &input = m_dic->convertFromInput(wfq(rack->text().toUpper()));
     const wdstring &disp = m_dic->convertToDisplay(input);
     model->setHeaderData(0, Qt::Horizontal,
                          _q("Regular expression: %1").arg(qfw(disp)),
@@ -289,6 +311,47 @@ void DicToolsWidget::refreshRegexp()
 }
 
 
+void DicToolsWidget::saveRegexpResults()
+{
+    if (m_dic == NULL)
+        return;
+
+    QString fileName = QFileDialog::getSaveFileName(this, _q("Save words list"));
+    if (fileName != "")
+    {
+        const wstring &input = m_dic->convertFromInput(wfq(lineEditRegexp->text().toUpper()));
+        unsigned lmin = spinBoxMinLength->value();
+        unsigned lmax = spinBoxMaxLength->value();
+        vector<wstring> wordList;
+        try
+        {
+            m_dic->searchRegExp(input, wordList, lmin, lmax, 0);
+        }
+        catch (InvalidRegexpException &e)
+        {
+            QMessageBox::warning(this, _q("Eliot - Error"),
+                                 _q("Invalid regular expression: %1").arg(qfl(e.what())));
+            return;
+        }
+        try
+        {
+            ofstream file(lfq(fileName).c_str());
+            foreach (const wstring &word, wordList)
+            {
+                file << lfw(word) << endl;
+            }
+            QMessageBox::information(this, _q("Save words list"),
+                                     _q("File '%1' successfully saved").arg(fileName));
+        }
+        catch (std::exception &e)
+        {
+            QMessageBox::warning(this, _q("Eliot - Error"),
+                                 _q("Cannot save the words list: %1").arg(e.what()));
+        }
+    }
+}
+
+
 void DicToolsWidget::refreshDicInfo()
 {
     if (m_dic == NULL)
@@ -345,75 +408,53 @@ void DicToolsWidget::refreshDicInfo()
 }
 
 
-
-DicRackValidator::DicRackValidator(QObject *parent,
-                                   const Dictionary *iDic,
-                                   bool acceptJoker)
-    : QValidator(parent), m_dic(iDic), m_acceptJoker(acceptJoker)
-{
-}
-
-
-QValidator::State DicRackValidator::validate(QString &input, int &) const
+void DicToolsWidget::exportWordsList()
 {
     if (m_dic == NULL)
-        return Invalid;
-
-    if (input == "")
-        return Intermediate;
-
-    // The string is invalid if it contains invalid input characters
-    const wistring &winput = qtw(input);
-    if (!m_dic->validateInputChars(winput))
-        return Invalid;
-
-    // Convert the string to internal letters
-    const wstring &intInput = m_dic->convertFromInput(winput);
-    // The string is invalid if it contains characters not present
-    // in the dictionary
-    if (!m_dic->validateLetters(intInput))
-        return Intermediate;
-
-    // A '?' may not be acceptable
-    if (!m_acceptJoker && input.contains('?'))
-        return Invalid;
-    // Do not accept more than 2 jokers
-    if (input.count('?') > 2)
-        return Invalid;
-    return Acceptable;
+        return;
+    QString fileName = QFileDialog::getSaveFileName(this, _q("Save words list"));
+    if (fileName != "")
+    {
+        try
+        {
+            ofstream file(lfq(fileName).c_str());
+            ListDic::printWords(file, *m_dic);
+            QMessageBox::information(this, _q("Save words list"),
+                                     _q("File '%1' successfully saved").arg(fileName));
+        }
+        catch (std::exception &e)
+        {
+            QMessageBox::warning(this, _q("Eliot - Error"),
+                                 _q("Cannot save the words list: %1").arg(e.what()));
+        }
+    }
 }
 
 
-
-RegexpValidator::RegexpValidator(QObject *parent,
-                                 const Dictionary *iDic)
-    : QValidator(parent), m_dic(iDic)
+void DicToolsWidget::populateMenuPlus1(QMenu &iMenu, const QPoint &iPoint)
 {
+    const QModelIndex &index = treeViewPlus1->indexAt(iPoint);
+    if (!index.isValid() || !index.parent().isValid())
+        return;
+
+    // Find the selected word
+    const QModelIndex &wordIndex = m_plus1Model->index(index.row(), 0, index.parent());
+    QString selectedWord = m_plus1Model->data(wordIndex).toString();
+
+    iMenu.addAction(m_customPopupPlus1->getShowDefinitionEntry(selectedWord));
 }
 
 
-QValidator::State RegexpValidator::validate(QString &input, int &) const
+void DicToolsWidget::populateMenuRegexp(QMenu &iMenu, const QPoint &iPoint)
 {
-    if (m_dic == NULL)
-        return Invalid;
+    const QModelIndex &index = treeViewRegexp->indexAt(iPoint);
+    if (!index.isValid())
+        return;
 
-    if (input == "")
-        return Intermediate;
+    // Find the selected word
+    const QModelIndex &wordIndex = m_regexpModel->index(index.row(), 0);
+    QString selectedWord = m_regexpModel->data(wordIndex).toString();
 
-    wstring authorizedChars = L".[]()*+?:^";
-
-    // The string is invalid if it contains invalid input characters
-    const wistring &winput = qtw(input);
-    if (!m_dic->validateInputChars(winput, authorizedChars))
-        return Invalid;
-
-    // Convert the string to internal letters
-    const wstring &intInput = m_dic->convertFromInput(winput);
-    // The string is invalid if it contains characters not present
-    // in the dictionary
-    if (!m_dic->validateLetters(intInput, authorizedChars))
-        return Intermediate;
-
-    return Acceptable;
+    iMenu.addAction(m_customPopupRegexp->getShowDefinitionEntry(selectedWord));
 }
 
