@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include <iostream>
 #include <string>
 #include <exception>
 #include <QApplication>
@@ -44,7 +45,9 @@
 using namespace std;
 
 
+#ifdef HAVE_EXECINFO_H
 static void bt_sighandler(int);
+#endif
 
 // Custom QApplication to catch and log exceptions properly
 // See http://forum.qtfr.org/viewtopic.php?id=7615
@@ -119,6 +122,9 @@ int main(int argc, char **argv)
 
     MyApplication app(argc, argv);
     app.setWindowIcon(QIcon(":/images/eliot.xpm"));
+    // Used for QSettings
+    app.setApplicationName(PACKAGE_NAME);
+    app.setOrganizationName("eliot");
 
 #ifdef ENABLE_NLS
     // Set the message domain
@@ -157,9 +163,25 @@ int main(int argc, char **argv)
     app.installTranslator(&translator);
 #endif
 
-    MainWindow qmain;
-    qmain.show();
-    return app.exec();
+    try
+    {
+        MainWindow qmain;
+        qmain.show();
+        return app.exec();
+    }
+    catch (const BaseException &e)
+    {
+        cerr << "Exception caught: " << e.what() << "\n" << e.getStackTrace();
+    }
+    catch (const std::exception &e)
+    {
+        cerr << "Exception caught: " << e.what();
+    }
+    catch (...)
+    {
+        cerr << "Unknown exception caught";
+    }
+    return 1;
 }
 
 #ifdef HAVE_EXECINFO_H
