@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008-2010 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,8 +20,8 @@
 
 #include "config.h"
 
-//#include <iostream>
 #include <QtGui/QLabel>
+#include <QtGui/QFrame>
 #include <QtGui/QMessageBox>
 #include <QtGui/QFileDialog>
 #include <QtGui/QDockWidget>
@@ -43,39 +43,54 @@
 #include "public_game.h"
 #include "player.h"
 #include "history.h"
-#include "turn.h"
+#include "turn_data.h"
 #include "move.h"
 #include "debug.h"
+#include "round.h"
+#include "settings.h"
+
 #include "new_game.h"
+#include "tables_dialog.h"
 #include "prefs_dialog.h"
 #include "bag_widget.h"
 #include "board_widget.h"
+#include "rack_widget.h"
 #include "score_widget.h"
 #include "player_widget.h"
+#include "training_widget.h"
+#include "arbitration_widget.h"
 #include "history_widget.h"
+#include "stats_widget.h"
 #include "dic_tools_widget.h"
+#include "players_table_helper.h"
+#include "fav_players.h"
+#include "timer_widget.h"
 #include "dic_wizard.h"
 #include "aux_window.h"
 #include "qtcommon.h"
 
-#include "round.h"
-#include "coord.h"
-
 
 INIT_LOGGER(qt, MainWindow);
 
 const char *MainWindow::m_windowName = "MainWindow";
 
 MainWindow::MainWindow(QWidget *iParent)
-    : QMainWindow(iParent), m_dic(NULL), m_game(NULL), m_newGameDialog(NULL),
-    m_prefsDialog(NULL), m_bagWindow(NULL), m_boardWindow(NULL),
-    m_historyWindow(NULL), m_dicToolsWindow(NULL), m_dicNameLabel(NULL)
+    : QMainWindow(iParent), m_dic(NULL), m_game(NULL),
+    m_playersWidget(NULL), m_trainingWidget(NULL),
+    m_arbitrationWidget(NULL), m_scoresWidget(NULL),
+    m_bagWindow(NULL), m_boardWindow(NULL),
+    m_historyWindow(NULL), m_statsWindow(NULL), m_timerWindow(NULL),
+    m_dicToolsWindow(NULL), m_dicNameLabel(NULL), m_timerModel(NULL),
+    m_currentTurn(0)
 {
 #ifdef DEBUG
     // Check that the string conversion routines are not buggy
-    checkConversions();
+    QtCommon::CheckConversions();
 #endif
 
+    // Path to the auto-saved game
+    m_autoSaveGame = Settings::GetConfigFileDir() + "autosave.xml";
+
     LOG_DEBUG("Creating main window");
     m_ui.setupUi(this);
     createMenu();
@@ -93,15 +108,30 @@ MainWindow::MainWindow(QWidget *iParent)
     // Make it easier to reproduce bugs
     LOG_DEBUG("Rand seed: " << val);
 
-    QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
-                     this, SLOT(updateForGame(const PublicGame*)));
+    QSettings qs;
+    int timerTotal = qs.value(PrefsDialog::kINTF_TIMER_TOTAL_DURATION, 180).toInt();
+    int timerAlert = qs.value(PrefsDialog::kINTF_TIMER_ALERT_DURATION, 30).toInt();
+    m_timerModel = new TimerModel(timerTotal, timerAlert);
+    QObject::connect(m_timerModel, SIGNAL(alert(int)),
+                     this, SLOT(beep()));
+    QObject::connect(m_timerModel, SIGNAL(expired()),
+                     this, SLOT(beep()));
+
+    QObject::connect(this, SIGNAL(gameChangedNonConst(PublicGame*)),
+                     this, SLOT(updateForGame(PublicGame*)));
     QObject::connect(this, SIGNAL(gameUpdated()),
                      this, SLOT(refresh()));
-    refresh();
 
     // Status bar
     statusBar()->addWidget(new QLabel, 1);
-    m_dicNameLabel = new QLabel;
+    // First widget, not added yet
+    m_lettersLabel = new QLabel(statusBar());
+    m_lettersLabel->setFrameStyle(QFrame::Sunken | QFrame::Panel);
+    // Second widget, not added yet
+    m_turnLabel = new QLabel(statusBar());
+    m_turnLabel->setFrameStyle(QFrame::Sunken | QFrame::Panel);
+    // Third widget
+    m_dicNameLabel = new QLabel(this);
     m_dicNameLabel->setFrameStyle(QFrame::Sunken | QFrame::Panel);
     statusBar()->addPermanentWidget(m_dicNameLabel);
     QObject::connect(this, SIGNAL(dicChanged(const Dictionary*)),
@@ -129,47 +159,27 @@ MainWindow::MainWindow(QWidget *iParent)
 
 #if 1
     // History
-    HistoryTabWidget *historyTab = new HistoryTabWidget;
+    m_historyTabWidget = new HistoryTabWidget;
     QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
-                     historyTab, SLOT(setGame(const PublicGame*)));
+                     m_historyTabWidget, SLOT(setGame(const PublicGame*)));
     QObject::connect(this, SIGNAL(gameUpdated()),
-                     historyTab, SLOT(refresh()));
-    QObject::connect(historyTab, SIGNAL(requestDefinition(QString)),
+                     m_historyTabWidget, SLOT(refresh()));
+    QObject::connect(m_historyTabWidget, SIGNAL(requestDefinition(QString)),
                      this, SLOT(showDefinition(QString)));
     QHBoxLayout *hlayout2 = new QHBoxLayout;
-    hlayout2->addWidget(historyTab);
+    hlayout2->addWidget(m_historyTabWidget);
     m_ui.groupBoxHistory->setLayout(hlayout2);
 #else
     m_ui.groupBoxHistory->hide();
 #endif
 
-    // Players racks
+    // Hide the players group box
     m_ui.groupBoxPlayers->hide();
-    m_playersWidget = new PlayerTabWidget(m_coordModel, NULL);
-    m_ui.groupBoxPlayers->layout()->addWidget(m_playersWidget);
-    QObject::connect(this, SIGNAL(gameChangedNonConst(PublicGame*)),
-                     m_playersWidget, SLOT(setGame(PublicGame*)));
-    QObject::connect(this, SIGNAL(gameUpdated()), m_playersWidget, SLOT(refresh()));
-
-    QObject::connect(m_playersWidget, SIGNAL(gameUpdated()), this, SIGNAL(gameUpdated()));
-    QObject::connect(m_playersWidget, SIGNAL(notifyProblem(QString)),
-                     this, SLOT(displayErrorMsg(QString)));
-    QObject::connect(m_playersWidget, SIGNAL(requestDefinition(QString)),
-                     this, SLOT(showDefinition(QString)));
-
-    // Players score
-    ScoreWidget *scores = new ScoreWidget;
-    QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
-                     scores, SLOT(setGame(const PublicGame*)));
-    QObject::connect(this, SIGNAL(gameUpdated()),
-                     scores, SLOT(refresh()));
-    m_ui.groupBoxPlayers->layout()->addWidget(scores);
 
     emit gameChangedNonConst(NULL);
     emit gameChanged(NULL);
 
     // Load dictionary
-    QSettings qs(ORGANIZATION, PACKAGE_NAME);
     QString dicPath = qs.value(PrefsDialog::kINTF_DIC_PATH, "").toString();
     if (dicPath != "")
     {
@@ -178,7 +188,7 @@ MainWindow::MainWindow(QWidget *iParent)
         {
             m_dic = new Dictionary(lfq(dicPath));
         }
-        catch (DicException &e)
+        catch (const std::exception &e)
         {
             displayErrorMsg(_q("Cannot load dictionary '%1' indicated in the "
                                "preferences.\nReason: %2").arg(dicPath).arg(e.what()));
@@ -194,9 +204,16 @@ MainWindow::~MainWindow()
     delete m_bagWindow;
     delete m_boardWindow;
     delete m_historyWindow;
+    delete m_statsWindow;
+    delete m_timerWindow;
     delete m_dicToolsWindow;
     delete m_game;
     delete m_dic;
+    delete m_timerModel;
+
+    // Destroy singletons
+    GameFactory::Destroy();
+    Settings::Destroy();
 }
 
 
@@ -225,6 +242,15 @@ void MainWindow::refresh()
 {
     if (m_game != NULL)
     {
+        const Bag &bag = m_game->getBag();
+        m_lettersLabel->setText(_q("Consonants: %1 | Vowels: %2 | Jokers: %3")
+                                .arg(bag.getNbConsonants())
+                                .arg(bag.getNbVowels())
+                                .arg(bag.in(Tile::Joker())));
+        unsigned currTurn = m_game->getCurrTurn();
+        m_turnLabel->setText(_q("Turn %1/%2")
+                             .arg(currTurn)
+                             .arg(m_game->getNbTurns()));
         bool isFirstTurn = m_game->isFirstTurn();
         bool isLastTurn = m_game->isLastTurn();
         m_actionHistoryFirstTurn->setEnabled(!isFirstTurn);
@@ -234,37 +260,76 @@ void MainWindow::refresh()
         m_actionHistoryReplayTurn->setEnabled(!isLastTurn);
         if (m_game->isFinished())
             displayInfoMsg(_q("End of the game"));
+        // Emit the turnChanged() sign if needed.
+        if (currTurn != m_currentTurn)
+        {
+            m_currentTurn = currTurn;
+            emit turnChanged(currTurn, isLastTurn);
+        }
+
+        // Update the auto-saved game
+        try
+        {
+            m_game->save(m_autoSaveGame);
+        }
+        catch (const std::exception &e)
+        {
+            LOG_ERROR("Error during auto-save: " << e.what());
+            displayErrorMsg(_q("Error during auto-save of the game: %1").arg(e.what()));
+        }
+
 #ifdef DEBUG
-        //m_game->printTurns();
+        m_game->printTurns();
 #endif
     }
 }
 
 
+void MainWindow::beep()
+{
+    QSettings qs;
+    if (!qs.value(PrefsDialog::kINTF_TIMER_BEEPS, true).toBool())
+        return;
+
+    LOG_DEBUG("Beep!")
+    // FIXME: doesn't seem to be working on Ubuntu
+    // (even acter 'modprobe pcspkr' and when the 'beep' application works)
+    QApplication::beep();
+}
+
+
+void MainWindow::linkArbitrationAnd7P1()
+{
+    if (m_arbitrationWidget == NULL || m_dicToolsWindow == NULL)
+        return;
+
+    // Disconnect the arbitration rack updates from the "Plus 1" tab of the
+    // dictionary tools
+    m_arbitrationWidget->disconnect(SIGNAL(rackUpdated(const QString&)));
+    // Reconnect it only if needed
+    QSettings qs;
+    if (qs.value(PrefsDialog::kARBIT_LINK_7P1, false).toBool())
+    {
+        QObject::connect(m_arbitrationWidget,
+                         SIGNAL(rackUpdated(const QString&)),
+                         &m_dicToolsWindow->getWidget(),
+                         SLOT(setPlus1Rack(const QString&)));
+    }
+}
+
+
 void MainWindow::prefsUpdated()
 {
     LOG_DEBUG("Preferences updated");
-    // Disconnect the training rack updates from the "Plus 1" tab of the
-    // dictionary tools
-    m_playersWidget->disconnect(SIGNAL(trainingRackUpdated(const QString&)));
-    // Reconnect it only if needed
-    if (m_dicToolsWindow != NULL)
-    {
-        QSettings qs(ORGANIZATION, PACKAGE_NAME);
-        if (qs.value(PrefsDialog::kINTF_LINK_TRAINING_7P1, false).toBool())
-        {
-            QObject::connect(m_playersWidget,
-                             SIGNAL(trainingRackUpdated(const QString&)),
-                             &m_dicToolsWindow->getWidget(),
-                             SLOT(setPlus1Rack(const QString&)));
-        }
-    }
+    // Refresh one signal/slot connection
+    linkArbitrationAnd7P1();
 
-    // Refresh the default level for the Eliot player
-    if (m_newGameDialog != NULL)
-    {
-        m_newGameDialog->refresh();
-    }
+    // Refresh the timer values
+    QSettings qs;
+    int timerTotal = qs.value(PrefsDialog::kINTF_TIMER_TOTAL_DURATION).toInt();
+    int timerAlert = qs.value(PrefsDialog::kINTF_TIMER_ALERT_DURATION).toInt();
+    m_timerModel->setTotalDuration(timerTotal);
+    m_timerModel->setAlertDuration(timerAlert);
 
     // Probably useless in most cases (currently only used for
     // the History alignment)
@@ -272,7 +337,7 @@ void MainWindow::prefsUpdated()
 }
 
 
-void MainWindow::updateForGame(const PublicGame *iGame)
+void MainWindow::updateForGame(PublicGame *iGame)
 {
     if (iGame == NULL)
     {
@@ -283,23 +348,116 @@ void MainWindow::updateForGame(const PublicGame *iGame)
         m_actionHistoryNextTurn->setEnabled(false);
         m_actionHistoryLastTurn->setEnabled(false);
         m_actionHistoryReplayTurn->setEnabled(false);
+        m_actionSettingsDefineTables->setEnabled(false);
         setWindowTitle(_q("No game") + " - Eliot");
+        statusBar()->removeWidget(m_lettersLabel);
+        statusBar()->removeWidget(m_turnLabel);
+
+        // Destroy the players widget
+        QtCommon::DestroyObject(m_playersWidget, this);
+        m_playersWidget = NULL;
+
+        // Destroy the training widget
+        QtCommon::DestroyObject(m_trainingWidget, this);
+        m_trainingWidget = NULL;
+
+        // Destroy the arbitration widget
+        QtCommon::DestroyObject(m_arbitrationWidget, this);
+        m_arbitrationWidget = NULL;
+
+        // Destroy the scores widget
+        QtCommon::DestroyObject(m_scoresWidget, this);
+        m_scoresWidget = NULL;
     }
     else
     {
         m_actionGamePrint->setEnabled(true);
         m_actionGameSaveAs->setEnabled(true);
+        m_actionSettingsDefineTables->setEnabled(iGame->getMode() == PublicGame::kARBITRATION);
+        statusBar()->addWidget(m_lettersLabel);
+        m_lettersLabel->show();
+        statusBar()->addWidget(m_turnLabel);
+        m_turnLabel->show();
+
         if (iGame->getMode() == PublicGame::kTRAINING)
         {
             setWindowTitle(_q("Training mode") + " - Eliot");
+            m_ui.groupBoxPlayers->setTitle(_q("Training"));
+
+            // Training widget
+            m_trainingWidget = new TrainingWidget(NULL, m_coordModel, iGame);
+            m_ui.groupBoxPlayers->layout()->addWidget(m_trainingWidget);
+            QObject::connect(m_trainingWidget, SIGNAL(gameUpdated()),
+                             this, SIGNAL(gameUpdated()));
+            QObject::connect(m_trainingWidget, SIGNAL(notifyInfo(QString)),
+                             this, SLOT(displayInfoMsg(QString)));
+            QObject::connect(m_trainingWidget, SIGNAL(notifyProblem(QString)),
+                             this, SLOT(displayErrorMsg(QString)));
+            QObject::connect(m_trainingWidget, SIGNAL(requestDefinition(QString)),
+                             this, SLOT(showDefinition(QString)));
+            QObject::connect(this, SIGNAL(gameUpdated()),
+                             m_trainingWidget, SLOT(refresh()));
+
+            // Players score
+            m_scoresWidget = new ScoreWidget(NULL, iGame);
+            m_ui.groupBoxPlayers->layout()->addWidget(m_scoresWidget);
+            QObject::connect(this, SIGNAL(gameUpdated()),
+                             m_scoresWidget, SLOT(refresh()));
         }
-        else if (iGame->getMode() == PublicGame::kDUPLICATE)
+        else if (iGame->getMode() == PublicGame::kARBITRATION)
         {
-            setWindowTitle(_q("Duplicate game") + " - Eliot");
+            setWindowTitle(_q("Arbitration game") + " - Eliot");
+            m_ui.groupBoxPlayers->setTitle(_q("Arbitration"));
+
+            // Note: we specify the parent immediately, otherwise the
+            // reparenting creates focus issues
+            m_arbitrationWidget = new ArbitrationWidget(m_ui.groupBoxPlayers, iGame, m_coordModel);
+            m_ui.groupBoxPlayers->layout()->addWidget(m_arbitrationWidget);
+            QObject::connect(m_arbitrationWidget, SIGNAL(gameUpdated()),
+                             this, SIGNAL(gameUpdated()));
+            QObject::connect(m_arbitrationWidget, SIGNAL(notifyInfo(QString)),
+                             this, SLOT(displayInfoMsg(QString)));
+            QObject::connect(m_arbitrationWidget, SIGNAL(notifyProblem(QString)),
+                             this, SLOT(displayErrorMsg(QString)));
+            QObject::connect(m_arbitrationWidget, SIGNAL(requestDefinition(QString)),
+                             this, SLOT(showDefinition(QString)));
+            QObject::connect(this, SIGNAL(gameUpdated()),
+                             m_arbitrationWidget, SLOT(refresh()));
+            // Connect with the dictionary tools only if needed
+            linkArbitrationAnd7P1();
+
+            // When a player is selected, show his history
+            QObject::connect(m_arbitrationWidget, SIGNAL(playerSelected(unsigned)),
+                             this, SLOT(onPlayerSelected(unsigned)));
         }
         else
         {
-            setWindowTitle(_q("Free game") + " - Eliot");
+            if (iGame->getMode() == PublicGame::kDUPLICATE)
+                setWindowTitle(_q("Duplicate game") + " - Eliot");
+            else
+                setWindowTitle(_q("Free game") + " - Eliot");
+            m_ui.groupBoxPlayers->setTitle(_q("Players"));
+
+            // Players widget
+            m_playersWidget = new PlayerTabWidget(m_coordModel, NULL);
+            m_ui.groupBoxPlayers->layout()->addWidget(m_playersWidget);
+            QObject::connect(m_playersWidget, SIGNAL(gameUpdated()),
+                             this, SIGNAL(gameUpdated()));
+            QObject::connect(m_playersWidget, SIGNAL(notifyInfo(QString)),
+                             this, SLOT(displayInfoMsg(QString)));
+            QObject::connect(m_playersWidget, SIGNAL(notifyProblem(QString)),
+                             this, SLOT(displayErrorMsg(QString)));
+            QObject::connect(m_playersWidget, SIGNAL(requestDefinition(QString)),
+                             this, SLOT(showDefinition(QString)));
+            QObject::connect(this, SIGNAL(gameUpdated()),
+                             m_playersWidget, SLOT(refresh()));
+            m_playersWidget->setGame(iGame);
+
+            // Players score
+            m_scoresWidget = new ScoreWidget(NULL, iGame);
+            m_ui.groupBoxPlayers->layout()->addWidget(m_scoresWidget);
+            QObject::connect(this, SIGNAL(gameUpdated()),
+                             m_scoresWidget, SLOT(refresh()));
         }
     }
 }
@@ -330,13 +488,13 @@ void MainWindow::displayErrorMsg(QString iMsg, QString iContext)
 void MainWindow::displayInfoMsg(QString iMsg)
 {
     LOG_INFO("Displayed message: " << lfq(iMsg));
-    statusBar()->showMessage(iMsg);
+    statusBar()->showMessage(iMsg, 2000);
 }
 
 
 void MainWindow::showDefinition(QString iWord)
 {
-    QSettings qs(ORGANIZATION, PACKAGE_NAME);
+    QSettings qs;
     QString url = qs.value(PrefsDialog::kINTF_DEFINITIONS_SITE_URL,
                            PrefsDialog::kDEFAULT_DEF_SITE).toString();
     if (url == "")
@@ -356,8 +514,30 @@ void MainWindow::showDefinition(QString iWord)
 }
 
 
+void MainWindow::onPlayerSelected(unsigned iPlayerId)
+{
+    // Select the corresponding tab in the history widget
+    // Note: we don't do the same for the external history widget...
+    m_historyTabWidget->setCurrentIndex(1 + iPlayerId);
+}
+
+
 void MainWindow::closeEvent(QCloseEvent *event)
 {
+    if (m_game)
+    {
+        QString msg = _q("A game has been started.");
+        QString question = _q("Do you really want to quit?");
+        if (!QtCommon::requestConfirmation(PrefsDialog::kCONFO_QUIT_GAME,
+                                           msg, question, this))
+        {
+            event->ignore();
+            return;
+        }
+    }
+
+    LOG_INFO("Exiting");
+
     // Make sure auxiliary windows don't survive after the main one
     if (m_bagWindow)
         m_bagWindow->close();
@@ -365,6 +545,10 @@ void MainWindow::closeEvent(QCloseEvent *event)
         m_boardWindow->close();
     if (m_historyWindow)
         m_historyWindow->close();
+    if (m_statsWindow)
+        m_statsWindow->close();
+    if (m_timerWindow)
+        m_timerWindow->close();
     if (m_dicToolsWindow)
         m_dicToolsWindow->close();
     writeSettings();
@@ -374,7 +558,7 @@ void MainWindow::closeEvent(QCloseEvent *event)
 
 void MainWindow::writeSettings() const
 {
-    QSettings settings(ORGANIZATION, PACKAGE_NAME);
+    QSettings settings;
     settings.beginGroup(m_windowName);
     settings.setValue("size", size());
     settings.setValue("pos", pos());
@@ -389,7 +573,7 @@ void MainWindow::writeSettings() const
 
 void MainWindow::readSettings()
 {
-    QSettings settings(ORGANIZATION, PACKAGE_NAME);
+    QSettings settings;
     settings.beginGroup(m_windowName);
     QSize size = settings.value("size").toSize();
     if (size.isValid())
@@ -408,8 +592,11 @@ void MainWindow::changeDictionary(QString iFileName)
         if (m_game)
         {
             QString msg = _q("Loading a dictionary will stop the current game.");
-            if (!requestConfirmation(msg))
+            if (!QtCommon::requestConfirmation(PrefsDialog::kCONFO_LOAD_DIC,
+                                               msg, "", this))
+            {
                 return;
+            }
         }
 
         LOG_INFO("Loading new dictionary file: " << lfq(iFileName));
@@ -425,10 +612,10 @@ void MainWindow::changeDictionary(QString iFileName)
             displayInfoMsg(_q("Loaded dictionary '%1'").arg(iFileName));
 
             // Save the location of the dictionary in the preferences
-            QSettings qs(ORGANIZATION, PACKAGE_NAME);
+            QSettings qs;
             qs.setValue(PrefsDialog::kINTF_DIC_PATH, iFileName);
         }
-        catch (std::exception &e)
+        catch (const std::exception &e)
         {
             displayErrorMsg(e.what());
         }
@@ -436,18 +623,6 @@ void MainWindow::changeDictionary(QString iFileName)
 }
 
 
-bool MainWindow::requestConfirmation(QString msg)
-{
-    QMessageBox confirmationBox(QMessageBox::Question, _q("Eliot"), msg,
-                                QMessageBox::Yes | QMessageBox::No, this);
-    confirmationBox.setInformativeText(_q("Do you want to continue?"));
-    confirmationBox.setDefaultButton(QMessageBox::Yes);
-    confirmationBox.setEscapeButton(QMessageBox::No);
-    int res = confirmationBox.exec();
-    return res == QMessageBox::Yes;
-}
-
-
 QAction * MainWindow::addMenuAction(QMenu *menu, QString iText,
                                     const QKeySequence &iShortcut,
                                     QString iStatusTip, const char *iMember,
@@ -467,7 +642,7 @@ QAction * MainWindow::addMenuAction(QMenu *menu, QString iText,
 void MainWindow::createMenu()
 {
     // Decide whether to show the toolbar
-    QSettings qs(ORGANIZATION, PACKAGE_NAME);
+    QSettings qs;
     bool showToolBar = qs.value(PrefsDialog::kINTF_SHOW_TOOLBAR, true).toBool();
     m_ui.toolBar->setVisible(showToolBar);
 
@@ -479,6 +654,9 @@ void MainWindow::createMenu()
     menuFile->addSeparator();
     addMenuAction(menuFile, _q("&Load..."), _q("Ctrl+O"),
                   _q("Load an existing game"), SLOT(onGameLoad()));
+    addMenuAction(menuFile, _q("Load the auto-saved game"), QString(""),
+                  _q("Load the automatically saved game (useful after a crash)"),
+                  SLOT(onGameLoadAutoSave()));
     m_actionGameSaveAs = addMenuAction(menuFile, _q("&Save as..."), _q("Ctrl+S"),
                   _q("Save the current game"), SLOT(onGameSaveAs()));
     menuFile->addSeparator();
@@ -519,11 +697,19 @@ void MainWindow::createMenu()
     QMenu *menuSettings = new QMenu(m_ui.menubar);
     m_ui.menubar->addAction(menuSettings->menuAction());
     menuSettings->setTitle(_q("&Settings"));
-    addMenuAction(menuSettings, _q("&Choose dictionary..."), _q("Ctrl+C"),
+    addMenuAction(menuSettings, _q("&Choose dictionary..."), _q("Ctrl+I"),
                   _q("Select a new dictionary"), SLOT(onSettingsChooseDic()));
     addMenuAction(menuSettings, _q("Create &new dictionary..."), QString(""),
                   _q("Start the wizard for creating a new dictionary "
                      "from an existing word list"), SLOT(onSettingsCreateDic()));
+    menuSettings->addSeparator();
+    addMenuAction(menuSettings, _q("&Favorite players..."), QString(""),
+                  _q("Define frequently used players for faster game creation"),
+                  SLOT(onSettingsFavPlayers()));
+    m_actionSettingsDefineTables =
+        addMenuAction(menuSettings, _q("&Define players tables..."), QString(""),
+                  _q("Define the tables where the players are sitting, in arbitration mode"),
+                  SLOT(onSettingsDefineTables()));
     addMenuAction(menuSettings, _q("&Preferences..."), _q("Ctrl+F"),
                   _q("Edit the preferences"), SLOT(onSettingsPreferences()),
                   false, QIcon(":/images/preferences.png"));
@@ -541,12 +727,16 @@ void MainWindow::createMenu()
     m_actionWindowsHistory = addMenuAction(menuWindows, _q("&History"), _q("Ctrl+H"),
                   _q("Show/hide the game history"), SLOT(onWindowsHistory()),
                   true, QIcon(":/images/playlist_16px.png"));
+    m_actionWindowsStats = addMenuAction(menuWindows, _q("&Statistics"), QString(""),
+                  _q("Show/hide the statistics"), SLOT(onWindowsStatistics()), true);
+    m_actionWindowsTimer = addMenuAction(menuWindows, _q("Ti&mer"), QString(""),
+                  _q("Show/hide the timer"), SLOT(onWindowsTimer()), true);
     m_actionWindowsDicTools = addMenuAction(menuWindows, _q("&Dictionary tools"), _q("Ctrl+D"),
                   _q("Show/hide the dictionary tools"), SLOT(onWindowsDicTools()), true);
 
     QMenu *menuHelp = new QMenu(m_ui.menubar);
     m_ui.menubar->addAction(menuHelp->menuAction());
-    menuHelp->setTitle(_q("Hel&p"));
+    menuHelp->setTitle(_q("He&lp"));
     addMenuAction(menuHelp, _q("&About..."), QString(""),
                   _q("About Eliot"), SLOT(onHelpAbout()),
                   false, QIcon(":/images/info_16px.png"));
@@ -566,25 +756,26 @@ void MainWindow::onGameNew()
         return;
     }
 
-    if (m_newGameDialog == NULL)
-        m_newGameDialog = new NewGame(this);
-
-    int res = m_newGameDialog->exec();
-    if (res == QDialog::Rejected)
-        return;
-
     if (m_game)
     {
         QString msg = _q("Starting a new game will stop the current one.");
-        if (!requestConfirmation(msg))
+        if (!QtCommon::requestConfirmation(PrefsDialog::kCONFO_START_GAME,
+                                           msg, "", this))
+        {
             return;
+        }
     }
 
+    NewGame newGameDialog;
+    int res = newGameDialog.exec();
+    if (res == QDialog::Rejected)
+        return;
+
     // Destroy the game and the associated controls
     destroyCurrentGame();
 
     // Create a new game
-    m_game = m_newGameDialog->createGame(*m_dic);
+    m_game = newGameDialog.createGame(*m_dic);
     if (m_game == NULL)
         return;
 
@@ -599,6 +790,18 @@ void MainWindow::onGameNew()
 
 
 void MainWindow::onGameLoad()
+{
+    loadGame("");
+}
+
+
+void MainWindow::onGameLoadAutoSave()
+{
+    loadGame(qfl(m_autoSaveGame));
+}
+
+
+void MainWindow::loadGame(QString fileName)
 {
     if (m_dic == NULL)
     {
@@ -606,7 +809,18 @@ void MainWindow::onGameLoad()
         return;
     }
 
-    QString fileName = QFileDialog::getOpenFileName(this, _q("Load a game"));
+    if (m_game)
+    {
+        QString msg = _q("Loading a saved game will stop the current game.");
+        if (!QtCommon::requestConfirmation(PrefsDialog::kCONFO_LOAD_GAME,
+                                           msg, "", this))
+        {
+            return;
+        }
+    }
+
+    if (fileName == "")
+        fileName = QFileDialog::getOpenFileName(this, _q("Load a game"));
     if (fileName != "")
     {
         try
@@ -615,7 +829,7 @@ void MainWindow::onGameLoad()
             destroyCurrentGame();
             m_game = tmpGame;
         }
-        catch (std::exception &e)
+        catch (const std::exception &e)
         {
             displayErrorMsg(_q("Error while loading the game:\n") + e.what());
             return;
@@ -642,7 +856,7 @@ void MainWindow::onGameSaveAs()
             m_game->save(lfq(fileName));
             displayInfoMsg(_q("Game saved"));
         }
-        catch (std::exception &e)
+        catch (const std::exception &e)
         {
             displayErrorMsg(_q("Error saving game: %1").arg(e.what()));
         }
@@ -738,7 +952,7 @@ void MainWindow::onGamePrint()
             nextHeight = curHeight + LINE_HEIGHT;
         for (unsigned int i = 0; i < history.getSize(); ++i)
         {
-            const Turn &t = history.getTurn(i);
+            const TurnData &t = history.getTurn(i);
             const Move &m = t.getMove();
 
             curWidth = TEXT_OFFSET;
@@ -755,7 +969,7 @@ void MainWindow::onGamePrint()
             curWidth += colWidths[1];
 
             // Word and coordinates
-            if (m.getType() == Move::VALID_ROUND)
+            if (m.isValid())
             {
                 const Round &r = m.getRound();
                 painter.drawText(curWidth, nextHeight, qfw(r.getWord()));
@@ -764,7 +978,7 @@ void MainWindow::onGamePrint()
                                  qfw(r.getCoord().toString()));
                 curWidth += colWidths[3];
             }
-            else if (m.getType() == Move::INVALID_WORD)
+            else if (m.isInvalid())
             {
                 painter.drawText(curWidth, nextHeight,
                                  "<" + qfw(m.getBadWord()) + ">");
@@ -772,7 +986,13 @@ void MainWindow::onGamePrint()
                 painter.drawText(curWidth, nextHeight, qfw(m.getBadCoord()));
                 curWidth += colWidths[3];
             }
-            else if (m.getType() == Move::PASS)
+            else if (m.isNull())
+            {
+                painter.drawText(curWidth, nextHeight, _q("(NO MOVE)"));
+                curWidth += colWidths[2];
+                curWidth += colWidths[3];
+            }
+            else if (m.isPass())
             {
                 painter.drawText(curWidth, nextHeight, _q("(PASS)"));
                 curWidth += colWidths[2];
@@ -803,20 +1023,16 @@ void MainWindow::onGamePrint()
 
 void MainWindow::onGameQuit()
 {
-    LOG_INFO("Exiting");
     close();
 }
 
 
 void MainWindow::onSettingsPreferences()
 {
-    if (m_prefsDialog == NULL)
-    {
-        m_prefsDialog = new PrefsDialog(this);
-        QObject::connect(m_prefsDialog, SIGNAL(prefsUpdated()),
-                         this, SLOT(prefsUpdated()));
-    }
-    m_prefsDialog->exec();
+    PrefsDialog *prefsDialog = new PrefsDialog(this);
+    QObject::connect(prefsDialog, SIGNAL(prefsUpdated()),
+                     this, SLOT(prefsUpdated()));
+    prefsDialog->exec();
 }
 
 
@@ -830,23 +1046,48 @@ void MainWindow::onSettingsChooseDic()
 
 void MainWindow::onSettingsCreateDic()
 {
-    DicWizard *wizard = new DicWizard(this);
+    DicWizard *wizard = new DicWizard(this, m_dic);
     wizard->setWindowTitle(_q("Dictionary creation wizard"));
-    connect(wizard, SIGNAL(infoMsg(QString)),
+    connect(wizard, SIGNAL(notifyInfo(QString)),
             this, SLOT(displayInfoMsg(QString)));
+    connect(wizard, SIGNAL(notifyProblem(QString)),
+            this, SLOT(displayErrorMsg(QString)));
     connect(wizard, SIGNAL(loadDictionary(QString)),
             this, SLOT(changeDictionary(QString)));
     wizard->show();
 }
 
 
+void MainWindow::onSettingsFavPlayers()
+{
+    FavPlayersDialog *dialog = new FavPlayersDialog(this);
+    QObject::connect(dialog, SIGNAL(notifyProblem(QString)),
+                     this, SLOT(displayErrorMsg(QString)));
+    dialog->exec();
+}
+
+
+void MainWindow::onSettingsDefineTables()
+{
+    ASSERT(m_game != NULL, "A game should have been started");
+
+    TablesDialog *dialog = new TablesDialog(this, *m_game);
+    QObject::connect(dialog, SIGNAL(notifyProblem(QString)),
+                     this, SLOT(displayErrorMsg(QString)));
+    if (dialog->exec() == QDialog::Accepted)
+    {
+        emit gameUpdated();
+    }
+}
+
+
 void MainWindow::onWindowsToolbar()
 {
     if (m_ui.toolBar->isVisible())
         m_ui.toolBar->hide();
     else
         m_ui.toolBar->show();
-    QSettings qs(ORGANIZATION, PACKAGE_NAME);
+    QSettings qs;
     qs.setValue(PrefsDialog::kINTF_SHOW_TOOLBAR, m_ui.toolBar->isVisible());
 }
 
@@ -856,7 +1097,7 @@ void MainWindow::onWindowsBag()
     if (m_bagWindow == NULL)
     {
         // Create the window
-        BagWidget *bag = new BagWidget(NULL);
+        BagWidget2 *bag = new BagWidget2(NULL);
         bag->setGame(m_game);
         m_bagWindow = new AuxWindow(*bag, _q("Bag"), "BagWindow",
                                     m_actionWindowsBag);
@@ -874,14 +1115,59 @@ void MainWindow::onWindowsBoard()
     if (m_boardWindow == NULL)
     {
         // Create the window
-        BoardWidget *board = new BoardWidget(m_coordModel, NULL);
+        QFrame *frame = new QFrame;
+        QLayout *hLayout = new QHBoxLayout;
+        frame->setLayout(hLayout);
+
+        QSplitter *vSplitter = new QSplitter(Qt::Vertical);
+        hLayout->addWidget(vSplitter);
+
+        QSplitter *hSplitter = new QSplitter(Qt::Horizontal);
+        vSplitter->addWidget(hSplitter);
+
+        RackWidget *rackWidget = new RackWidget;
+        rackWidget->setShowOnlyLastTurn(true);
+        rackWidget->setGame(m_game);
+        QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                         rackWidget, SLOT(setGame(const PublicGame*)));
+        QObject::connect(this, SIGNAL(gameUpdated()),
+                         rackWidget, SLOT(refresh()));
+        hSplitter->addWidget(rackWidget);
+
+        hSplitter->addWidget(new QWidget);
+
+        TimerWidget *timerWidget = new TimerWidget(NULL, *m_timerModel);
+        hSplitter->addWidget(timerWidget);
+
+        vSplitter->addWidget(new QWidget);
+
+        BoardWidget *board = new BoardWidget(m_coordModel);
+        board->setShowTempSigns(false);
+        board->setShowOnlyLastTurn(true);
         board->setGame(m_game);
-        m_boardWindow = new AuxWindow(*board, _q("Board"), "BoardWindow",
-                                      m_actionWindowsBoard);
         QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
                          board, SLOT(setGame(const PublicGame*)));
         QObject::connect(this, SIGNAL(gameUpdated()),
                          board, SLOT(refresh()));
+        vSplitter->addWidget(board);
+
+        m_boardWindow = new AuxWindow(*frame, _q("Board"), "BoardWindow",
+                                      m_actionWindowsBoard);
+
+        // Try to have decent initial sizes
+        QList<int> sizesVertical;
+        sizesVertical << 80 << 20 << 1;
+        vSplitter->setSizes(sizesVertical);
+        vSplitter->setStretchFactor(0, 0);
+        vSplitter->setStretchFactor(1, 0);
+        vSplitter->setStretchFactor(2, 1);
+
+        QList<int> sizesHorizontal;
+        sizesHorizontal << 1 << 40 << 150;
+        hSplitter->setSizes(sizesHorizontal);
+        hSplitter->setStretchFactor(0, 1);
+        hSplitter->setStretchFactor(1, 0);
+        hSplitter->setStretchFactor(2, 0);
     }
     m_boardWindow->toggleVisibility();
 }
@@ -905,6 +1191,37 @@ void MainWindow::onWindowsHistory()
 }
 
 
+void MainWindow::onWindowsStatistics()
+{
+    if (m_statsWindow == NULL)
+    {
+        // Create the window
+        StatsWidget *stats = new StatsWidget;
+        stats->setGame(m_game);
+        m_statsWindow = new AuxWindow(*stats, _q("Statistics"), "StatsWindow",
+                                      m_actionWindowsStats);
+        QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                         stats, SLOT(setGame(const PublicGame*)));
+        QObject::connect(this, SIGNAL(gameUpdated()),
+                         stats, SLOT(refresh()));
+    }
+    m_statsWindow->toggleVisibility();
+}
+
+
+void MainWindow::onWindowsTimer()
+{
+    if (m_timerWindow == NULL)
+    {
+        // Create the window
+        TimerWidget *timer = new TimerWidget(NULL, *m_timerModel);
+        m_timerWindow = new AuxWindow(*timer, _q("Timer"), "TimerWindow",
+                                      m_actionWindowsTimer);
+    }
+    m_timerWindow->toggleVisibility();
+}
+
+
 void MainWindow::onWindowsDicTools()
 {
     if (m_dicToolsWindow == NULL)
@@ -917,14 +1234,8 @@ void MainWindow::onWindowsDicTools()
                          dicTools, SLOT(setDic(const Dictionary*)));
         QObject::connect(dicTools, SIGNAL(requestDefinition(QString)),
                          this, SLOT(showDefinition(QString)));
-        // Link the training rack with the "Plus 1" one
-        QSettings qs(ORGANIZATION, PACKAGE_NAME);
-        if (qs.value(PrefsDialog::kINTF_LINK_TRAINING_7P1, false).toBool())
-        {
-            QObject::connect(m_playersWidget,
-                             SIGNAL(trainingRackUpdated(const QString&)),
-                             dicTools, SLOT(setPlus1Rack(const QString&)));
-        }
+        // Link the arbitration rack with the "Plus 1" one
+        linkArbitrationAnd7P1();
         // Fake a dictionary selection
         dicTools->setDic(m_dic);
         dicTools->setFocus();
@@ -935,20 +1246,23 @@ void MainWindow::onWindowsDicTools()
 
 void MainWindow::onHelpAbout()
 {
-    QString msg;
-    msg.sprintf("Eliot %s\n\n", VERSION);
+    QString msg = QString("Eliot %1<br><br>").arg(VERSION);
+    msg += _q("Copyright (C) 1999-2012 - Antoine Fraboulet & Olivier Teuliere.");
+    msg += "<br><br>";
     msg += _q( \
-        "Copyright (C) 1999-2011 - Antoine Fraboulet & Olivier Teuliere\n\n" \
         "This program is free software; you can redistribute it and/or " \
         "modify it under the terms of the GNU General Public License as " \
         "published by the Free Software Foundation; either version 2 of " \
         "the License, or (at your option) any later version.");
-    msg += "\n\n";
-    msg += _q("Web site: http://www.nongnu.org/eliot/en/");
+    msg += "<br><br>";
+    // Translate the URL, because the web site is translated in French
+    QString url = _q("http://www.nongnu.org/eliot/en/");
+    msg += _q("Web site: %1").arg(QString("<a href=\"%1\">%2</a>").arg(url).arg(url));
     // QMessageBox::about() doesn't add the nice information icon, so we create
     // the box manually (not much work...)
     QMessageBox aboutBox(QMessageBox::Information, _q("About Eliot"),
                          msg, QMessageBox::Ok, this);
+    aboutBox.setTextFormat(Qt::RichText);
     aboutBox.exec();
 }
 
@@ -1010,15 +1324,14 @@ void MainWindow::onHistoryReplayTurn()
     if (m_game == NULL)
         return;
 
-    QSettings settings(ORGANIZATION, PACKAGE_NAME);
-    bool warn = settings.value(PrefsDialog::kINTF_WARN_REPLAY_TURN, true).toBool();
-    if (warn) {
-        // Ask for a confirmation, because this may lead to data loss
-        QString msg = _q("Replaying this turn will modify the game history "
-                         "by deleting the turns after the displayed one (i.e. "
-                         "turns \"in the future\").");
-        if (!requestConfirmation(msg))
-            return;
+    // Ask for a confirmation, because this may lead to data loss
+    QString msg = _q("Replaying this turn will modify the game history "
+                     "by deleting the turns after the displayed one (i.e. "
+                     "turns \"in the future\").");
+    if (!QtCommon::requestConfirmation(PrefsDialog::kCONFO_REPLAY_TURN,
+                                       msg, "", this))
+    {
+        return;
     }
 
     unsigned int currTurn = m_game->getCurrTurn();
