@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,26 +21,35 @@
 #ifndef MAIN_WINDOW_H_
 #define MAIN_WINDOW_H_
 
+#include <string>
 #include <QMainWindow>
 
 #include <ui/main_window.ui.h>
+#include "logging.h"
 #include "coord_model.h"
 
 
+using std::string;
+
+
 class Dictionary;
 class Bag;
 class Board;
 class History;
 class PublicGame;
-class NewGame;
-class PrefsDialog;
+class HistoryTabWidget;
 class PlayerTabWidget;
+class ScoreWidget;
+class TrainingWidget;
+class ArbitrationWidget;
 class AuxWindow;
+class TimerModel;
 class QLabel;
 class QAction;
 
 class MainWindow: public QMainWindow
 {
+    DEFINE_LOGGER();
     Q_OBJECT;
 
 public:
@@ -52,6 +61,7 @@ signals:
     void gameChanged(const PublicGame *iGame);
     void gameChangedNonConst(PublicGame *iGame);
     void gameUpdated();
+    void turnChanged(int iCurrTurn, bool isLastTurn);
 
 public slots:
     /// Display an error message to the user
@@ -65,15 +75,21 @@ protected:
 private slots:
     void onGameNew();
     void onGameLoad();
+    void onGameLoadAutoSave();
     void onGameSaveAs();
     void onGamePrint();
+    void onGameQuit();
     void onSettingsChooseDic();
     void onSettingsCreateDic();
+    void onSettingsFavPlayers();
+    void onSettingsDefineTables();
     void onSettingsPreferences();
     void onWindowsToolbar();
     void onWindowsBag();
     void onWindowsBoard();
     void onWindowsHistory();
+    void onWindowsStatistics();
+    void onWindowsTimer();
     void onWindowsDicTools();
     void onHelpAbout();
 
@@ -83,6 +99,9 @@ private slots:
     void onHistoryLastTurn();
     void onHistoryReplayTurn();
 
+    /// Simply emit a beep with the system speakers
+    void beep();
+
     /** Load a new dictionary */
     void changeDictionary(QString);
 
@@ -92,15 +111,21 @@ private slots:
     /** Perform work when the preferences were updated */
     void prefsUpdated();
 
+    /** Select the correct tab in the history widget when the player changes */
+    void onPlayerSelected(unsigned iPlayerId);
+
     /**
      * Perform several updates when the game changes (title bar, status bar,
      * grey out some menu items, ...)
      */
-    void updateForGame(const PublicGame *iGame);
+    void updateForGame(PublicGame *iGame);
 
     /// Update the status bar contents
     void updateStatusBar(const Dictionary *iDic);
 
+    /** Show the definition of the given word */
+    void showDefinition(QString iWord);
+
 private:
     /// Current dictionary
     const Dictionary *m_dic;
@@ -111,15 +136,21 @@ private:
     /// The UI file generated with Qt Designer
     Ui::MainWindow m_ui;
 
-    /// Dialog for creating a new game
-    NewGame *m_newGameDialog;
-
-    /// Dialog for the preferences
-    PrefsDialog *m_prefsDialog;
+    /// Widget for the game history
+    HistoryTabWidget *m_historyTabWidget;
 
     /// Widget for the players
     PlayerTabWidget *m_playersWidget;
 
+    /// Widget for the training mode
+    TrainingWidget *m_trainingWidget;
+
+    /// Widget for the arbitration mode
+    ArbitrationWidget *m_arbitrationWidget;
+
+    /// Widget for the scores
+    ScoreWidget *m_scoresWidget;
+
     /// Actions enabled or disabled depending on the game state
     QAction *m_actionGamePrint;
     QAction *m_actionGameSaveAs;
@@ -129,9 +160,12 @@ private:
     QAction *m_actionHistoryLastTurn;
     QAction *m_actionHistoryReplayTurn;
     QAction *m_actionWindowsToolbar;
+    QAction *m_actionSettingsDefineTables;
 
     static const char * m_windowName;
 
+    string m_autoSaveGame;
+
     /// Auxiliary windows
     //@{
     QAction *m_actionWindowsBag;
@@ -140,16 +174,30 @@ private:
     AuxWindow *m_boardWindow;
     QAction *m_actionWindowsHistory;
     AuxWindow *m_historyWindow;
+    QAction *m_actionWindowsStats;
+    AuxWindow *m_statsWindow;
+    QAction *m_actionWindowsTimer;
+    AuxWindow *m_timerWindow;
     QAction *m_actionWindowsDicTools;
     AuxWindow *m_dicToolsWindow;
     //@}
 
-    /// Label indicating the name of the current dictionary
+    /// Status bar label indicating the current turn (when there is a game)
+    QLabel *m_turnLabel;
+    /// Status bar label indicating the remaining letters (when there is a game)
+    QLabel *m_lettersLabel;
+    /// Status bar label indicating the name of the current dictionary
     QLabel *m_dicNameLabel;
 
     /// Model for the coordinates of the word to play
     CoordModel m_coordModel;
 
+    /// Model for the timer widgets
+    TimerModel *m_timerModel;
+
+    /// Last known turn number. Used to emit turnChanged().
+    unsigned m_currentTurn;
+
     /// Save window state
     void writeSettings() const;
     /// Restore window state
@@ -167,6 +215,18 @@ private:
     /// Destroy the current game (if any) and the associated widgets
     void destroyCurrentGame();
 
+    /**
+     * Handle correctly the signal/slot connection between the Arbitration
+     * widget and the 7+1 dictionary tool
+     */
+    void linkArbitrationAnd7P1();
+
+    /**
+     * Load the game saved in the given file. If iFileName is an empty string,
+     * the auto-saved game will be loaded.
+     */
+    void loadGame(QString iFileName);
+
 };
 
 #endif
