@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2009 Olivier Teulière
+ * Copyright (C) 2009-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include <QtGui/QPushButton>
 
 #include "play_word_mediator.h"
+#include "validator_factory.h"
 #include "coord_model.h"
 #include "qtcommon.h"
 
@@ -31,27 +32,7 @@
 #include "debug.h"
 
 
-/// Validator used for the "play word" line edit
-class PlayWordValidator: public QValidator
-{
-public:
-    explicit PlayWordValidator(QObject *parent,
-                               const Dictionary &iDic);
-    virtual State validate(QString &input, int &pos) const;
-
-private:
-    const Dictionary &m_dic;
-};
-
-
-/// Validator used for the "coords" line edit
-class CoordsValidator: public QValidator
-{
-public:
-    explicit CoordsValidator(QObject *parent);
-    virtual State validate(QString &input, int &pos) const;
-};
-
+INIT_LOGGER(qt, PlayWordMediator);
 
 
 PlayWordMediator::PlayWordMediator(QObject *parent, QLineEdit &iEditPlay,
@@ -62,40 +43,85 @@ PlayWordMediator::PlayWordMediator(QObject *parent, QLineEdit &iEditPlay,
     m_lineEditCoord(iEditCoord), m_lineEditPoints(iEditPoints),
     m_pushButtonPlay(iButtonPlay), m_coordModel(iCoordModel)
 {
-    m_lineEditPlay.setFocus();
-    // These strings cannot be in the .ui file, because of the newlines
-    m_lineEditPlay.setToolTip(_q("Enter the word to play (case-insensitive).\n"
-            "A joker from the rack must be written in parentheses.\n"
-            "E.g.: w(o)rd or W(O)RD"));
-    m_lineEditCoord.setToolTip(_q("Enter the coordinates of the word.\n"
-            "Specify the row before the column for horizontal words,\n"
-            "and the column before the row for vertical words.\n"
-            "E.g.: H4 or 4H"));
+    SetTooltips(m_lineEditPlay, m_lineEditCoord);
 
     /// Set validators;
     if (m_game)
     {
-        m_lineEditPlay.setValidator(new PlayWordValidator(this, m_game->getDic()));
-        m_lineEditCoord.setValidator(new CoordsValidator(this));
+        m_lineEditPlay.setValidator(ValidatorFactory::newPlayWordValidator(this, m_game->getDic()));
+        m_lineEditCoord.setValidator(ValidatorFactory::newCoordsValidator(this));
     }
 
     // Set all the connections
     QObject::connect(&m_lineEditPlay, SIGNAL(textChanged(const QString&)),
-                     this, SLOT(lineEditPlay_textChanged()));
+                     this, SLOT(updatePointsAndState()));
     QObject::connect(&m_lineEditPlay, SIGNAL(returnPressed()),
-                     this, SLOT(lineEditPlay_returnPressed()));
+                     this, SLOT(playWord()));
     QObject::connect(&m_lineEditCoord, SIGNAL(textChanged(const QString&)),
-                     this, SLOT(lineEditCoord_textChanged(const QString&)));
+                     this, SLOT(onCoordChanged(const QString&)));
     QObject::connect(&m_lineEditCoord, SIGNAL(returnPressed()),
-                     this, SLOT(lineEditCoord_returnPressed()));
+                     this, SLOT(playWord()));
     QObject::connect(&m_pushButtonPlay, SIGNAL(clicked()),
-                     this, SLOT(pushButtonPlay_clicked()));
+                     this, SLOT(playWord()));
     QObject::connect(&m_coordModel, SIGNAL(coordChanged(const Coord&, const Coord&)),
                      this, SLOT(updateCoord(const Coord&, const Coord&)));
 }
 
 
-void PlayWordMediator::lineEditPlay_textChanged()
+void PlayWordMediator::setCleverFocus()
+{
+    if (!m_lineEditPlay.hasFocus() && !m_lineEditCoord.hasFocus())
+        m_lineEditPlay.setFocus();
+}
+
+
+void PlayWordMediator::SetTooltips(QLineEdit &iEditWord, QLineEdit &iEditCoord)
+{
+    // These strings cannot be in the .ui file, because of the newlines
+    iEditWord.setToolTip(_q("Enter the word to play (case-insensitive).\n"
+            "A joker from the rack must be written in parentheses.\n"
+            "E.g.: w(o)rd or W(O)RD"));
+    iEditCoord.setToolTip(_q("Enter the coordinates of the word.\n"
+            "Specify the row before the column for horizontal words,\n"
+            "and the column before the row for vertical words.\n"
+            "E.g.: H4 or 4H"));
+}
+
+
+bool PlayWordMediator::GetPlayedWord(QLineEdit &iEditWord,
+                                     const Dictionary &iDic,
+                                     wstring *oPlayedWord,
+                                     QString *oProblemCause)
+{
+    // Convert the jokers to lowercase
+    const wistring &inputWord = wfq(iEditWord.text().toUpper());
+    // Convert to internal representation, then back to QString
+    QString word = qfw(iDic.convertFromInput(inputWord));
+
+    int pos;
+    while ((pos = word.indexOf('(')) != -1)
+    {
+        if (word.size() < pos + 3 || word[pos + 2] != ')' ||
+            !iDic.validateLetters(wfq(QString(word[pos + 1]))))
+        {
+            // Cannot parse the string...
+            *oPlayedWord = wfq(word);
+            *oProblemCause = _q("Cannot play word: misplaced parentheses");
+            return false;
+        }
+
+        QChar chr = word[pos + 1].toLower();
+        word.remove(pos, 3);
+        word.insert(pos, chr);
+    }
+
+    // Convert the input string into an internal one
+    *oPlayedWord = wfq(word);
+    return true;
+}
+
+
+void PlayWordMediator::updatePointsAndState()
 {
     bool acceptableInput =
         m_lineEditPlay.hasAcceptableInput() &&
@@ -118,19 +144,18 @@ void PlayWordMediator::lineEditPlay_textChanged()
 }
 
 
-void PlayWordMediator::lineEditPlay_returnPressed()
+void PlayWordMediator::playWord()
 {
     if (!m_lineEditPlay.hasAcceptableInput() ||
         !m_lineEditCoord.hasAcceptableInput())
         return;
 
-    const wstring &word = getWord(true);
+    const wstring &word = getWord();
     QString coords = m_lineEditCoord.text();
     int res = m_game->play(word, wfq(coords));
     if (res == 0)
     {
         emit gameUpdated();
-        m_lineEditPlay.setFocus();
     }
     else
     {
@@ -186,132 +211,46 @@ void PlayWordMediator::lineEditPlay_returnPressed()
 }
 
 
-wstring PlayWordMediator::getWord(bool emitSignal)
+void PlayWordMediator::onCoordChanged(const QString &iText)
 {
-    // Convert the jokers to lowercase
-    const wistring &inputWord = wfq(m_lineEditPlay.text().toUpper());
-    // Convert to internal representation, then back to QString
-    QString word = qfw(m_game->getDic().convertFromInput(inputWord));
-
-    int pos;
-    while ((pos = word.indexOf('(')) != -1)
-    {
-        if (word.size() < pos + 3 || word[pos + 2] != ')' ||
-            !m_game->getDic().validateLetters(wfq(QString(word[pos + 1]))))
-        {
-            // Bug in validate()!
-            // This should never happen
-            if (emitSignal)
-            {
-                QString msg = _q("Cannot play word: misplaced parentheses");
-                emit notifyProblem(msg);
-            }
-            break;
-        }
-        else
-        {
-            QChar chr = word[pos + 1].toLower();
-            word.remove(pos, 3);
-            word.insert(pos, chr);
-        }
-    }
-
-    // Convert the input string into an internal one
-    const wstring intWord =
-        m_game->getDic().convertFromInput(wfq(word));
-
-    return intWord;
-}
-
-
-void PlayWordMediator::lineEditCoord_textChanged(const QString &iText)
-{
-    Coord c(wfq(iText));
-    if (!(m_coordModel.getCoord() == c))
-        m_coordModel.setCoord(Coord(wfq(iText)));
-    lineEditPlay_textChanged();
+    Coord coord(wfq(iText));
+    if (!(m_coordModel.getCoord() == coord))
+        m_coordModel.setCoord(coord);
+    updatePointsAndState();
 }
 
 
 void PlayWordMediator::updateCoord(const Coord &, const Coord &iNewCoord)
 {
+    // Ignore updates to non-visible controls (which happens when there
+    // are several human players).
+    // It fixes a focus problem.
+    if (!m_lineEditPlay.isVisible())
+        return;
+
     if (iNewCoord.isValid() && m_lineEditCoord.text() != qfw(iNewCoord.toString()))
         m_lineEditCoord.setText(qfw(iNewCoord.toString()));
     else if (!iNewCoord.isValid() && m_lineEditCoord.hasAcceptableInput())
         m_lineEditCoord.setText("");
-    lineEditPlay_textChanged();
+    // Set the focus to the "Play word" zone is the focus is not
+    // already on a QLineEdit (because of a click on the board)
+    setCleverFocus();
+
+    updatePointsAndState();
 }
 
 
-// ------ Validators ------
-
-PlayWordValidator::PlayWordValidator(QObject *parent,
-                                     const Dictionary &iDic)
-    : QValidator(parent), m_dic(iDic)
+wstring PlayWordMediator::getWord()
 {
+    wstring playedWord;
+    QString msg;
+    // Avoid a warning in non debug mode (unused variable)
+#ifdef DEBUG
+    bool ok =
+#endif
+        GetPlayedWord(m_lineEditPlay, m_game->getDic(), &playedWord, &msg);
+    // The method should only be called for a correct input
+    ASSERT(ok, "Invalid word (should not be possible)");
+    return playedWord;
 }
 
-
-QValidator::State PlayWordValidator::validate(QString &input, int &) const
-{
-    if (input == "")
-        return Intermediate;
-
-    const wistring &winput = wfq(input);
-    // The string is invalid if it contains invalid input characters
-    if (!m_dic.validateInputChars(winput, L"()") || input.contains('?'))
-        return Invalid;
-
-    // Convert the string to internal letters
-    const wstring &intInput = m_dic.convertFromInput(winput);
-    // The string is invalid if it contains characters not present
-    // in the dictionary (ignoring parentheses)
-    if (!m_dic.validateLetters(intInput, L"()"))
-        return Intermediate;
-
-    // Check the parentheses pairs
-    QString qintInput = qfw(intInput);
-    int pos;
-    while ((pos = qintInput.indexOf('(')) != -1)
-    {
-        if (qintInput.size() < pos + 3 || qintInput[pos + 2] != ')' ||
-            !m_dic.validateLetters(wfq(QString(qintInput[pos + 1]))))
-        {
-            return Intermediate;
-        }
-        else
-        {
-            qintInput.remove(pos, 3);
-        }
-    }
-    if (qintInput.indexOf(')') != -1)
-        return Intermediate;
-
-    return Acceptable;
-}
-
-
-
-CoordsValidator::CoordsValidator(QObject *parent)
-    : QValidator(parent)
-{
-}
-
-
-QValidator::State CoordsValidator::validate(QString &input, int &) const
-{
-    // Only authorize characters part of a valid coordinate
-    wstring copy = wfq(input.toUpper());
-    wstring authorized = L"ABCDEFGHIJKLMNO1234567890";
-    if (copy.find_first_not_of(authorized) != wstring::npos)
-        return Invalid;
-
-    // Check coordinates
-    Coord c(wfq(input));
-    if (!c.isValid())
-        return Intermediate;
-
-    return Acceptable;
-}
-
-
