@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,14 +18,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#include <QtGui/QLineEdit>
 #include <QtGui/QHBoxLayout>
-#include <QtGui/QValidator>
 #include <QtCore/QStringList>
 
 #include "player_widget.h"
-#include "training_widget.h"
 #include "play_word_mediator.h"
+#include "validator_factory.h"
 #include "qtcommon.h"
 #include "public_game.h"
 #include "player.h"
@@ -38,19 +36,7 @@
 #include "encoding.h"
 
 
-/// Validator used for the "change letters" line edit
-class ChangeValidator: public QValidator
-{
-public:
-    explicit ChangeValidator(QObject *parent,
-                             const QLineEdit &iLineEdit,
-                             const Dictionary &iDic);
-    virtual State validate(QString &input, int &pos) const;
-
-private:
-    const QLineEdit &m_lineEdit;
-    const Dictionary &m_dic;
-};
+INIT_LOGGER(qt, PlayerWidget);
 
 
 PlayerWidget::PlayerWidget(QWidget *parent, CoordModel &iCoordModel,
@@ -68,14 +54,24 @@ PlayerWidget::PlayerWidget(QWidget *parent, CoordModel &iCoordModel,
     QObject::connect(m_mediator, SIGNAL(notifyProblem(QString)),
                      this, SIGNAL(notifyProblem(QString)));
 
+    QObject::connect(pushButtonShuffle, SIGNAL(clicked()),
+                     this, SLOT(shuffle()));
+    QObject::connect(pushButtonPass, SIGNAL(clicked()),
+                     this, SLOT(pass()));
+    QObject::connect(pushButtonChange, SIGNAL(clicked()),
+                     this, SLOT(changeLetters()));
+    QObject::connect(lineEditChange, SIGNAL(returnPressed()),
+                     this, SLOT(changeLetters()));
+    QObject::connect(lineEditChange, SIGNAL(textChanged(const QString&)),
+                     this, SLOT(enableChangeButton()));
+
+    lineEditRack->setReadOnly(true);
+
     if (m_game)
     {
         // Do not allow messing with AI players
         if (!m_game->getPlayer(m_player).isHuman())
             setEnabled(false);
-
-        // Changing the rack is authorized only in training mode
-        lineEditRack->setReadOnly(m_game->getMode() != PublicGame::kTRAINING);
     }
     if (m_game == NULL || m_game->getMode() != PublicGame::kFREEGAME)
     {
@@ -87,10 +83,14 @@ PlayerWidget::PlayerWidget(QWidget *parent, CoordModel &iCoordModel,
     }
     else
     {
-        lineEditChange->setValidator(new ChangeValidator(this, *lineEditRack,
-                                                         m_game->getDic()));
+        QValidator * val = ValidatorFactory::newChangeValidator(this, *lineEditRack, m_game->getDic());
+        lineEditChange->setValidator(val);
     }
 
+    // When the widget gains the focus, transfer it
+    // to the "main" line edit by default
+    setFocusProxy(lineEditPlay);
+
     refresh();
 }
 
@@ -130,14 +130,15 @@ void PlayerWidget::refresh()
 }
 
 
-void PlayerWidget::on_pushButtonShuffle_clicked()
+void PlayerWidget::shuffle()
 {
     m_game->shuffleRack();
     emit gameUpdated();
+    m_mediator->setCleverFocus();
 }
 
 
-void PlayerWidget::on_lineEditChange_textChanged()
+void PlayerWidget::enableChangeButton()
 {
     pushButtonChange->setEnabled(lineEditChange->hasAcceptableInput() &&
                                  lineEditChange->text() != "");
@@ -146,25 +147,25 @@ void PlayerWidget::on_lineEditChange_textChanged()
 }
 
 
-void PlayerWidget::on_lineEditChange_returnPressed()
+void PlayerWidget::changeLetters()
 {
     ASSERT(m_game->getMode() == PublicGame::kFREEGAME,
            "Trying to change letters while not in free game mode");
 
-    pass(lineEditChange->text());
+    helperChangePass(lineEditChange->text());
 }
 
 
-void PlayerWidget::on_pushButtonPass_clicked()
+void PlayerWidget::pass()
 {
     ASSERT(m_game->getMode() == PublicGame::kFREEGAME,
            "Trying to pass while not in free game mode");
 
-    pass("");
+    helperChangePass("");
 }
 
 
-void PlayerWidget::pass(QString inputLetters)
+void PlayerWidget::helperChangePass(QString inputLetters)
 {
     // Convert the input string into an internal one
     const wstring &letters =
@@ -173,7 +174,10 @@ void PlayerWidget::pass(QString inputLetters)
     // Pass the turn (and possibly change letters)
     int res = m_game->freeGamePass(letters);
     if (res == 0)
+    {
         emit gameUpdated();
+        m_mediator->setCleverFocus();
+    }
     else
     {
         QString msg;
@@ -187,7 +191,7 @@ void PlayerWidget::pass(QString inputLetters)
             msg = msg.arg(_q("The rack of the current player does not contain all the listed letters"));
         else if (res == 3)
             msg = msg.arg(_q("The game is already finished!"));
-        else if (res == 3)
+        else if (res == 4)
             msg = msg.arg(_q("Some letters are invalid for the current dictionary"));
         else
             msg = msg.arg(_q("Unknown error"));
@@ -197,51 +201,8 @@ void PlayerWidget::pass(QString inputLetters)
 
 
 
-ChangeValidator::ChangeValidator(QObject *parent,
-                                 const QLineEdit &iLineEdit,
-                                 const Dictionary &iDic)
-    : QValidator(parent), m_lineEdit(iLineEdit), m_dic(iDic)
-{
-}
-
-
-QValidator::State ChangeValidator::validate(QString &input, int &) const
-{
-    // The string is invalid if it contains invalid input characters
-    const wistring &winput = wfq(input);
-    if (!m_dic.validateInputChars(winput))
-        return Invalid;
-
-    // Convert the string to internal letters
-    const wstring &intInput = m_dic.convertFromInput(winput);
-    // The string is invalid if it contains characters not present
-    // in the dictionary
-    if (!m_dic.validateLetters(intInput))
-        return Intermediate;
-
-    const wstring &rack = m_dic.convertFromInput(wfq(m_lineEdit.text()));
-    if (intInput.size() > rack.size())
-        return Intermediate;
-    // The letters to change must be in the rack
-    // We convert back to QString objects, because their count() method is
-    // very practical...
-    QString qrack = qfw(rack);
-    QString qinput = qfw(intInput);
-    for (int i = 0; i < qinput.size(); ++i)
-    {
-        if (qinput.count(qinput[i], Qt::CaseInsensitive) >
-            qrack.count(qinput[i], Qt::CaseInsensitive))
-        {
-            return Intermediate;
-        }
-    }
-    return Acceptable;
-}
-
-
-
 PlayerTabWidget::PlayerTabWidget(CoordModel &iCoordModel, QWidget *parent)
-    : QTabWidget(parent), m_coordModel(iCoordModel)
+    : QTabWidget(parent), m_game(NULL), m_coordModel(iCoordModel)
 {
     QObject::connect(this, SIGNAL(currentChanged(int)),
                      this, SLOT(changeCurrentPlayer(int)));
@@ -265,47 +226,24 @@ void PlayerTabWidget::setGame(PublicGame *iGame)
 
     if (iGame != NULL)
     {
-        // Training mode: use a dedicated widget
-        if (iGame->getMode() == PublicGame::kTRAINING)
+        // Add one tab per player
+        for (unsigned int i = 0; i < iGame->getNbPlayers(); ++i)
         {
-            const Player &player = iGame->getPlayer(0);
-            TrainingWidget *trWidget = new TrainingWidget(NULL, m_coordModel, iGame);
-            QObject::connect(this, SIGNAL(refreshSignal()),
-                             trWidget, SLOT(refresh()));
+            const Player &player = iGame->getPlayer(i);
+            PlayerWidget *p = new PlayerWidget(NULL, m_coordModel, i, iGame);
+            QObject::connect(this, SIGNAL(refreshSignal()), p, SLOT(refresh()));
             // Forward signals to the outside
-            QObject::connect(trWidget, SIGNAL(notifyProblem(QString)),
+            QObject::connect(p, SIGNAL(notifyProblem(QString)),
                              this, SIGNAL(notifyProblem(QString)));
-            QObject::connect(trWidget, SIGNAL(notifyInfo(QString)),
-                             this, SIGNAL(notifyInfo(QString)));
-            QObject::connect(trWidget, SIGNAL(gameUpdated()),
+            QObject::connect(p, SIGNAL(gameUpdated()),
                              this, SIGNAL(gameUpdated()));
-            QObject::connect(trWidget, SIGNAL(rackUpdated(const QString&)),
-                             this, SIGNAL(trainingRackUpdated(const QString&)));
-            QObject::connect(trWidget, SIGNAL(requestDefinition(QString)),
-                             this, SIGNAL(requestDefinition(QString)));
-            addTab(trWidget, qfw(player.getName()));
-        }
-        else
-        {
-            // Add one tab per player
-            for (unsigned int i = 0; i < iGame->getNbPlayers(); ++i)
-            {
-                const Player &player = iGame->getPlayer(i);
-                PlayerWidget *p = new PlayerWidget(NULL, m_coordModel, i, iGame);
-                QObject::connect(this, SIGNAL(refreshSignal()), p, SLOT(refresh()));
-                // Forward signals to the outside
-                QObject::connect(p, SIGNAL(notifyProblem(QString)),
-                                 this, SIGNAL(notifyProblem(QString)));
-                QObject::connect(p, SIGNAL(gameUpdated()),
-                                 this, SIGNAL(gameUpdated()));
-                addTab(p, qfw(player.getName()));
-                // Switching to a tab corresponding to an AI player
-                // is forbidden
-                if (!player.isHuman())
-                    setTabEnabled(i, false);
-            }
-            setCurrentIndex(iGame->getCurrentPlayer().getId());
+            addTab(p, qfw(player.getName()));
+            // Switching to a tab corresponding to an AI player
+            // is forbidden
+            if (!player.isHuman())
+                setTabEnabled(i, false);
         }
+        setCurrentIndex(iGame->getCurrentPlayer().getId());
     }
 }
 
@@ -315,6 +253,10 @@ void PlayerTabWidget::refresh()
     if (m_game)
         setCurrentIndex(m_game->getCurrentPlayer().getId());
     emit refreshSignal();
+
+    // Give the focus to the current widget. Without this, the widget does
+    // not get focus in some cases
+    currentWidget()->setFocus();
 }
 
 
