@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,9 @@
 
 #include <QtGui/QWidget>
 #include <QtGui/QTabWidget>
+
 #include "ui/player_widget.ui.h"
+#include "logging.h"
 
 
 class QLineEdit;
@@ -35,6 +37,7 @@ class Coord;
 class PlayerWidget: public QWidget, private Ui::PlayerWidget
 {
     Q_OBJECT;
+    DEFINE_LOGGER();
 
 public:
     explicit PlayerWidget(QWidget *parent,
@@ -54,15 +57,12 @@ protected:
     virtual QSize sizeHint() const;
 
 private slots:
-    void on_pushButtonShuffle_clicked();
-    void on_pushButtonChange_clicked() { on_lineEditChange_returnPressed(); }
-    void on_pushButtonPass_clicked();
-    void on_lineEditChange_textChanged();
-    void on_lineEditChange_returnPressed();
+    void shuffle();
+    void pass();
+    void changeLetters();
+    void enableChangeButton();
 
 private:
-    void pass(QString inputLetters);
-
     /// Encapsulated game, can be NULL
     PublicGame *m_game;
 
@@ -72,6 +72,8 @@ private:
     /// Encapsulated player, valid iff m_game is not NULL
     unsigned int m_player;
 
+    void helperChangePass(QString inputLetters = "");
+
 };
 
 
@@ -91,7 +93,6 @@ signals:
     void gameUpdated();
     void notifyProblem(QString iMsg);
     void notifyInfo(QString iMsg);
-    void trainingRackUpdated(const QString &iRack);
     void requestDefinition(QString iWord);
 
 private slots:
