@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2010 Olivier Teulière
+ * Copyright (C) 2010-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,11 +19,14 @@
  *****************************************************************************/
 
 #include <QtGui/QStandardItemModel>
-#include <QtGui/QValidator>
+#include <QtGui/QMenu>
+#include <QtGui/QHeaderView>
 
 #include "training_widget.h"
 #include "qtcommon.h"
 #include "play_word_mediator.h"
+#include "custom_popup.h"
+#include "validator_factory.h"
 
 #include "dic.h"
 #include "bag.h"
@@ -31,35 +34,31 @@
 #include "game_exception.h"
 #include "player.h"
 #include "results.h"
+#include "debug.h"
 
 using namespace std;
 
 
-/// Validator used for the rack line edit
-class RackValidator: public QValidator
-{
-public:
-    explicit RackValidator(QObject *parent, const Bag *iBag);
-    virtual State validate(QString &input, int &pos) const;
+INIT_LOGGER(qt, TrainingWidget);
 
-private:
-    const Bag *m_bag;
-};
+
+static const int HIDDEN_COLUMN = 6;
 
 
 TrainingWidget::TrainingWidget(QWidget *parent, CoordModel &iCoordModel, PublicGame *iGame)
-    : QWidget(parent), m_game(iGame)
+    : QWidget(parent), m_game(iGame), m_autoResizeColumns(true)
 {
     setupUi(this);
+    treeViewResults->setAlternatingRowColors(true);
 
+    blackPalette = lineEditRack->palette();
     redPalette = lineEditRack->palette();
     redPalette.setColor(QPalette::Text, Qt::red);
-    blackPalette = lineEditRack->palette();
-    blackPalette.setColor(QPalette::Text, Qt::black);
 
     // Use the mediator
     m_mediator = new PlayWordMediator(this, *lineEditPlay, *lineEditCoords,
-                                      *pushButtonPlay, iCoordModel, m_game);
+                                      *lineEditPoints, *pushButtonPlay,
+                                      iCoordModel, m_game);
     QObject::connect(m_mediator, SIGNAL(gameUpdated()),
                      this, SIGNAL(gameUpdated()));
     QObject::connect(m_mediator, SIGNAL(notifyProblem(QString)),
@@ -68,15 +67,51 @@ TrainingWidget::TrainingWidget(QWidget *parent, CoordModel &iCoordModel, PublicG
     // Associate the model to the view
     m_model = new QStandardItemModel(this);
     treeViewResults->setModel(m_model);
-    m_model->setColumnCount(6);
+    m_model->setColumnCount(7);
     m_model->setHeaderData(0, Qt::Horizontal, _q("Word"), Qt::DisplayRole);
     m_model->setHeaderData(1, Qt::Horizontal, _q("Ref"), Qt::DisplayRole);
     m_model->setHeaderData(2, Qt::Horizontal, _q("Points"), Qt::DisplayRole);
     m_model->setHeaderData(3, Qt::Horizontal, "*", Qt::DisplayRole);
     m_model->setHeaderData(4, Qt::Horizontal, "", Qt::DisplayRole);
-    // Hidden column, used to store internal data
     m_model->setHeaderData(5, Qt::Horizontal, "", Qt::DisplayRole);
-    treeViewResults->setColumnHidden(5, true);
+    // Hidden column, used to store internal data
+    m_model->setHeaderData(HIDDEN_COLUMN, Qt::Horizontal, "", Qt::DisplayRole);
+    treeViewResults->setColumnHidden(HIDDEN_COLUMN, true);
+
+    QObject::connect(lineEditRack, SIGNAL(returnPressed()),
+                     this, SLOT(search()));
+    QObject::connect(lineEditRack, SIGNAL(textEdited(const QString &)),
+                     this, SLOT(onRackEdited(const QString &)));
+
+    QObject::connect(pushButtonRack, SIGNAL(clicked()),
+                     this, SLOT(setNewRack()));
+    QObject::connect(pushButtonComplement, SIGNAL(clicked()),
+                     this, SLOT(completeRack()));
+    QObject::connect(pushButtonSearch, SIGNAL(clicked()),
+                     this, SLOT(search()));
+    QObject::connect(pushButtonPlaySelected, SIGNAL(clicked()),
+                     this, SLOT(playSelectedWord()));
+    QObject::connect(treeViewResults, SIGNAL(activated(const QModelIndex&)),
+                     this, SLOT(playSelectedWord()));
+
+    // Add a context menu to the tree header
+    QAction *lockSizesAction = new QAction(_q("Lock columns sizes"), this);
+    lockSizesAction->setCheckable(true);
+    lockSizesAction->setStatusTip(_q("Disable auto-resizing of the columns"));
+    treeViewResults->header()->addAction(lockSizesAction);
+    treeViewResults->header()->setContextMenuPolicy(Qt::ActionsContextMenu);
+    QObject::connect(lockSizesAction, SIGNAL(toggled(bool)),
+                     this, SLOT(lockSizesChanged(bool)));
+
+    // Add another context menu for the results
+    m_customPopup = new CustomPopup(treeViewResults);
+    QObject::connect(m_customPopup, SIGNAL(popupCreated(QMenu&, const QPoint&)),
+                     this, SLOT(populateMenu(QMenu&, const QPoint&)));
+    QObject::connect(m_customPopup, SIGNAL(requestDefinition(QString)),
+                     this, SIGNAL(requestDefinition(QString)));
+
+    // Allow very thin columns
+    treeViewResults->header()->setMinimumSectionSize(1);
 
     // Enable the Play button only when there is a selection in the tree
     QObject::connect(treeViewResults->selectionModel(),
@@ -90,7 +125,10 @@ TrainingWidget::TrainingWidget(QWidget *parent, CoordModel &iCoordModel, PublicG
                      SLOT(showPreview(const QItemSelection&, const QItemSelection&)));
 
     if (m_game)
-        lineEditRack->setValidator(new RackValidator(this, &m_game->getBag()));
+    {
+        QValidator * val = ValidatorFactory::newRackValidator(this, m_game->getBag());
+        lineEditRack->setValidator(val);
+    }
 
     // Notify that the rack changed
     QObject::connect(lineEditRack, SIGNAL(textChanged(const QString&)),
@@ -141,12 +179,20 @@ void TrainingWidget::updateModel()
         return;
     }
 
+    // Clear the results
     m_model->removeRows(0, m_model->rowCount());
 
+    // Force the sort column
+    treeViewResults->sortByColumn(2);
+
     if (m_game == NULL)
         return;
 
     const Results &results = m_game->trainingGetResults();
+    // Find the highest score
+    int bestScore = -1;
+    if (results.size() != 0)
+        bestScore = results.get(0).getPoints();
     for (unsigned int i = 0; i < results.size(); ++i)
     {
         const Round &r = results.get(i);
@@ -158,18 +204,45 @@ void TrainingWidget::updateModel()
         m_model->setData(m_model->index(rowNum, 2), r.getPoints());
         m_model->setData(m_model->index(rowNum, 3),
                          r.getBonus() ? "*": "");
+        if (r.getPoints() == bestScore)
+        {
+            // Color the line in red if this is the top score
+            const QBrush redBrush(Qt::red);
+            for (int j = 0; j < HIDDEN_COLUMN; ++j)
+            {
+                m_model->setData(m_model->index(rowNum, j),
+                                 redBrush, Qt::ForegroundRole);
+            }
+        }
+        else
+        {
+            // Otherwise indicate the difference with the best score
+            m_model->setData(m_model->index(rowNum, 4),
+                             r.getPoints() - bestScore);
+        }
+
         // Hidden data, used to handle proper sorting in the tree view
-        m_model->setData(m_model->index(rowNum, 5), i);
+        m_model->setData(m_model->index(rowNum, HIDDEN_COLUMN), i);
     }
 
     // Clear the status bar when there is no search result
     if (m_model->rowCount() == 0)
         emit notifyInfo("");
+    else
+    {
+        // Otherwise, select the first line
+        treeViewResults->selectionModel()->select(m_model->index(0, 0),
+                                                  QItemSelectionModel::SelectCurrent | QItemSelectionModel::Rows);
+    }
 
-    treeViewResults->resizeColumnToContents(0);
-    treeViewResults->resizeColumnToContents(1);
-    treeViewResults->resizeColumnToContents(2);
-    treeViewResults->resizeColumnToContents(3);
+    if (m_autoResizeColumns)
+    {
+        treeViewResults->resizeColumnToContents(0);
+        treeViewResults->resizeColumnToContents(1);
+        treeViewResults->resizeColumnToContents(2);
+        treeViewResults->resizeColumnToContents(3);
+        treeViewResults->resizeColumnToContents(4);
+    }
 }
 
 
@@ -185,22 +258,45 @@ void TrainingWidget::enablePlayButton(const QItemSelection &iSelected,
 void TrainingWidget::showPreview(const QItemSelection &iSelected,
                                  const QItemSelection &)
 {
-    m_game->trainingRemoveTestPlay();
+    m_game->removeTestRound();
     if (!iSelected.indexes().empty())
     {
         // Use the hidden column to get the result number
         const QModelIndex &index =
-            m_model->index(iSelected.indexes().first().row(), 5);
-        m_game->trainingTestPlay(m_model->data(index).toUInt());
+            m_model->index(iSelected.indexes().first().row(), HIDDEN_COLUMN);
+        unsigned int resNb = m_model->data(index).toUInt();
+
+        const Results &results = m_game->trainingGetResults();
+        ASSERT(resNb < results.size(), "Wrong result number");
+        m_game->setTestRound(results.get(resNb));
         emit gameUpdated();
     }
 }
 
 
-void TrainingWidget::on_lineEditRack_textEdited(const QString &iText)
+void TrainingWidget::populateMenu(QMenu &iMenu, const QPoint &iPoint)
 {
-    // FIXME: first parameter is hardcoded
-    m_game->trainingRemoveTestPlay();
+    const QModelIndex &index = treeViewResults->indexAt(iPoint);
+    if (!index.isValid())
+        return;
+
+    // Find the selected word
+    const QModelIndex &wordIndex = m_model->index(index.row(), 0);
+    QString selectedWord = m_model->data(wordIndex).toString();
+
+    iMenu.addAction(m_customPopup->getShowDefinitionEntry(selectedWord));
+}
+
+
+void TrainingWidget::lockSizesChanged(bool checked)
+{
+    m_autoResizeColumns = !checked;
+}
+
+
+void TrainingWidget::onRackEdited(const QString &iText)
+{
+    m_game->removeTestRound();
     if (!lineEditRack->hasAcceptableInput())
     {
         lineEditRack->setPalette(redPalette);
@@ -209,7 +305,7 @@ void TrainingWidget::on_lineEditRack_textEdited(const QString &iText)
     try
     {
         lineEditRack->setPalette(blackPalette);
-        const wstring &input = m_game->getDic().convertFromInput(qtw(iText));
+        const wstring &input = m_game->getDic().convertFromInput(wfq(iText));
         m_game->trainingSetRackManual(false, input);
         pushButtonSearch->setEnabled(m_model->rowCount() == 0 &&
                                      lineEditRack->text() != "");
@@ -218,19 +314,33 @@ void TrainingWidget::on_lineEditRack_textEdited(const QString &iText)
     catch (std::exception &e)
     {
         lineEditRack->setPalette(redPalette);
-        emit notifyProblem(_q("Warning: Cannot set the rack to '%1'").arg(iText));
+        emit notifyProblem(_q("Warning: Cannot set the rack to '%1'\n%2").arg(iText).arg(e.what()));
     }
 }
 
 
-void TrainingWidget::on_pushButtonRack_clicked()
+void TrainingWidget::setNewRack()
 {
-    m_game->trainingRemoveTestPlay();
+    helperSetRack(true);
+}
+
+
+void TrainingWidget::completeRack()
+{
+    helperSetRack(false);
+}
+
+
+void TrainingWidget::helperSetRack(bool iAll)
+{
+    m_game->removeTestRound();
     try
     {
         // FIXME: first parameter is hardcoded
-        m_game->trainingSetRackRandom(true, PublicGame::kRACK_ALL);
+        m_game->trainingSetRackRandom(true,
+                iAll ? PublicGame::kRACK_NEW : PublicGame::kRACK_NEW);
         emit gameUpdated();
+        lineEditRack->setFocus();
     }
     catch (std::exception &e)
     {
@@ -239,51 +349,33 @@ void TrainingWidget::on_pushButtonRack_clicked()
 }
 
 
-void TrainingWidget::on_pushButtonComplement_clicked()
+void TrainingWidget::search()
 {
-    m_game->trainingRemoveTestPlay();
-    try
-    {
-        // FIXME: first parameter is hardcoded
-        m_game->trainingSetRackRandom(true, PublicGame::kRACK_NEW);
-        emit gameUpdated();
-    }
-    catch (std::exception &e)
-    {
-        emit notifyProblem(_q(e.what()));
-    }
-}
-
-
-void TrainingWidget::on_pushButtonSearch_clicked()
-{
-    m_game->trainingRemoveTestPlay();
+    m_game->removeTestRound();
     emit notifyInfo(_q("Searching with rack '%1'...").arg(lineEditRack->text()));
     m_game->trainingSearch();
     emit notifyInfo(_q("Search done"));
     emit gameUpdated();
+    treeViewResults->setFocus();
 }
 
 
-void TrainingWidget::on_pushButtonPlaySelected_clicked()
+void TrainingWidget::playSelectedWord()
 {
     QModelIndexList indexList = treeViewResults->selectionModel()->selectedIndexes();
     if (indexList.empty())
         return;
-    // Forward the work to another slot
-    on_treeViewResults_doubleClicked(indexList.front());
-}
 
-
-void TrainingWidget::on_treeViewResults_doubleClicked(const QModelIndex &iIndex)
-{
-    if (!iIndex.isValid())
+    const QModelIndex &selIndex = indexList.front();
+    if (!selIndex.isValid())
         return;
-    m_game->trainingRemoveTestPlay();
+
+    m_game->removeTestRound();
     // Use the hidden column to get the result number
-    const QModelIndex &index = m_model->index(iIndex.row(), 5);
+    const QModelIndex &index = m_model->index(selIndex.row(), HIDDEN_COLUMN);
     m_game->trainingPlayResult(m_model->data(index).toUInt());
     emit gameUpdated();
+    lineEditRack->setFocus();
 }
 
 
@@ -292,48 +384,3 @@ QSize TrainingWidget::sizeHint() const
     return QSize(160, 300);
 }
 
-
-
-RackValidator::RackValidator(QObject *parent, const Bag *iBag)
-    : QValidator(parent), m_bag(iBag)
-{
-}
-
-
-QValidator::State RackValidator::validate(QString &input, int &) const
-{
-    // This should never happen, since the control should be disabled in
-    // such a case, but checking doesn't hurt...
-    if (m_bag == NULL)
-        return Invalid;
-
-    input = input.toUpper();
-
-    const Dictionary &dic = m_bag->getDic();
-
-    // The string is invalid if it contains invalid input characters
-    const wistring &winput = qtw(input);
-    if (!dic.validateInputChars(winput))
-        return Invalid;
-
-    // Convert the string to internal letters
-    const wstring &intInput = dic.convertFromInput(winput);
-    // The string is invalid if it contains characters not present
-    // in the dictionary
-    if (!dic.validateLetters(intInput))
-        return Intermediate;
-
-    QString qinput = qfw(intInput);
-    // The letters must be in the bag
-    for (int i = 0; i < qinput.size(); ++i)
-    {
-        if ((unsigned int)qinput.count(qinput[i], Qt::CaseInsensitive) >
-            m_bag->in(intInput[i]))
-        {
-            return Invalid;
-        }
-    }
-    return Acceptable;
-}
-
-
