@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008-2011 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 
 #include <QtGui/QWidget>
 #include "ui/training_widget.ui.h"
+#include "logging.h"
 
 
 class QStandardItemModel;
@@ -36,6 +37,7 @@ class PublicGame;
 class TrainingWidget: public QWidget, private Ui::TrainingWidget
 {
     Q_OBJECT;
+    DEFINE_LOGGER();
 
 public:
     explicit TrainingWidget(QWidget *parent, CoordModel &iCoordModel, PublicGame *iGame);
@@ -61,19 +63,17 @@ private slots:
     void lockSizesChanged(bool checked);
     void populateMenu(QMenu &iMenu, const QPoint &iPoint);
 
-    // These slots are automatically connected
-    void on_lineEditRack_textEdited(const QString &iText);
-    void on_pushButtonRack_clicked();
-    void on_pushButtonComplement_clicked();
-    void on_pushButtonSearch_clicked();
-    void on_pushButtonPlaySelected_clicked();
-    void on_treeViewResults_doubleClicked(const QModelIndex &iIndex);
+    void onRackEdited(const QString &iText);
+    void setNewRack();
+    void completeRack();
+    void search();
+    void playSelectedWord();
 
 private:
     /// Encapsulated training game, can be NULL
     PublicGame *m_game;
 
-    /// Inidicate whether the columns should be resized after a search
+    /// Indicate whether the columns should be resized after a search
     bool m_autoResizeColumns;
 
     /// Model of the search results
@@ -82,17 +82,20 @@ private:
     /// Mediator for the "play word" controls
     PlayWordMediator *m_mediator;
 
-    /// Palette to write text in red
-    QPalette redPalette;
-
     /// Palette to write text in black
     QPalette blackPalette;
 
+    /// Palette to write text in red
+    QPalette redPalette;
+
     /// Popup menu for words definition
     CustomPopup *m_customPopup;
 
     /// Force synchronizing the model with the contents of the search results
     void updateModel();
+
+    /// Helper method to set the rack
+    void helperSetRack(bool iAll);
 };
 
 #endif
