@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (C) 1999-2005 Eliot
+ * Copyright (C) 1999-2012 Eliot
  * Authors: Antoine Fraboulet <antoine.fraboulet@free.fr>
  *          Olivier Teuliere  <ipkiss@via.ecp.fr>
  *
@@ -35,7 +35,7 @@
 #include "player.h"
 #include "encoding.h"
 #include "history.h"
-#include "turn.h"
+#include "turn_data.h"
 #include "move.h"
 #include "round.h"
 
@@ -45,6 +45,9 @@ using boost::format;
 using boost::wformat;
 
 
+INIT_LOGGER(utils, GameIO);
+
+
 #define __UNUSED__ __attribute__((unused))
 
 void GameIO::printBoard(ostream &out, const PublicGame &iGame)
@@ -69,6 +72,7 @@ void GameIO::printBoard(ostream &out, const PublicGame &iGame)
     }
 }
 
+
 /* this mode is used for regression tests */
 void GameIO::printBoardDebug(ostream &out, const PublicGame &iGame)
 {
@@ -96,33 +100,6 @@ void GameIO::printBoardDebug(ostream &out, const PublicGame &iGame)
     }
 }
 
-void GameIO::printBoardJoker(ostream &out, const PublicGame &iGame)
-{
-    int row,col;
-
-    out << "   ";
-    for (col = BOARD_MIN; col <= BOARD_MAX; col++)
-        out << setw(3) << col - BOARD_MIN + 1;
-    out << endl;
-
-    for (row = BOARD_MIN; row <= BOARD_MAX; row++)
-    {
-        out << " " << (char)(row - BOARD_MIN + 'A') << " ";
-        for (col = BOARD_MIN; col <= BOARD_MAX; col++)
-        {
-            if (iGame.getBoard().isVacant(row, col))
-                out << " --";
-            else
-            {
-                bool j = iGame.getBoard().isJoker(row, col);
-                out << (j ? " ." : "  ")
-                    << lfw(iGame.getBoard().getDisplayStr(row, col));
-            }
-        }
-        out << endl;
-    }
-}
-
 
 void GameIO::printBoardMultipliers(ostream &out, const PublicGame &iGame)
 {
@@ -158,39 +135,6 @@ void GameIO::printBoardMultipliers(ostream &out, const PublicGame &iGame)
 }
 
 
-void GameIO::printBoardMultipliers2(ostream &out, const PublicGame &iGame)
-{
-    int row, col;
-
-    out << "   ";
-    for (col = BOARD_MIN; col <= BOARD_MAX; col++)
-        out << setw(3) << col - BOARD_MIN + 1;
-    out << endl;
-
-    for (row = BOARD_MIN; row <= BOARD_MAX; row++)
-    {
-        out << " " << (char)(row - BOARD_MIN + 'A') << " ";
-        for (col = BOARD_MIN; col <= BOARD_MAX; col++)
-        {
-            int wm = iGame.getBoard().GetWordMultiplier(row, col);
-            int tm = iGame.getBoard().GetLetterMultiplier(row, col);
-
-            if (wm > 1)
-                out << " " << ((wm == 3) ? '@' : '#');
-            else if (tm > 1)
-                out << " " << ((tm == 3) ? '*' : '+');
-            else
-                out << " -";
-            if (iGame.getBoard().isVacant(row, col))
-                out << "-";
-            else
-                out << lfw(iGame.getBoard().getDisplayStr(row, col));
-        }
-        out << endl;
-    }
-}
-
-
 void GameIO::printNonPlayed(ostream &out, const PublicGame &iGame)
 {
     const Bag &bag = iGame.getBag();
@@ -212,7 +156,7 @@ void GameIO::printNonPlayed(ostream &out, const PublicGame &iGame)
 
 void GameIO::printPlayedRack(ostream &out, const PublicGame &iGame, int __UNUSED__ n)
 {
-    out << lfw(iGame.getCurrentPlayer().getCurrentRack().toString(PlayedRack::RACK_SIMPLE)) << endl;
+    out << lfw(iGame.getCurrentRack().toString(PlayedRack::RACK_SIMPLE)) << endl;
 }
 
 
@@ -252,7 +196,7 @@ void GameIO::printSearchResults(ostream &out, const Results &iResults, int num)
 
 void GameIO::printPoints(ostream &out, const PublicGame &iGame)
 {
-    out << iGame.getPlayer(0).getPoints() << endl;
+    out << iGame.getPlayer(0).getTotalScore() << endl;
 }
 
 
@@ -261,7 +205,7 @@ void GameIO::printAllPoints(ostream &out, const PublicGame &iGame)
     for (unsigned int i = 0; i < iGame.getNbPlayers(); i++)
     {
         out << "Score " << i << ": "
-            << setw(4) << iGame.getPlayer(i).getPoints() << endl;
+            << setw(4) << iGame.getPlayer(i).getTotalScore() << endl;
     }
 }
 
@@ -276,6 +220,8 @@ void GameIO::printGameDebug(ostream &out, const PublicGame &iGame)
         out << "Game: mode=Free game" << endl;
     else if (iGame.getParams().getMode() == GameParams::kTRAINING)
         out << "Game: mode=Training" << endl;
+    else if (iGame.getParams().getMode() == GameParams::kARBITRATION)
+        out << "Game: mode=Arbitration" << endl;
     if (iGame.getParams().hasVariant(GameParams::kJOKER))
         out << "Game: variant=joker" << endl;
     if (iGame.getParams().hasVariant(GameParams::kEXPLOSIVE))
@@ -287,13 +233,13 @@ void GameIO::printGameDebug(ostream &out, const PublicGame &iGame)
     out << "   ===|===|==========|================|=====|=====|======" << endl;
     for (unsigned int i = 0; i < iGame.getHistory().getSize(); ++i)
     {
-        const Turn &turn = iGame.getHistory().getTurn(i);
+        const TurnData &turn = iGame.getHistory().getTurn(i);
         const Move &move = turn.getMove();
         format fmter("%1% | %2% | %3% | %4% | %5% | %6% | %7%");
         fmter % padAndConvert(str(wformat(L"%1%") % (i + 1)), 5);
         fmter % padAndConvert(str(wformat(L"%1%") % turn.getPlayer()), 1);
         fmter % padAndConvert(turn.getPlayedRack().toString(), 8);
-        if (move.getType() == Move::VALID_ROUND)
+        if (move.isValid())
         {
             const Round &round = move.getRound();
             fmter % padAndConvert(round.getWord(), 14, false);
@@ -303,19 +249,23 @@ void GameIO::printGameDebug(ostream &out, const PublicGame &iGame)
         }
         else
         {
-            if (move.getType() == Move::INVALID_WORD)
+            if (move.isInvalid())
             {
                 fmter % padAndConvert(L"#" + move.getBadWord() + L"#", 14, false);
                 fmter % padAndConvert(move.getBadCoord(), 3);
             }
-            else if (move.getType() == Move::CHANGE_LETTERS)
+            else if (move.isChangeLetters())
             {
                 fmter % padAndConvert(L"[" + move.getChangedLetters() + L"]", 14, false) % " - ";
             }
-            else
+            else if (move.isPass())
             {
                 fmter % padAndConvert(L"(PASS)", 14, false) % " - ";
             }
+            else
+            {
+                fmter % padAndConvert(L"(NO MOVE)", 14, false) % " - ";
+            }
             fmter % "  0" % " ";
         }
         out << fmter.str() << endl;
