@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (C) 2005-2008 Eliot
+ * Copyright (C) 2005-2012 Eliot
  * Authors: Olivier Teuliere  <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #   include <windows.h>
 #endif
 
+#include <iostream>
 #include <ctype.h>
 #include <cstring> // For strlen
 #include <cwctype> // For iswalnum
@@ -46,13 +47,17 @@
 #include "results.h"
 #include "player.h"
 #include "history.h"
-#include "turn.h"
+#include "turn_data.h"
 #include "game_exception.h"
 #include "encoding.h"
 
 using namespace std;
 
 
+INIT_LOGGER(utils, Box);
+INIT_LOGGER(utils, CursesIntf);
+
+
 Box::Box(WINDOW *win, int y, int x, int h, int w,
          unsigned int iHeadingLines)
     : m_win(win),  m_x(x), m_y(y), m_w(w), m_h(h),
@@ -302,7 +307,7 @@ void CursesIntf::drawScoresRacks(WINDOW *win, int y, int x) const
         mvwprintw(win, y + i + 1, x + 2, _("%s: %d"),
                   truncOrPad(lfw(m_game->getPlayer(i).getName()),
                              maxForScores).c_str(),
-                  m_game->getPlayer(i).getPoints());
+                  m_game->getPlayer(i).getTotalScore());
         if (m_game->getMode() != PublicGame::kTRAINING && i == currId)
             attroff(A_BOLD);
     }
@@ -391,9 +396,9 @@ void CursesIntf::drawHistory(Box &ioBox) const
     for (i = ioBox.getFirstLine();
          i < (int)m_game->getHistory().getSize() && i < ioBox.getLastLine(); i++)
     {
-        const Turn& t = m_game->getHistory().getTurn(i);
+        const TurnData& t = m_game->getHistory().getTurn(i);
         const Move& m = t.getMove();
-        if (m.getType() == Move::VALID_ROUND)
+        if (m.isValid())
         {
             // The move corresponds to a played round: display it
             const Round &r = m.getRound();
@@ -405,7 +410,7 @@ void CursesIntf::drawHistory(Box &ioBox) const
                 padAndConvert(coord, 3).c_str(), r.getPoints(),
                 t.getPlayer(), r.getBonus() ? '*' : ' ');
         }
-        else if (m.getType() == Move::INVALID_WORD)
+        else if (m.isInvalid())
         {
             // The move corresponds to an invalid word: display it
             wstring invWord = L"<" + m.getBadWord() + L">";
@@ -420,9 +425,11 @@ void CursesIntf::drawHistory(Box &ioBox) const
         {
             // The move corresponds to a passed turn or changed letters
             wstring action;
-            if (m.getType() == Move::PASS)
+            if (m.isNull())
+                action = wfl(_("(NO MOVE)"));
+            else if (m.isPass())
                 action = wfl(_("(PASS)"));
-            else if (m.getType() == Move::CHANGE_LETTERS)
+            else if (m.isChangeLetters())
                 action = L"(-" + m.getChangedLetters() + L")";
 
             ioBox.printDataLine(i, x,
@@ -1120,9 +1127,7 @@ void CursesIntf::redraw(WINDOW *win)
         variant = string(" - ") + _("Joker game");
     string title = "Eliot (" + mode + variant + ") " + _("[h for help]");
 
-    int lines;
-    int cols;
-    getmaxyx(m_win, lines, cols);
+    int cols = getmaxx(m_win);
     mvwprintw(win, 0, 0, truncOrPad(title, cols).c_str());
     attroff(A_REVERSE);
 
@@ -1156,64 +1161,91 @@ int main(int argc, char ** argv)
 
     srand(time(NULL));
 
-    Game *realGame = GameFactory::Instance()->createFromCmdLine(argc, argv);
-    if (realGame == NULL)
+    int retCode = 1;
+    try
     {
-        GameFactory::Destroy();
-        return 1;
-    }
-    PublicGame *game = new PublicGame(*realGame);
-
-    game->start();
-
-    // Initialize the ncurses library
-    WINDOW *wBoard = initscr();
-    keypad(wBoard, true);
-    // Take input chars one at a time
-    cbreak();
-    // Do not do NL -> NL/CR
-    nonl();
-    // Hide the cursor
-    curs_set(0);
-
-    if (has_colors())
-    {
-        start_color();
-
-        // Simple color assignment
-        init_pair(COLOR_BLACK, COLOR_BLACK, COLOR_BLACK);
-        init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
-        init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
-        init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_RED);
-
-        init_pair(COLOR_BLUE, COLOR_BLACK, COLOR_BLUE);
-        init_pair(COLOR_CYAN, COLOR_BLACK, COLOR_CYAN);
-        init_pair(COLOR_MAGENTA, COLOR_BLACK, COLOR_MAGENTA);
-        init_pair(COLOR_RED, COLOR_BLACK, COLOR_RED);
-    }
-
-    // Do not echo
-    noecho();
-
-    // mainIntf will take care of destroying game for us
-    CursesIntf mainIntf(wBoard, *game);
-    mainIntf.redraw(wBoard);
-
-    while (!mainIntf.isDying())
-    {
-        int c = getch();
-        if (mainIntf.handleKey(c) == 1)
+        Game *realGame = GameFactory::Instance()->createFromCmdLine(argc, argv);
+        if (realGame == NULL)
         {
-            mainIntf.redraw(wBoard);
+            GameFactory::Destroy();
+            return 1;
         }
+        PublicGame *game = new PublicGame(*realGame);
+
+        game->start();
+
+        // Initialize the ncurses library
+        WINDOW *wBoard = initscr();
+        try
+        {
+            keypad(wBoard, true);
+            // Take input chars one at a time
+            cbreak();
+            // Do not do NL -> NL/CR
+            nonl();
+            // Hide the cursor
+            curs_set(0);
+
+            if (has_colors())
+            {
+                start_color();
+
+                // Simple color assignment
+                init_pair(COLOR_BLACK, COLOR_BLACK, COLOR_BLACK);
+                init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
+                init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
+                init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_RED);
+
+                init_pair(COLOR_BLUE, COLOR_BLACK, COLOR_BLUE);
+                init_pair(COLOR_CYAN, COLOR_BLACK, COLOR_CYAN);
+                init_pair(COLOR_MAGENTA, COLOR_BLACK, COLOR_MAGENTA);
+                init_pair(COLOR_RED, COLOR_BLACK, COLOR_RED);
+            }
+
+            // Do not echo
+            noecho();
+
+            // mainIntf will take care of destroying game for us
+            CursesIntf mainIntf(wBoard, *game);
+            mainIntf.redraw(wBoard);
+
+            while (!mainIntf.isDying())
+            {
+                int c = getch();
+                if (mainIntf.handleKey(c) == 1)
+                {
+                    mainIntf.redraw(wBoard);
+                }
+            }
+        }
+        catch (...)
+        {
+            // Clean up
+            delwin(wBoard);
+
+            // Exit the ncurses library
+            endwin();
+
+            // Rethrow the exception
+            throw;
+        }
+
+        retCode = 0;
+    }
+    catch (const BaseException &e)
+    {
+        cerr << "Exception caught: " << e.what() << "\n" << e.getStackTrace();
+    }
+    catch (const std::exception &e)
+    {
+        cerr << "Exception caught: " << e.what();
+    }
+    catch (...)
+    {
+        cerr << "Unknown exception caught";
     }
-
-    delwin(wBoard);
-
-    // Exit the ncurses library
-    endwin();
 
     GameFactory::Destroy();
 
-    return 0;
+    return retCode;
 }
