@@ -1,6 +1,7 @@
 /*****************************************************************************
- * Copyright (C) 2005 Eliot
- * Authors: Olivier Teuliere  <ipkiss@via.ecp.fr>
+ * Eliot
+ * Copyright (C) 2005-2012 Olivier Teulière
+ * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +24,8 @@
 #include <ncursesw/curses.h>
 #include <string>
 
+#include "logging.h"
+
 class PublicGame;
 
 using std::string;
@@ -31,6 +34,8 @@ using std::wstring;
 
 class Box
 {
+    DEFINE_LOGGER();
+
     public:
         // Create a titled box with the specified position and size,
         // containing iHeadingLines non-scrolling lines.
@@ -97,6 +102,7 @@ class Box
  */
 class CursesIntf
 {
+    DEFINE_LOGGER();
 public:
     // Pre-requisite: the given Game object MUST have been allocated with new
     // (in particular: not on the stack)
